#include "commute.h"

#include "pegasus.h"
#include "alphas.h"
#include "scales.h"
#include "resint.h"
#include "resconst.h"
#include "settings.h"
#include "mesq.h"
#include "pdfevol.h"
#include "evoltrunc.h"
#include "parton.h"

#include <iostream>

using namespace parton;

/*
  pdf(b) = U(b,F)*pdf(F)
    = U(b,Q)*U(Q,F)*pdf(F)
    = (U(Q,F)*U(b,Q)-[U(b,Q),U(Q,F)])*pdf(F)
    = (U(Q,F)*U(b,F)*U(F,Q)-[U(b,Q),U(Q,F)])*pdf(F)
    = (U(Q,F)*(U(F,Q)*U(b,F) - [U(b,F),U(F,Q)])-[U(b,Q),U(Q,F)])*pdf(F)
    = (U(Q,F)*U(F,Q)*U(b,F) - U(Q,F)*[U(b,F),U(F,Q)] - [U(b,Q),U(Q,F)])*pdf(F)
    = (U(Q,F)*U(F,Q)*U(b,F) - U(Q,F)*U(b,F)*U(F,Q) + U(Q,F)*U(F,Q)*U(b,F) - U(b,Q)*U(Q,F) + U(Q,F)*U(b,Q))*pdf(F)
    = (U(Q,F)*U(F,Q)*U(b,F) - U(Q,F)*U(b,Q) + U(b,F) - U(b,F) + U(Q,F)*U(b,Q))*pdf(F)
    = U(Q,F)*U(F,Q)*U(b,F)*pdf(F)
    This module compute the operator U(F,Q) for the evolution from Q to muF
*/

//commute::allocate()
//{
//}

void commute::evolve()
{
  //Evolve from Q to muF
  if (!(opts.evolmode == 2 || opts.evolmode == 3 || opts.commute))
    return;
  if (opts.kmufac == opts.kmures)
    return;

  if (opts.evolmode == 2 || opts.evolmode == 3 || opts.evolmode == 4 || opts.evolmode == 5)
    {
      evoltrunc::commute();
      return;
    }
  //else if (opts.evolmode == 0 || opts.evolmode == 1)
  
  int NF = resconst::NF;
  /*
  //Fixed flavour evolution from Q to muF
  ASI = resint::alpqres;
  ASF = resint::alpqfac;
  //order_.npord_ = pdf::order; //order of evolution read from LHAPDF
  //evmod_.imodev_ = 1; //reproduces evolution in x-space
  order_.npord_ = opts.order_evol-1;
  evmod_.imodev_ = 4;
  */
  //cout << "alpqres " << resint::alpqres << endl;
  //cout << "alpqfac " << resint::alpqfac << endl;
  //cout << " ratio " << resint::alpqfac/resint::alpqres << endl;
      
  //Fixed flavour evolution from Q to muF
  //order_.npord_ = max(2,opts.order_evol-1);
  order_.npord_ = opts.order_evol-1;

  //should this piece be done in MSBAR or in CMW? --> the gamma anomalous dimension in the LQF*gamma terms in the H coefficients are in msbar, but the C coefficients and the beta function are in CMW
  double beta0 = alphas::beta0_msbar;
  double beta1 = alphas::beta1_msbar;
  double beta2 = alphas::beta2_msbar;
  double beta3 = alphas::beta3_msbar;
      
  //evmod_.imodev_ = 1; //iterative evolution
  //evmod_.imodev_ = 4; //truncated evolution
  if (opts.iterative)
    evmod_.imodev_ = 1; //iterative evolution
  else
    evmod_.imodev_ = 4; //truncated evolution
  double blog = log(pow(scales::res/scales::fac,2));
  double as = resint::aass;
  double LQR = resint::LR-resint::LQ;
  double as2 = pow(as,2);
  double as3 = pow(as,3);
  double xlambda = beta0*as*blog;
  double log1xlambda = log(1.-xlambda);
  double logas = 0.;

  if (opts.order_evol >= 1)
    logas += log1xlambda;
  if (opts.order_evol >= 2)
    logas += as* beta1/beta0*log1xlambda/(1.-xlambda);
  if (opts.order_evol >= 3)
    logas += as2* ((pow(beta1/beta0,2)-beta2/beta0) *xlambda/pow(1.-xlambda,2)
		   + pow(beta1/beta0,2)             *log1xlambda/pow(1.-xlambda,2)
		   - pow(beta1/beta0,2)             *pow(log1xlambda,2)/(2.*pow(1.-xlambda,2)));
  if (opts.order_evol >= 4)
    logas += as3*
      (3.*xlambda*(pow(beta0,2)*beta3*(-2. + xlambda)
		   - 2.*beta0*beta1*beta2*(-1. + xlambda)
		   + pow(beta1,3)*xlambda)
       + 6.*(-(pow(beta1,3)*xlambda)
	     + beta0*beta1*beta2*(1. + xlambda))*log1xlambda
       - 9.*pow(beta1,3)*pow(log1xlambda,2)
       + 2.*pow(beta1,3)*pow(log1xlambda,3)
       )/(6.*pow(beta0,3)*pow(1. - xlambda,3));
  if (opts.order_evol >= 2)
    logas += as*(LQR)
      *beta0*xlambda/(1.-xlambda);
  if (opts.order_evol >= 3)
    logas += as2*(+LQR*beta1                   *(xlambda-log1xlambda)/pow(1.-xlambda,2)
		  +LQR*beta1                   *xlambda/(1.-xlambda)                      //missing piece
		  +0.5*pow(LQR,2)*pow(beta0,2) *xlambda*(xlambda-2.)/pow(1.-xlambda,2));  //missing piece
  if (opts.order_evol >= 4)
    logas += as3*
      (LQR*(6.*pow(beta1,2)*pow(log1xlambda,2)
	    + 6.*beta1*log1xlambda*(-3.*beta1 + pow(beta0,2)*LQR)
	    + xlambda*(-6.*pow(beta1,2) + 6.*beta0*beta2*(4. - 3.*xlambda + pow(xlambda,2))
		       - 9.*pow(beta0,2)*beta1*(3. - 3.*xlambda + pow(xlambda,2))*LQR
		       + 2.*pow(beta0,4)*(3. - 3.*xlambda + pow(xlambda,2))*pow(LQR,2))))/(6.*beta0*pow(1. - xlambda,3));
  fcomplex ASI = fcx(resint::alpqres);
  fcomplex ASF = fcx(exp(-logas)*resint::alpqres);

  //cout << resint::alpqres << "  " << exp(-logas)*resint::alpqres << endl;
  //double FR2 = pow(scales::ren/scales::fac,2);//ratio of muren2/mufac2
  //frrat_.logfr_ = log(FR2);

  //Need to be sure that all pegasus moments were allocates and updated properly
  for (int sign = mesq::positive; sign <= mesq::negative; sign++)
    for (int n = 0; n < pegasus::dim; n++)
      {

	int nn;
	if (opts.mellin1d)
	  nn = n;
	else
	  if (n < mellinint::mdim)
	    nn = n;
	  else
	    nn = n-mellinint::mdim;
	
	if (opts.mellin1d)
	  if (sign == mesq::positive)
	    pdfevol::retrieve1d_pos(nn);
	  else
	    pdfevol::retrieve1d_pos(nn+mellinint::mdim);
	else
	  if (n < mellinint::mdim)
	    if (sign == mesq::positive)
	      pdfevol::retrieve_beam1(nn);
	    else
	      pdfevol::retrieve_beam1(nn+mellinint::mdim);
	  else
	    if (sign == mesq::positive)
	      pdfevol::retrieve_beam2_pos(nn);
	    else
	      pdfevol::retrieve_beam2_pos(nn+mellinint::mdim);

	int i = pegasus::index(n,sign);

	complex<double>* fn;
	if (n < mellinint::mdim)
	  fn = pdfevol::fn1;
	else
	  fn = pdfevol::fn2;
	
	//gluon
	painp_.gli_[i] = fcx(fn[g]);

	//cout << endl;
	//cout << fn[u] << endl;
	//cout << fn[d] << endl;
	//cout << fn[s] << endl;
	//cout << fn[c] << endl;
	//cout << fn[b] << endl;
	//cout << fn[g] << endl;
	//cout << fn[ub] << endl;
	//cout << fn[db] << endl;
	//cout << fn[sb] << endl;
	//cout << fn[cb] << endl;
	//cout << fn[bb] << endl;

	complex <double> qp[5];
	complex <double> qm[5];

	if (n < mellinint::mdim || opts.ih2 == 1)
	  {
	    qp[0] = fn[u] + fn[ub];
	    qp[1] = fn[d] + fn[db];
	    qp[2] = fn[s] + fn[sb];
	    qp[3] = fn[c] + fn[cb];
	    qp[4] = fn[b] + fn[bb];

	    qm[0] = fn[u] -  fn[ub];
	    qm[1] = fn[d] -  fn[db];
	    qm[2] = fn[s] -  fn[sb];
	    qm[3] = fn[c] -  fn[cb];
	    qm[4] = fn[b] -  fn[bb];
	  }
	else //flip back pbar -> p
	  {
	    qp[0] = fn[ub] + fn[u];
	    qp[1] = fn[db] + fn[d];
	    qp[2] = fn[sb] + fn[s];
	    qp[3] = fn[cb] + fn[c];
	    qp[4] = fn[bb] + fn[b];

	    qm[0] = fn[ub] -  fn[u];
	    qm[1] = fn[db] -  fn[d];
	    qm[2] = fn[sb] -  fn[s];
	    qm[3] = fn[cb] -  fn[c];
	    qm[4] = fn[bb] -  fn[b];
	  }

	painp_.vai_[i] = fcx(qm[0]+qm[1]+qm[2]+qm[3]+qm[4]);
	painp_.m3i_[i] = fcx(qm[0]-qm[1]);
	painp_.m8i_[i] = fcx(qm[0]+qm[1]-2.*qm[2]);
	hfpainp_.m15i_[i] = fcx(qm[0]+qm[1]+qm[2]-3.*qm[3]);
	hfpainp_.m24i_[i] = fcx(qm[0]+qm[1]+qm[2]+qm[3]-4.*qm[4]);
  
	painp_.sgi_[i] = fcx(qp[0]+qp[1]+qp[2]+qp[3]+qp[4]);
	painp_.p3i_[i] = fcx(qp[0]-qp[1]);
	painp_.p8i_[i] = fcx(qp[0]+qp[1]-2.*qp[2]);
	hfpainp_.p15i_[i] = fcx(qp[0]+qp[1]+qp[2]-3.*qp[3]);
	hfpainp_.p24i_[i] = fcx(qp[0]+qp[1]+qp[2]+qp[3]-4.*qp[4]);
      }

  //cout << endl;
  //cout << endl;
  //for (int i = 0; i < mellinint::mdim; i++)
  //	for (int p = -MAXNF; p <= MAXNF; p++)
  //	  cout << i << "  " << p << "  " << cx(PDFN[p+6][i]) << endl;
  //      fcomplex PDFN_evQF[13][ndim];

  int nlow = 1;
  int nhigh = 2*pegasus::dim;
  int IPSTD = 1; //IPSTD = 1 return PDFN as valence sea, IPSTD = = return PDFN as q, qb (as in LHAPDF but with u and d inverted)
  fcomplex PDFN[13][ndim];
  dyevnffn_(PDFN, ASI, ASF, NF, nlow, nhigh, IPSTD); //modified ffn evolution with charm and bottom at the starting scale
  //cout << endl;
  //for (int i = 0; i < mellinint::mdim; i++)
  //  for (int i = 0; i < 2*pegasus::dim; i++)
  //    for (int p = -MAXNF; p <= MAXNF; p++)
  //	cout << i << "  " << p << "  " << cx(PDFN[p+6][i]) << endl;
  //retrieve();
  //free();

  //restore pegasus settings
  //pegasus::init();
  //order_.npord_ = pdf::order; //order of evolution from LHAPDF
  //evmod_.imodev_ = 1;         //reproduces evolution in x-space
  //frrat_.logfr_ = 0.;

  complex <double> fx[11];
  for (int sign = mesq::positive; sign <= mesq::negative; sign++)
    for (int n = 0; n < pegasus::dim; n++)
      {
	int i = pegasus::index(n,sign);
	for (int p = -MAXNF; p <= MAXNF; p++)
	  fx[p+MAXNF] = cx(PDFN[p+6][i]);
	if (opts.mellin1d)
	  pdfevol::storemoments(n, sign, fx);
	else
	  if (n < mellinint::mdim)
	    pdfevol::storemoments_1(n, sign, fx);
	  else
	    pdfevol::storemoments_2(n-mellinint::mdim, sign, fx);
      }
}

//commute::free()
//{
//}
