#include "muf.h"
#include "resint.h"
#include "pmom.h"
#include "mesq.h"
#include "settings.h"
#include "ccoeff.h"
#include "resconst.h"
#include "expc.h"

using namespace pmom;
using namespace resconst;
using namespace ccoeff;

void muf::calc1d(complex <double> b)
{
  LQ  = resint::LQ;
  LQ2 = pow(LQ,2);
  LQ3 = pow(LQ,3);
  LQ4 = pow(LQ,4);
  LQ5 = pow(LQ,5);
  LQ6 = pow(LQ,6);
  LQ7 = pow(LQ,7);

  LR  = resint::LR;
  LR2 = pow(LR,2);
  LR3 = pow(LR,3);

  LF  = resint::LF;
  LF2 = pow(LF,2);
  LF3 = pow(LF,3);
  LF4 = pow(LF,4);
  LQF  = LF - LQ;
  LQF2 = pow(LQF,2);
  LQF3 = pow(LQF,3);
  LQF4 = pow(LQF,4);

  double as = resint::aass;
  double as2 = pow(as,2);
  double as3 = pow(as,3);
  double as4 = pow(as,4);

  A1 = A1q;
  A2 = A2q;
  A3 = A3q;
  A4 = A4q;

  B1 = B1q;
  B2 = B2q;
  B3 = B3q;
  B4 = B4q;

  for (int sign = mesq::positive; sign <= mesq::negative; sign++)
    for (int i = 0; i < mellinint::mdim; i++)
      {
	int idx = pmom::index(i,sign);
	EC1qq   [idx] = ccoeff::C1qq[idx];
	EC2qq   [idx] = ccoeff::C2qq[idx];
	EC3qq   [idx] = ccoeff::C3qq[idx];
	EC1qg   [idx] = ccoeff::C1qg[idx]*expc::aexpqg[idx]/expc::aexpqq[idx];
	EC2qg   [idx] = ccoeff::C2qg[idx]*expc::aexpqg[idx]/expc::aexpqq[idx];
	EC3qg   [idx] = ccoeff::C3qg[idx]*expc::aexpqg[idx]/expc::aexpqq[idx];
	EC2qqp  [idx] = ccoeff::C2qqp[idx]*expc::aexpqqp[idx]/expc::aexpqq[idx];
	EC2qqb  [idx] = ccoeff::C2qqb[idx]*expc::aexpqqb[idx]/expc::aexpqq[idx];
	EC2qqbp [idx] = ccoeff::C2qqbp[idx]*expc::aexpqqbp[idx]/expc::aexpqq[idx];
	EC3qqp  [idx] = ccoeff::C3qqp[idx]*expc::aexpqqp[idx]/expc::aexpqq[idx];
	EC3qqb  [idx] = ccoeff::C3qqb[idx]*expc::aexpqqb[idx]/expc::aexpqq[idx];
	EC3qqbp [idx] = ccoeff::C3qqbp[idx]*expc::aexpqqbp[idx]/expc::aexpqq[idx];
      }  
  
  for (int sign = mesq::positive; sign <= mesq::negative; sign++)
    for (int i = 0; i < mellinint::mdim; i++)
      {
	int idx = pmom::index(i,sign);

	//Include exponentiation in the C coefficients
	complex <double> eC1qq   = ccoeff::C1qq[idx];
	complex <double> eC2qq   = ccoeff::C2qq[idx];
	complex <double> eC3qq   = ccoeff::C3qq[idx];
	complex <double> eC1qg   = ccoeff::C1qg[idx]*expc::aexpqg[idx]/expc::aexpqq[idx];
	complex <double> eC2qg   = ccoeff::C2qg[idx]*expc::aexpqg[idx]/expc::aexpqq[idx];
	complex <double> eC3qg   = ccoeff::C3qg[idx]*expc::aexpqg[idx]/expc::aexpqq[idx];
	complex <double> eC2qqp  = ccoeff::C2qqp[idx]*expc::aexpqqp[idx]/expc::aexpqq[idx];
	complex <double> eC2qqb  = ccoeff::C2qqb[idx]*expc::aexpqqb[idx]/expc::aexpqq[idx];
	complex <double> eC2qqbp = ccoeff::C2qqbp[idx]*expc::aexpqqbp[idx]/expc::aexpqq[idx];
	complex <double> eC3qqp  = ccoeff::C3qqp[idx]*expc::aexpqqp[idx]/expc::aexpqq[idx];
	complex <double> eC3qqb  = ccoeff::C3qqb[idx]*expc::aexpqqb[idx]/expc::aexpqq[idx];
	complex <double> eC3qqbp = ccoeff::C3qqbp[idx]*expc::aexpqqbp[idx]/expc::aexpqq[idx];
	
	//NLO
	complex <double> H1st_qqb = 2.*gamma1qq[idx]*LQF;
	complex <double> H1st_qg  = gamma1qg[idx]*LQF;
	
	qqb[idx] = as*H1st_qqb;
	qg[idx]  = as*H1st_qg;

	if (opts.order_hcoef == 1)
	  continue;

	//NNLO
	complex <double> H2st_qqb  = LQF*(2.*eC1qg*gamma1gq[idx] + 2.*gamma2qq[idx] + gamma1gq[idx]*gamma1qg[idx]*LQF + gamma1qq[idx]*(4.*eC1qq + 2.*gamma1qq[idx]*LF - 2.*(B1q + gamma1qq[idx])*LQ - A1q*LQ2 + beta0*(LF + LQ)));
	complex <double> H2st_qq   = ((2.*eC1qg*gamma1gq[idx] + 2.*gamma2qqb[idx] + gamma1gq[idx]*gamma1qg[idx]*LQF)*LQF)/2.;
	complex <double> H2st_qqp  = ((2.*eC1qg*gamma1gq[idx] + 2.*gamma2qqp[idx] + gamma1gq[idx]*gamma1qg[idx]*LQF)*LQF)/2.;
	complex <double> H2st_qqbp = ((2.*eC1qg*gamma1gq[idx] + 2.*gamma2qqp[idx] + gamma1gq[idx]*gamma1qg[idx]*LQF)*LQF)/2.;
	complex <double> H2st_qg   = (LQF*(4.*eC1qq*gamma1qg[idx] + 2.*eC1qg*(gamma1gg[idx] + gamma1qq[idx]) + 2.*gamma2qg[idx] + beta0*gamma1qg[idx]*LF + gamma1gg[idx]*gamma1qg[idx]*LF + 3.*gamma1qg[idx]*gamma1qq[idx]*LF - 2.*B1q*gamma1qg[idx]*LQ + beta0*gamma1qg[idx]*LQ - gamma1gg[idx]*gamma1qg[idx]*LQ - 3.*gamma1qg[idx]*gamma1qq[idx]*LQ - A1q*gamma1qg[idx]*LQ2))/2.;
	complex <double> H2st_gg   = gamma1qg[idx]*(2.*eC1qg + gamma1qg[idx]*LQF)*LQF;
	  
	//Mixed LR*LQF variations
	H2st_qqb += -beta0*H1st_qqb*LR;
	H2st_qg  += -beta0*H1st_qg*LR;
	
	qqb[idx]  += as2*H2st_qqb;
	qg[idx]   += as2*H2st_qg;
	qq[idx]   = as2*H2st_qq;
	qqp[idx]  = as2*H2st_qqp;
	qqbp[idx] = as2*H2st_qqbp;
	gg[idx]   = as2*H2st_gg;

	if (opts.order_hcoef == 2)
	  continue;

	//NNNLO
	double nf = 5.;
	complex <double> H3st_qqb  = (LQF*(24.*eC2qg*gamma1gq[idx] + 24.*pow(eC1qq,2)*gamma1qq[idx] + 48.*eC2qq*gamma1qq[idx] + 48.*eC1qq*gamma2qq[idx] + 24.*gamma3qq[idx] + 24.*eC1qq*gamma1gq[idx]*gamma1qg[idx]*LF + 12.*beta1*gamma1qq[idx]*LF + 24.*beta0*eC1qq*gamma1qq[idx]*LF + 48.*eC1qq*pow(gamma1qq[idx],2)*LF + 12.*gamma1qg[idx]*gamma2gq[idx]*LF + 12.*gamma1gq[idx]*gamma2qg[idx]*LF + 24.*beta0*gamma2qq[idx]*LF + 48.*gamma1qq[idx]*gamma2qq[idx]*LF + 12.*beta0*gamma1gq[idx]*gamma1qg[idx]*LF2 + 4.*gamma1gg[idx]*gamma1gq[idx]*gamma1qg[idx]*LF2 + 8.*pow(beta0,2)*gamma1qq[idx]*LF2 + 20.*gamma1gq[idx]*gamma1qg[idx]*gamma1qq[idx]*LF2 + 24.*beta0*pow(gamma1qq[idx],2)*LF2 + 16.*pow(gamma1qq[idx],3)*LF2 - 24.*eC1qq*gamma1gq[idx]*gamma1qg[idx]*LQ - 24.*B2q*gamma1qq[idx]*LQ + 12.*beta1*gamma1qq[idx]*LQ - 48.*B1q*eC1qq*gamma1qq[idx]*LQ + 72.*beta0*eC1qq*gamma1qq[idx]*LQ - 48.*eC1qq*pow(gamma1qq[idx],2)*LQ - 12.*gamma1qg[idx]*gamma2gq[idx]*LQ - 12.*gamma1gq[idx]*gamma2qg[idx]*LQ - 24.*B1q*gamma2qq[idx]*LQ + 24.*beta0*gamma2qq[idx]*LQ - 48.*gamma1qq[idx]*gamma2qq[idx]*LQ - 12.*B1q*gamma1gq[idx]*gamma1qg[idx]*LF*LQ - 8.*gamma1gg[idx]*gamma1gq[idx]*gamma1qg[idx]*LF*LQ - 12.*B1q*beta0*gamma1qq[idx]*LF*LQ + 8.*pow(beta0,2)*gamma1qq[idx]*LF*LQ - 40.*gamma1gq[idx]*gamma1qg[idx]*gamma1qq[idx]*LF*LQ - 24.*B1q*pow(gamma1qq[idx],2)*LF*LQ - 32.*pow(gamma1qq[idx],3)*LF*LQ + 12.*B1q*gamma1gq[idx]*gamma1qg[idx]*LQ2 - 12.*beta0*gamma1gq[idx]*gamma1qg[idx]*LQ2 + 4.*gamma1gg[idx]*gamma1gq[idx]*gamma1qg[idx]*LQ2 - 12.*A2q*gamma1qq[idx]*LQ2 + 12.*pow(B1q,2)*gamma1qq[idx]*LQ2 - 24.*B1q*beta0*gamma1qq[idx]*LQ2 + 8.*pow(beta0,2)*gamma1qq[idx]*LQ2 - 24.*A1q*eC1qq*gamma1qq[idx]*LQ2 + 20.*gamma1gq[idx]*gamma1qg[idx]*gamma1qq[idx]*LQ2 + 24.*B1q*pow(gamma1qq[idx],2)*LQ2 - 24.*beta0*pow(gamma1qq[idx],2)*LQ2 + 16.*pow(gamma1qq[idx],3)*LQ2 - 12.*A1q*gamma2qq[idx]*LQ2 - 6.*A1q*gamma1gq[idx]*gamma1qg[idx]*LF*LQ2 - 6.*A1q*beta0*gamma1qq[idx]*LF*LQ2 - 12.*A1q*pow(gamma1qq[idx],2)*LF*LQ2 + 6.*A1q*gamma1gq[idx]*gamma1qg[idx]*LQ3 + 12.*A1q*B1q*gamma1qq[idx]*LQ3 - 14.*A1q*beta0*gamma1qq[idx]*LQ3 + 12.*A1q*pow(gamma1qq[idx],2)*LQ3 + 3.*pow(A1q,2)*gamma1qq[idx]*LQ4 + 12.*eC1qg*(2.*eC1qq*gamma1gq[idx] + 2.*gamma2gq[idx] + gamma1gq[idx]*(beta0*LF + gamma1gg[idx]*LF + 3.*gamma1qq[idx]*LF - 2.*B1q*LQ + 3.*beta0*LQ - gamma1gg[idx]*LQ - 3.*gamma1qq[idx]*LQ - A1q*LQ2))))/12.;
	complex <double> H3st_qq   = (LQF*(12.*(eC2qg*gamma1gq[idx] + 2.*eC2qqb*gamma1qq[idx] + 2.*eC1qq*gamma2qqb[idx] + gamma3qqb[idx]) + 6.*(2.*eC1qq*gamma1gq[idx]*gamma1qg[idx] + gamma1qg[idx]*gamma2gq[idx] + gamma1gq[idx]*gamma2qg[idx] + 2.*beta0*gamma2qqb[idx] + 4.*gamma1qq[idx]*gamma2qqb[idx])*LF + 2.*gamma1gq[idx]*gamma1qg[idx]*(3.*beta0 + gamma1gg[idx] + 5.*gamma1qq[idx])*LF2 - 2.*(6.*eC1qq*gamma1gq[idx]*gamma1qg[idx] + 3.*gamma1qg[idx]*gamma2gq[idx] + 3.*gamma1gq[idx]*gamma2qg[idx] + 6.*(B1q - beta0 + 2.*gamma1qq[idx])*gamma2qqb[idx] + gamma1gq[idx]*gamma1qg[idx]*(3.*B1q + 2.*gamma1gg[idx] + 10.*gamma1qq[idx])*LF)*LQ + (-6.*A1q*gamma2qqb[idx] + gamma1gq[idx]*gamma1qg[idx]*(6.*B1q - 6.*beta0 + 2.*gamma1gg[idx] + 10.*gamma1qq[idx] - 3.*A1q*LF))*LQ2 + 3.*A1q*gamma1gq[idx]*gamma1qg[idx]*LQ3 + 6.*eC1qg*(2.*gamma2gq[idx] + gamma1gq[idx]*(2.*eC1qq + (beta0 + gamma1gg[idx] + 3.*gamma1qq[idx])*LF - (2.*B1q - 3.*beta0 + gamma1gg[idx] + 3.*gamma1qq[idx])*LQ - A1q*LQ2))))/12.;
	complex <double> H3st_qqp  = (LQF*(12.*eC2qg*gamma1gq[idx] + 24.*eC2qqbp*gamma1qq[idx] + 24.*eC1qq*gamma2qqbp[idx] + 12.*gamma3qqbp[idx] + 12.*eC1qq*gamma1gq[idx]*gamma1qg[idx]*LF + 6.*gamma1qg[idx]*gamma2gq[idx]*LF + 6.*gamma1gq[idx]*gamma2qg[idx]*LF + 12.*beta0*gamma2qqbp[idx]*LF + 24.*gamma1qq[idx]*gamma2qqbp[idx]*LF + 6.*beta0*gamma1gq[idx]*gamma1qg[idx]*LF2 + 2.*gamma1gg[idx]*gamma1gq[idx]*gamma1qg[idx]*LF2 + 10.*gamma1gq[idx]*gamma1qg[idx]*gamma1qq[idx]*LF2 - 12.*eC1qq*gamma1gq[idx]*gamma1qg[idx]*LQ - 6.*gamma1qg[idx]*gamma2gq[idx]*LQ - 6.*gamma1gq[idx]*gamma2qg[idx]*LQ - 12.*B1q*gamma2qqbp[idx]*LQ + 12.*beta0*gamma2qqbp[idx]*LQ - 24.*gamma1qq[idx]*gamma2qqbp[idx]*LQ - 6.*B1q*gamma1gq[idx]*gamma1qg[idx]*LF*LQ - 4.*gamma1gg[idx]*gamma1gq[idx]*gamma1qg[idx]*LF*LQ - 20.*gamma1gq[idx]*gamma1qg[idx]*gamma1qq[idx]*LF*LQ + 6.*B1q*gamma1gq[idx]*gamma1qg[idx]*LQ2 - 6.*beta0*gamma1gq[idx]*gamma1qg[idx]*LQ2 + 2.*gamma1gg[idx]*gamma1gq[idx]*gamma1qg[idx]*LQ2 + 10.*gamma1gq[idx]*gamma1qg[idx]*gamma1qq[idx]*LQ2 - 6.*A1q*gamma2qqbp[idx]*LQ2 - 3.*A1q*gamma1gq[idx]*gamma1qg[idx]*LF*LQ2 + 3.*A1q*gamma1gq[idx]*gamma1qg[idx]*LQ3 + 6.*eC1qg*(2.*eC1qq*gamma1gq[idx] + 2.*gamma2gq[idx] + gamma1gq[idx]*(beta0*LF + gamma1gg[idx]*LF + 3.*gamma1qq[idx]*LF - 2.*B1q*LQ + 3.*beta0*LQ - gamma1gg[idx]*LQ - 3.*gamma1qq[idx]*LQ - A1q*LQ2))))/12.;
	complex <double> H3st_qqbp = (LQF*(12.*(eC2qg*gamma1gq[idx] + 2.*eC2qqp*gamma1qq[idx] + 2.*eC1qq*gamma2qqp[idx] + gamma3qqp[idx]) + 6.*(2.*eC1qq*gamma1gq[idx]*gamma1qg[idx] + gamma1qg[idx]*gamma2gq[idx] + gamma1gq[idx]*gamma2qg[idx] + 2.*beta0*gamma2qqp[idx] + 4.*gamma1qq[idx]*gamma2qqp[idx])*LF + 2.*gamma1gq[idx]*gamma1qg[idx]*(3.*beta0 + gamma1gg[idx] + 5.*gamma1qq[idx])*LF2 - 2.*(6.*eC1qq*gamma1gq[idx]*gamma1qg[idx] + 3.*gamma1qg[idx]*gamma2gq[idx] + 3.*gamma1gq[idx]*gamma2qg[idx] + 6.*(B1q - beta0 + 2.*gamma1qq[idx])*gamma2qqp[idx] + gamma1gq[idx]*gamma1qg[idx]*(3.*B1q + 2.*gamma1gg[idx] + 10.*gamma1qq[idx])*LF)*LQ + (-6.*A1q*gamma2qqp[idx] + gamma1gq[idx]*gamma1qg[idx]*(6.*B1q - 6.*beta0 + 2.*gamma1gg[idx] + 10.*gamma1qq[idx] - 3.*A1q*LF))*LQ2 + 3.*A1q*gamma1gq[idx]*gamma1qg[idx]*LQ3 + 6.*eC1qg*(2.*gamma2gq[idx] + gamma1gq[idx]*(2.*eC1qq + (beta0 + gamma1gg[idx] + 3.*gamma1qq[idx])*LF - (2.*B1q - 3.*beta0 + gamma1gg[idx] + 3.*gamma1qq[idx])*LQ - A1q*LQ2))))/12.;
	complex <double> H3st_qg   = (LQF*(24.*pow(eC1qg,2)*gamma1gq[idx] + 24.*eC2qg*(gamma1gg[idx] + gamma1qq[idx]) + 24.*((pow(eC1qq,2) + 2.*eC2qq + eC2qqb - eC2qqbp - eC2qqp)*gamma1qg[idx] + 2.*eC1qq*gamma2qg[idx] + gamma3qg[idx]) + 12.*(beta1 + 2.*beta0*eC1qq)*gamma1qg[idx]*LF + 24.*eC1qq*gamma1gg[idx]*gamma1qg[idx]*LF + 72.*eC1qq*gamma1qg[idx]*gamma1qq[idx]*LF + 12.*gamma1qg[idx]*gamma2gg[idx]*LF + 24.*beta0*gamma2qg[idx]*LF + 12.*gamma1gg[idx]*gamma2qg[idx]*LF + 36.*gamma1qq[idx]*gamma2qg[idx]*LF + 36.*gamma1qg[idx]*gamma2qq[idx]*LF + 12.*gamma1qg[idx]*gamma2qqb[idx]*LF - 12.*gamma1qg[idx]*gamma2qqbp[idx]*LF - 12.*gamma1qg[idx]*gamma2qqp[idx]*LF + 8.*pow(beta0,2)*gamma1qg[idx]*LF2 + 12.*beta0*gamma1gg[idx]*gamma1qg[idx]*LF2 + 4.*pow(gamma1gg[idx],2)*gamma1qg[idx]*LF2 + 12.*gamma1gq[idx]*pow(gamma1qg[idx],2)*LF2 + 36.*beta0*gamma1qg[idx]*gamma1qq[idx]*LF2 + 16.*gamma1gg[idx]*gamma1qg[idx]*gamma1qq[idx]*LF2 + 28.*gamma1qg[idx]*pow(gamma1qq[idx],2)*LF2 - 24.*B2q*gamma1qg[idx]*LQ + 12.*beta1*gamma1qg[idx]*LQ - 48.*B1q*eC1qq*gamma1qg[idx]*LQ + 72.*beta0*eC1qq*gamma1qg[idx]*LQ - 24.*eC1qq*gamma1gg[idx]*gamma1qg[idx]*LQ - 72.*eC1qq*gamma1qg[idx]*gamma1qq[idx]*LQ - 12.*gamma1qg[idx]*gamma2gg[idx]*LQ - 24.*B1q*gamma2qg[idx]*LQ + 24.*beta0*gamma2qg[idx]*LQ - 12.*gamma1gg[idx]*gamma2qg[idx]*LQ - 36.*gamma1qq[idx]*gamma2qg[idx]*LQ - 36.*gamma1qg[idx]*gamma2qq[idx]*LQ - 12.*gamma1qg[idx]*gamma2qqb[idx]*LQ + 12.*gamma1qg[idx]*gamma2qqbp[idx]*LQ + 12.*gamma1qg[idx]*gamma2qqp[idx]*LQ - 12.*B1q*beta0*gamma1qg[idx]*LF*LQ + 8.*pow(beta0,2)*gamma1qg[idx]*LF*LQ - 12.*B1q*gamma1gg[idx]*gamma1qg[idx]*LF*LQ - 8.*pow(gamma1gg[idx],2)*gamma1qg[idx]*LF*LQ - 24.*gamma1gq[idx]*pow(gamma1qg[idx],2)*LF*LQ - 36.*B1q*gamma1qg[idx]*gamma1qq[idx]*LF*LQ - 32.*gamma1gg[idx]*gamma1qg[idx]*gamma1qq[idx]*LF*LQ - 56.*gamma1qg[idx]*pow(gamma1qq[idx],2)*LF*LQ - 12.*A2q*gamma1qg[idx]*LQ2 + 12.*pow(B1q,2)*gamma1qg[idx]*LQ2 - 24.*B1q*beta0*gamma1qg[idx]*LQ2 + 8.*pow(beta0,2)*gamma1qg[idx]*LQ2 - 24.*A1q*eC1qq*gamma1qg[idx]*LQ2 + 12.*B1q*gamma1gg[idx]*gamma1qg[idx]*LQ2 - 12.*beta0*gamma1gg[idx]*gamma1qg[idx]*LQ2 + 4.*pow(gamma1gg[idx],2)*gamma1qg[idx]*LQ2 + 12.*gamma1gq[idx]*pow(gamma1qg[idx],2)*LQ2 + 36.*B1q*gamma1qg[idx]*gamma1qq[idx]*LQ2 - 36.*beta0*gamma1qg[idx]*gamma1qq[idx]*LQ2 + 16.*gamma1gg[idx]*gamma1qg[idx]*gamma1qq[idx]*LQ2 + 28.*gamma1qg[idx]*pow(gamma1qq[idx],2)*LQ2 - 12.*A1q*gamma2qg[idx]*LQ2 - 6.*A1q*beta0*gamma1qg[idx]*LF*LQ2 - 6.*A1q*gamma1gg[idx]*gamma1qg[idx]*LF*LQ2 - 18.*A1q*gamma1qg[idx]*gamma1qq[idx]*LF*LQ2 + 12.*A1q*B1q*gamma1qg[idx]*LQ3 - 14.*A1q*beta0*gamma1qg[idx]*LQ3 + 6.*A1q*gamma1gg[idx]*gamma1qg[idx]*LQ3 + 18.*A1q*gamma1qg[idx]*gamma1qq[idx]*LQ3 + 3.*pow(A1q,2)*gamma1qg[idx]*LQ4 + 4.*gamma1qg[idx]*(6.*eC2qqbp + 6.*eC2qqp + (3.*gamma2qqbp[idx] + 3.*gamma2qqp[idx] + 2.*gamma1gq[idx]*gamma1qg[idx]*LQF)*LQF)*nf + 12.*eC1qg*(2.*eC1qq*(gamma1gg[idx] + gamma1qq[idx]) + 2.*(gamma2gg[idx] + gamma2qq[idx]) + pow(gamma1gg[idx],2)*LF + 3.*gamma1gq[idx]*gamma1qg[idx]*LF + 2.*gamma1gg[idx]*gamma1qq[idx]*LF + pow(gamma1qq[idx],2)*LF - 2.*B1q*gamma1gg[idx]*LQ - pow(gamma1gg[idx],2)*LQ - 3.*gamma1gq[idx]*gamma1qg[idx]*LQ - 2.*B1q*gamma1qq[idx]*LQ - 2.*gamma1gg[idx]*gamma1qq[idx]*LQ - pow(gamma1qq[idx],2)*LQ - A1q*gamma1gg[idx]*LQ2 - A1q*gamma1qq[idx]*LQ2 + beta0*(gamma1gg[idx] + gamma1qq[idx])*(LF + 3.*LQ) + 2.*gamma1gq[idx]*gamma1qg[idx]*LQF*nf)))/24.;
	complex <double> H3st_qbg  = ((2.*eC2qqb*gamma1qg[idx] + (eC1qg + gamma1qg[idx]*LQF)*(2.*eC1qg*gamma1gq[idx] + 2.*gamma2qqb[idx] + gamma1gq[idx]*gamma1qg[idx]*LQF))*LQF)/2.;
	complex <double> H3st_qpg  = ((2.*eC2qqp*gamma1qg[idx] + (eC1qg + gamma1qg[idx]*LQF)*(2.*eC1qg*gamma1gq[idx] + 2.*gamma2qqp[idx] + gamma1gq[idx]*gamma1qg[idx]*LQF))*LQF)/2.;
	complex <double> H3st_qbpg = ((2.*eC2qqbp*gamma1qg[idx] + (eC1qg + gamma1qg[idx]*LQF)*(2.*eC1qg*gamma1gq[idx] + 2.*gamma2qqbp[idx] + gamma1gq[idx]*gamma1qg[idx]*LQF))*LQF)/2.;
	complex <double> H3st_gg   = (LQF*(4.*pow(eC1qg,2)*gamma1gg[idx] + 4.*eC2qg*gamma1qg[idx] + 2.*eC1qg*(2.*eC1qq*gamma1qg[idx] + 2.*gamma2qg[idx] + gamma1qg[idx]*(beta0*LF + 3.*gamma1gg[idx]*LF + gamma1qq[idx]*LF - 2.*B1q*LQ + 3.*beta0*LQ - 3.*gamma1gg[idx]*LQ - gamma1qq[idx]*LQ - A1q*LQ2)) + gamma1qg[idx]*LQF*(4.*eC1qq*gamma1qg[idx] + 4.*gamma2qg[idx] + gamma1qg[idx]*(2.*gamma1gg[idx]*LF + 2.*gamma1qq[idx]*LF - 2.*B1q*LQ - 2.*gamma1gg[idx]*LQ - 2.*gamma1qq[idx]*LQ - A1q*LQ2 + 2.*beta0*(LF + LQ)))))/2.;
	  
	//Mixed LR*LQF variations
	H3st_qqb  += -2*beta0*H2st_qqb*LR - (beta1*LR + pow(beta0,2)*LR2)*H1st_qqb;
	H3st_qg   += -2*beta0*H2st_qg*LR  - (beta1*LR + pow(beta0,2)*LR2)*H1st_qg;
	H3st_qq   += -2*beta0*H2st_qq*LR;
	H3st_qqp  += -2*beta0*H2st_qqp*LR;
	H3st_qqbp += -2*beta0*H2st_qqbp*LR;
	H3st_gg   += -2*beta0*H2st_gg*LR;
	
	qqb[idx]  += as3*H3st_qqb;
	qg[idx]   += as3*H3st_qg;
	qq[idx]   += as3*H3st_qq;
	qqp[idx]  += as3*H3st_qqp;
	qqbp[idx] += as3*H3st_qqbp;
	gg[idx]   += as3*H3st_gg;
	qbg[idx]  += as3*H3st_qbg;
	qpg[idx]  += as3*H3st_qpg;
	qbpg[idx] += as3*H3st_qbpg;


	if (opts.order_hcoef == 3)
	  continue;

	//NNNNLO
	//complex <double> H4st_qqb   =+ pow(gamma2qqbp[idx],2)*LF2+ pow(gamma2qqbp[idx],2)*LQ2- 2.*pow(gamma2qqbp[idx],2)*LQ*LF+ gamma2qqp[idx]*gamma2qqp[idx]*LF2+ gamma2qqp[idx]*gamma2qqp[idx]*LQ2- 2.*gamma2qqp[idx]*gamma2qqp[idx]*LQ*LF+ pow(gamma2qqb[idx],2)*LF2+ pow(gamma2qqb[idx],2)*LQ2- 2.*pow(gamma2qqb[idx],2)*LQ*LF+ gamma2qg[idx]*gamma1gq[idx]*LF2+ gamma2qg[idx]*gamma1gq[idx]*LQ2- 2.*gamma2qg[idx]*gamma1gq[idx]*LQ*LF+ 2.*gamma1qg[idx]*gamma3gq[idx]*LF2+ 2.*gamma1qg[idx]*gamma3gq[idx]*LQ2 - 2.*gamma1qg[idx]*gamma3gq[idx]*LQ*LF+ gamma1qg[idx]*gamma2gq[idx]*gamma1gg[idx]*LF3- gamma1qg[idx]*gamma2gq[idx]*gamma1gg[idx]*LQ3+ gamma1qg[idx]*gamma2gq[idx]*gamma1gg[idx]*LQ2*LF- 2.*gamma1qg[idx]*gamma2gq[idx]*gamma1gg[idx]*LQ*LF2+ 1./12.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LF4+ 1./12.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LQ4- 1./3.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LQ3*LF+ 1./2.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LQ2*LF2- 1./3.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LQ*LF3+ 1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LF4+ 1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ4- pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ3*LF+ 3./2.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2*LF2- pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF3 + 2.*gamma4qq[idx]*LF- 2.*gamma4qq[idx]*LQ+ 2.*gamma2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3- 2.*gamma2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+ 4.*gamma2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF- 5.*gamma2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+ 2.*pow(gamma2qq[idx],2)*LF2+ 2.*pow(gamma2qq[idx],2)*LQ2- 4.*pow(gamma2qq[idx],2)*LQ*LF+ 2.*gamma1qq[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ2*LF- 2.*gamma1qq[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ*LF2+ 3.*gamma1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LF3- 3.*gamma1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ3+ 5.*gamma1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ2*LF- 6.*gamma1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ*LF2 + 1./2.*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]* gamma1gg[idx]*LF4+ 1./2.*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4- 2.*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF+ 3.*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF2- 2.*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF3+ 4.*gamma1qq[idx]*gamma3qq[idx]*LF2+ 4.*gamma1qq[idx]*gamma3qq[idx]*LQ2- 6.*gamma1qq[idx]*gamma3qq[idx]*LQ*LF+ 17./12.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LF4+ 17./12.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ4- 17./3.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF+ 17./2.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2- 17./3.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3+ 4.*pow(gamma1qq[idx],2)*gamma2qq[idx]*LF3 - 4.*pow(gamma1qq[idx],2)*gamma2qq[idx]*LQ3+ 10.*pow(gamma1qq[idx],2)*gamma2qq[idx]*LQ2*LF- 11.*pow(gamma1qq[idx],2)*gamma2qq[idx]*LQ*LF2+ 2./3.*pow(gamma1qq[idx],4)*LF4+ 2./3.*pow(gamma1qq[idx],4)*LQ4- 8./3.*pow(gamma1qq[idx],4)*LQ3*LF+ 4.*pow(gamma1qq[idx],4)*LQ2*LF2- 8./3.*pow(gamma1qq[idx],4)*LQ*LF3+ 2.*EC2qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LF2+ 2.*EC2qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2- 4.*EC2qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ*LF+ EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LF3- EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ3+ 3.*EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ2*LF- 3.*EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ*LF2 + 2.*EC1qg[idx]*gamma2qq[idx]*gamma1gq[idx]*LF2+ 2.*EC1qg[idx]*gamma2qq[idx]*gamma1gq[idx]*LQ2- 4.*EC1qg[idx]*gamma2qq[idx]*gamma1gq[idx]*LQ*LF+ 2.*EC1qg[idx]*gamma1qq[idx]*gamma2gq[idx]*LF2+ 2.*EC1qg[idx]*gamma1qq[idx]*gamma2gq[idx]*LQ2- 4.*EC1qg[idx]*gamma1qq[idx]*gamma2gq[idx]*LQ*LF+ EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*gamma1gg[idx]*LF3- EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3+ 3.*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF- 3.*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF2+ 2.*EC1qg[idx]*pow(gamma1qq[idx],2)*gamma1gq[idx]*LF3- 2.*EC1qg[idx]*pow(gamma1qq[idx],2)*gamma1gq[idx]*LQ3+ 6.*EC1qg[idx]*pow(gamma1qq[idx],2)*gamma1gq[idx]*LQ2*LF- 6.*EC1qg[idx]*pow(gamma1qq[idx],2)*gamma1gq[idx]*LQ*LF2+ pow(EC1qg[idx],2)*pow(gamma1gq[idx],2)*LF2 + pow(EC1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2- 2.*pow(EC1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF+ 2.*EC3qq[idx]*gamma1qq[idx]*LF- 2.*EC3qq[idx]*gamma1qq[idx]*LQ+ EC2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+ EC2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2- 2.*EC2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+ 2.*EC2qq[idx]*gamma2qq[idx]*LF- 2.*EC2qq[idx]*gamma2qq[idx]*LQ+ 3.*EC2qq[idx]*pow(gamma1qq[idx],2)*LF2+ 3.*EC2qq[idx]*pow(gamma1qq[idx],2)*LQ2- 6.*EC2qq[idx]*pow(gamma1qq[idx],2)*LQ*LF+ 2.*EC2qq[idx]*EC1qg[idx]*gamma1gq[idx]*LF- 2.*EC2qq[idx]*EC1qg[idx]*gamma1gq[idx]*LQ- 2.*EC1qq[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ*LF + 2.*EC1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LF2+ 2.*EC1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ2- 2.*EC1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ*LF+ 1./3.*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF3- 1./3.*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3+ EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF- EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF2+ 2.*EC1qq[idx]*gamma3qq[idx]*LF- 2.*EC1qq[idx]*gamma3qq[idx]*LQ+ 8./3.*EC1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3- 8./3.*EC1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+ 8.*EC1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF- 8.*EC1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+ 6.*EC1qq[idx]*gamma1qq[idx]*gamma2qq[idx]*LF2+ 6.*EC1qq[idx]*gamma1qq[idx]*gamma2qq[idx]*LQ2 - 12.*EC1qq[idx]*gamma1qq[idx]*gamma2qq[idx]*LQ*LF+ 7./3.*EC1qq[idx]*pow(gamma1qq[idx],3)*LF3- 7./3.*EC1qq[idx]*pow(gamma1qq[idx],3)*LQ3+ 7.*EC1qq[idx]*pow(gamma1qq[idx],3)*LQ2*LF- 7.*EC1qq[idx]*pow(gamma1qq[idx],3)*LQ*LF2+ 2.*EC1qq[idx]*EC2qg[idx]*gamma1gq[idx]*LF- 2.*EC1qq[idx]*EC2qg[idx]*gamma1gq[idx]*LQ+ 2.*EC1qq[idx]*EC1qg[idx]*gamma2gq[idx]*LF- 2.*EC1qq[idx]*EC1qg[idx]*gamma2gq[idx]*LQ+ EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF2+ EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2- 2.*EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF+ 3.*EC1qq[idx]*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LF2+ 3.*EC1qq[idx]*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2- 6.*EC1qq[idx]*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ*LF + 4.*EC1qq[idx]*EC2qq[idx]*gamma1qq[idx]*LF- 4.*EC1qq[idx]*EC2qq[idx]*gamma1qq[idx]*LQ+ pow(EC1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LF2+ pow(EC1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ2- 2.*pow(EC1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+ 2.*pow(EC1qq[idx],2)*gamma2qq[idx]*LF- 2.*pow(EC1qq[idx],2)*gamma2qq[idx]*LQ+ 2.*pow(EC1qq[idx],2)*pow(gamma1qq[idx],2)*LF2+ 2.*pow(EC1qq[idx],2)*pow(gamma1qq[idx],2)*LQ2- 4.*pow(EC1qq[idx],2)*pow(gamma1qq[idx],2)*LQ*LF+ 1./6.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LF4+ 1./6.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ4- 2./3.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ3*LF+ nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2*LF2- 2./3.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF3 + beta2*gamma1qq[idx]*LF2- beta2*gamma1qq[idx]*LQ2+ beta1*gamma1qg[idx]*gamma1gq[idx]*LF3+ beta1*gamma1qg[idx]*gamma1gq[idx]*LQ3- beta1*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF- beta1*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+ 2.*beta1*gamma2qq[idx]*LF2- 2.*beta1*gamma2qq[idx]*LQ2+ 2.*beta1*pow(gamma1qq[idx],2)*LF3+ 2.*beta1*pow(gamma1qq[idx],2)*LQ3- 2.*beta1*pow(gamma1qq[idx],2)*LQ2*LF- 2.*beta1*pow(gamma1qq[idx],2)*LQ*LF2+ beta1*EC1qg[idx]*gamma1gq[idx]*LF2- 3.*beta1*EC1qg[idx]*gamma1gq[idx]*LQ2+ 2.*beta1*EC1qg[idx]*gamma1gq[idx]*LQ*LF + 2.*beta1*EC1qq[idx]*gamma1qq[idx]*LF2- 6.*beta1*EC1qq[idx]*gamma1qq[idx]*LQ2+ 4.*beta1*EC1qq[idx]*gamma1qq[idx]*LQ*LF- 2.*beta0*gamma2qg[idx]*gamma1gq[idx]*LQ2*LF- beta0*gamma2qg[idx]*gamma1gq[idx]*LQ*LF2+ 3.*beta0*gamma1qg[idx]*gamma2gq[idx]*LF3+ 3.*beta0*gamma1qg[idx]*gamma2gq[idx]*LQ3- beta0*gamma1qg[idx]*gamma2gq[idx]*LQ2*LF- 2.*beta0*gamma1qg[idx]*gamma2gq[idx]*LQ*LF2+ 1./2.*beta0*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF4- 1./2.*beta0*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4+ beta0*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF- beta0*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF3+ 3.*beta0*gamma3qq[idx]*LF2- 3.*beta0*gamma3qq[idx]*LQ2 + 5./2.*beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]* LF4- 5./2.*beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4+ 5.*beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF- 5.*beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3+ 6.*beta0*gamma1qq[idx]*gamma2qq[idx]*LF3+ 6.*beta0*gamma1qq[idx]*gamma2qq[idx]*LQ3- 6.*beta0*gamma1qq[idx]*gamma2qq[idx]*LQ2*LF- 6.*beta0*gamma1qq[idx]*gamma2qq[idx]*LQ*LF2+ 2.*beta0*pow(gamma1qq[idx],3)*LF4- 2.*beta0*pow(gamma1qq[idx],3)*LQ4+ 4.*beta0*pow(gamma1qq[idx],3)*LQ3*LF- 4.*beta0*pow(gamma1qq[idx],3)*LQ*LF3+ beta0*EC2qg[idx]*gamma1gq[idx]*LF2- 5.*beta0*EC2qg[idx]*gamma1gq[idx]*LQ2 + 4.*beta0*EC2qg[idx]*gamma1gq[idx]*LQ*LF+ 2.*beta0*EC1qg[idx]*gamma2gq[idx]*LF2- 4.*beta0*EC1qg[idx]*gamma2gq[idx]*LQ2+ 2.*beta0*EC1qg[idx]*gamma2gq[idx]*LQ*LF+ beta0*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF3+ 2.*beta0*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3- 3.*beta0*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF+ 3.*beta0*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LF3+ 6.*beta0*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ3- 9.*beta0*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2*LF+ 2.*beta0*EC2qq[idx]*gamma1qq[idx]*LF2- 10.*beta0*EC2qq[idx]*gamma1qq[idx]*LQ2+ 8.*beta0*EC2qq[idx]*gamma1qq[idx]*LQ*LF+ 2.*beta0*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3+ 4.*beta0*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3 - 6.*beta0*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF+ 4.*beta0*EC1qq[idx]*gamma2qq[idx]*LF2- 8.*beta0*EC1qq[idx]*gamma2qq[idx]*LQ2+ 4.*beta0*EC1qq[idx]*gamma2qq[idx]*LQ*LF+ 4.*beta0*EC1qq[idx]*pow(gamma1qq[idx],2)*LF3+ 8.*beta0*EC1qq[idx]*pow(gamma1qq[idx],2)*LQ3- 12.*beta0*EC1qq[idx]*pow(gamma1qq[idx],2)*LQ2*LF+ beta0*EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*LF2- 5.*beta0*EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*LQ2+ 4.*beta0*EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*LQ*LF+ beta0*pow(EC1qq[idx],2)*gamma1qq[idx]*LF2- 5.*beta0*pow(EC1qq[idx],2)*gamma1qq[idx]*LQ2+ 4.*beta0*pow(EC1qq[idx],2)*gamma1qq[idx]*LQ*LF+ 5./3.*beta0*beta1*gamma1qq[idx]*LF3- 5./3.*beta0*beta1*gamma1qq[idx]*LQ3 + 11./12.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LF4+ 11./12.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LQ4- 2./3.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF- 1./2.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2- 2./3.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3+ 2.*pow(beta0,2)*gamma2qq[idx]*LF3- 2.*pow(beta0,2)*gamma2qq[idx]*LQ3+ 11./6.*pow(beta0,2)*pow(gamma1qq[idx],2)*LF4+ 11./6.*pow(beta0,2)*pow(gamma1qq[idx],2)*LQ4- 4./3.*pow(beta0,2)*pow(gamma1qq[idx],2)*LQ3*LF- pow(beta0,2)*pow(gamma1qq[idx],2)*LQ2*LF2- 4./3.*pow(beta0,2)*pow(gamma1qq[idx],2)*LQ*LF3+ 2./3.*pow(beta0,2)*EC1qg[idx]*gamma1gq[idx]*LF3- 11./3.*pow(beta0,2)*EC1qg[idx]*gamma1gq[idx]*LQ3+ 2.*pow(beta0,2)*EC1qg[idx]*gamma1gq[idx]*LQ2*LF + pow(beta0,2)*EC1qg[idx]*gamma1gq[idx]*LQ*LF2+ 4./3.*pow(beta0,2)*EC1qq[idx]*gamma1qq[idx]*LF3- 22./3.*pow(beta0,2)*EC1qq[idx]*gamma1qq[idx]*LQ3+ 4.*pow(beta0,2)*EC1qq[idx]*gamma1qq[idx]*LQ2*LF+ 2.*pow(beta0,2)*EC1qq[idx]*gamma1qq[idx]*LQ*LF2+ 1./2.*pow(beta0,3)*gamma1qq[idx]*LF4- 1./2.*pow(beta0,3)*gamma1qq[idx]*LQ4+ 2.*B3*gamma1qq[idx]*LQ2- 2.*B3*gamma1qq[idx]*LQ*LF- B2*gamma1qg[idx]*gamma1gq[idx]*LQ3+ 2.*B2*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF- B2*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+ 2.*B2*gamma2qq[idx]*LQ2- 2.*B2*gamma2qq[idx]*LQ*LF- 2.*B2*pow(gamma1qq[idx],2)*LQ3 + 4.*B2*pow(gamma1qq[idx],2)*LQ2*LF- 2.*B2*pow(gamma1qq[idx],2)*LQ*LF2+ 2.*B2*EC1qg[idx]*gamma1gq[idx]*LQ2- 2.*B2*EC1qg[idx]*gamma1gq[idx]*LQ*LF+ 4.*B2*EC1qq[idx]*gamma1qq[idx]*LQ2- 4.*B2*EC1qq[idx]*gamma1qq[idx]*LQ*LF+ 3.*B2*beta0*gamma1qq[idx]*LQ3- 2.*B2*beta0*gamma1qq[idx]*LQ2*LF- B2*beta0*gamma1qq[idx]*LQ*LF2+ 2.*B1*gamma2qg[idx]*gamma1gq[idx]*LQ2*LF- 2.*B1*gamma1qg[idx]*gamma2gq[idx]*LQ3+ 2.*B1*gamma1qg[idx]*gamma2gq[idx]*LQ2*LF- 2.*B1*gamma1qg[idx]*gamma2gq[idx]*LQ*LF2+ 1./3.*B1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4- B1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF + B1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF2- 1./3.*B1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF3+ 2.*B1*gamma3qq[idx]*LQ2- 2.*B1*gamma3qq[idx]*LQ*LF+ 5./3.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4- 5.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF+ 5.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2- 5./3.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3- 4.*B1*gamma1qq[idx]*gamma2qq[idx]*LQ3+ 8.*B1*gamma1qq[idx]*gamma2qq[idx]*LQ2*LF- 4.*B1*gamma1qq[idx]*gamma2qq[idx]*LQ*LF2+ 4./3.*B1*pow(gamma1qq[idx],3)*LQ4- 4.*B1*pow(gamma1qq[idx],3)*LQ3*LF+ 4.*B1*pow(gamma1qq[idx],3)*LQ2*LF2- 4./3.*B1*pow(gamma1qq[idx],3)*LQ*LF3 + 2.*B1*EC2qg[idx]*gamma1gq[idx]*LQ2- 2.*B1*EC2qg[idx]*gamma1gq[idx]*LQ*LF+ 2.*B1*EC1qg[idx]*gamma2gq[idx]*LQ2- 2.*B1*EC1qg[idx]*gamma2gq[idx]*LQ*LF- B1*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3+ 2.*B1*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF- B1*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF2- 3.*B1*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ3+ 6.*B1*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2*LF- 3.*B1*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ*LF2+ 4.*B1*EC2qq[idx]*gamma1qq[idx]*LQ2- 4.*B1*EC2qq[idx]*gamma1qq[idx]*LQ*LF- 2.*B1*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+ 4.*B1*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF- 2.*B1*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2 + 4.*B1*EC1qq[idx]*gamma2qq[idx]*LQ2- 4.*B1*EC1qq[idx]*gamma2qq[idx]*LQ*LF- 4.*B1*EC1qq[idx]*pow(gamma1qq[idx],2)*LQ3+ 8.*B1*EC1qq[idx]*pow(gamma1qq[idx],2)*LQ2*LF- 4.*B1*EC1qq[idx]*pow(gamma1qq[idx],2)*LQ*LF2+ 2.*B1*EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*LQ2- 2.*B1*EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*LQ*LF+ 2.*B1*pow(EC1qq[idx],2)*gamma1qq[idx]*LQ2- 2.*B1*pow(EC1qq[idx],2)*gamma1qq[idx]*LQ*LF+ 2.*B1*beta1*gamma1qq[idx]*LQ3- B1*beta1*gamma1qq[idx]*LQ2*LF- B1*beta1*gamma1qq[idx]*LQ*LF2- 3./2.*B1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ4+ 2.*B1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF+ 1./2.*B1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2 - B1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3+ 3.*B1*beta0*gamma2qq[idx]*LQ3- B1*beta0*gamma2qq[idx]*LQ2*LF- 2.*B1*beta0*gamma2qq[idx]*LQ*LF2- 3.*B1*beta0*pow(gamma1qq[idx],2)*LQ4+ 4.*B1*beta0*pow(gamma1qq[idx],2)*LQ3*LF+ B1*beta0*pow(gamma1qq[idx],2)*LQ2*LF2- 2.*B1*beta0*pow(gamma1qq[idx],2)*LQ*LF3+ 4.*B1*beta0*EC1qg[idx]*gamma1gq[idx]*LQ3- 3.*B1*beta0*EC1qg[idx]*gamma1gq[idx]*LQ2*LF- B1*beta0*EC1qg[idx]*gamma1gq[idx]*LQ*LF2+ 8.*B1*beta0*EC1qq[idx]*gamma1qq[idx]*LQ3- 6.*B1*beta0*EC1qq[idx]*gamma1qq[idx]*LQ2*LF- 2.*B1*beta0*EC1qq[idx]*gamma1qq[idx]*LQ*LF2+ 11./6.*B1*pow(beta0,2)*gamma1qq[idx]*LQ4 - 2./3.*B1*pow(beta0,2)*gamma1qq[idx]*LQ3*LF- 1./2.*B1*pow(beta0,2)*gamma1qq[idx]*LQ2*LF2- 2./3.*B1*pow(beta0,2)*gamma1qq[idx]*LQ*LF3- 2.*B1*B2*gamma1qq[idx]*LQ3+ 2.*B1*B2*gamma1qq[idx]*LQ2*LF+ 1./2.*pow(B1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ4- pow(B1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF+ 1./2.*pow(B1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2- pow(B1,2)*gamma2qq[idx]*LQ3+ pow(B1,2)*gamma2qq[idx]*LQ2*LF+ pow(B1,2)*pow(gamma1qq[idx],2)*LQ4- 2.*pow(B1,2)*pow(gamma1qq[idx],2)*LQ3*LF+ pow(B1,2)*pow(gamma1qq[idx],2)*LQ2*LF2- pow(B1,2)*EC1qg[idx]*gamma1gq[idx]*LQ3+ pow(B1,2)*EC1qg[idx]*gamma1gq[idx]*LQ2*LF - 2.*pow(B1,2)*EC1qq[idx]*gamma1qq[idx]*LQ3+ 2.*pow(B1,2)*EC1qq[idx]*gamma1qq[idx]*LQ2*LF- 3./2.*pow(B1,2)*beta0*gamma1qq[idx]*LQ4+ pow(B1,2)*beta0*gamma1qq[idx]*LQ3*LF+ 1./2.*pow(B1,2)*beta0*gamma1qq[idx]*LQ2*LF2+ 1./3.*pow(B1,3)*gamma1qq[idx]*LQ4- 1./3.*pow(B1,3)*gamma1qq[idx]*LQ3*LF+ A3*gamma1qq[idx]*LQ3- A3*gamma1qq[idx]*LQ2*LF- 1./2.*A2*gamma1qg[idx]*gamma1gq[idx]*LQ4+ A2*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF- 1./2.*A2*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2+ A2*gamma2qq[idx]*LQ3- A2*gamma2qq[idx]*LQ2*LF- A2*pow(gamma1qq[idx],2)*LQ4 + 2.*A2*pow(gamma1qq[idx],2)*LQ3*LF- A2*pow(gamma1qq[idx],2)*LQ2*LF2+ A2*EC1qg[idx]*gamma1gq[idx]*LQ3- A2*EC1qg[idx]*gamma1gq[idx]*LQ2*LF+ 2.*A2*EC1qq[idx]*gamma1qq[idx]*LQ3- 2.*A2*EC1qq[idx]*gamma1qq[idx]*LQ2*LF+ 11./6.*A2*beta0*gamma1qq[idx]*LQ4- 4./3.*A2*beta0*gamma1qq[idx]*LQ3*LF- 1./2.*A2*beta0*gamma1qq[idx]*LQ2*LF2- A2*B1*gamma1qq[idx]*LQ4+ A2*B1*gamma1qq[idx]*LQ3*LF+ A1*gamma2qg[idx]*gamma1gq[idx]*LQ3*LF- A1*gamma1qg[idx]*gamma2gq[idx]*LQ4+ A1*gamma1qg[idx]*gamma2gq[idx]*LQ3*LF- A1*gamma1qg[idx]*gamma2gq[idx]*LQ2*LF2 + 1./6.*A1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ5- 1./2.*A1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4*LF+ 1./2.*A1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF2- 1./6.*A1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF3+ A1*gamma3qq[idx]*LQ3- A1*gamma3qq[idx]*LQ2*LF+ 5./6.*A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ5- 5./2.*A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4*LF+ 5./2.*A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF2- 5./6.*A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF3- 2.*A1*gamma1qq[idx]*gamma2qq[idx]*LQ4+ 4.*A1*gamma1qq[idx]*gamma2qq[idx]*LQ3*LF- 2.*A1*gamma1qq[idx]*gamma2qq[idx]*LQ2*LF2+ 2./3.*A1*pow(gamma1qq[idx],3)*LQ5- 2.*A1*pow(gamma1qq[idx],3)*LQ4*LF + 2.*A1*pow(gamma1qq[idx],3)*LQ3*LF2- 2./3.*A1*pow(gamma1qq[idx],3)*LQ2*LF3+ A1*EC2qg[idx]*gamma1gq[idx]*LQ3- A1*EC2qg[idx]*gamma1gq[idx]*LQ2*LF+ A1*EC1qg[idx]*gamma2gq[idx]*LQ3- A1*EC1qg[idx]*gamma2gq[idx]*LQ2*LF- 1./2.*A1*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4+ A1*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF- 1./2.*A1*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF2- 3./2.*A1*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ4+ 3.*A1*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ3*LF- 3./2.*A1*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2*LF2+ 2.*A1*EC2qq[idx]*gamma1qq[idx]*LQ3- 2.*A1*EC2qq[idx]*gamma1qq[idx]*LQ2*LF- A1*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4 + 2.*A1*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF- A1*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2+ 2.*A1*EC1qq[idx]*gamma2qq[idx]*LQ3- 2.*A1*EC1qq[idx]*gamma2qq[idx]*LQ2*LF- 2.*A1*EC1qq[idx]*pow(gamma1qq[idx],2)*LQ4+ 4.*A1*EC1qq[idx]*pow(gamma1qq[idx],2)*LQ3*LF- 2.*A1*EC1qq[idx]*pow(gamma1qq[idx],2)*LQ2*LF2+ A1*EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*LQ3- A1*EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*LQ2*LF+ A1*pow(EC1qq[idx],2)*gamma1qq[idx]*LQ3- A1*pow(EC1qq[idx],2)*gamma1qq[idx]*LQ2*LF+ 7./6.*A1*beta1*gamma1qq[idx]*LQ4- 2./3.*A1*beta1*gamma1qq[idx]*LQ3*LF- 1./2.*A1*beta1*gamma1qq[idx]*LQ2*LF2- 5./6.*A1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ5 + 7./6.*A1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ4*LF+ 1./6.*A1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF2- 1./2.*A1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF3+ 5./3.*A1*beta0*gamma2qq[idx]*LQ4- 2./3.*A1*beta0*gamma2qq[idx]*LQ3*LF- A1*beta0*gamma2qq[idx]*LQ2*LF2- 5./3.*A1*beta0*pow(gamma1qq[idx],2)*LQ5+ 7./3.*A1*beta0*pow(gamma1qq[idx],2)*LQ4*LF+ 1./3.*A1*beta0*pow(gamma1qq[idx],2)*LQ3*LF2- A1*beta0*pow(gamma1qq[idx],2)*LQ2*LF3+ 13./6.*A1*beta0*EC1qg[idx]*gamma1gq[idx]*LQ4- 5./3.*A1*beta0*EC1qg[idx]*gamma1gq[idx]*LQ3*LF- 1./2.*A1*beta0*EC1qg[idx]*gamma1gq[idx]*LQ2*LF2+ 13./3.*A1*beta0*EC1qq[idx]*gamma1qq[idx]*LQ4- 10./3.*A1*beta0*EC1qq[idx]*gamma1qq[idx]*LQ3*LF - A1*beta0*EC1qq[idx]*gamma1qq[idx]*LQ2*LF2+ 7./6.*A1*pow(beta0,2)*gamma1qq[idx]*LQ5- 1./2.*A1*pow(beta0,2)*gamma1qq[idx]*LQ4*LF- 1./3.*A1*pow(beta0,2)*gamma1qq[idx]*LQ3*LF2- 1./3.*A1*pow(beta0,2)*gamma1qq[idx]*LQ2*LF3- A1*B2*gamma1qq[idx]*LQ4+ A1*B2*gamma1qq[idx]*LQ3*LF+ 1./2.*A1*B1*gamma1qg[idx]*gamma1gq[idx]*LQ5- A1*B1*gamma1qg[idx]*gamma1gq[idx]*LQ4*LF+ 1./2.*A1*B1*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF2- A1*B1*gamma2qq[idx]*LQ4+ A1*B1*gamma2qq[idx]*LQ3*LF+ A1*B1*pow(gamma1qq[idx],2)*LQ5- 2.*A1*B1*pow(gamma1qq[idx],2)*LQ4*LF+ A1*B1*pow(gamma1qq[idx],2)*LQ3*LF2 - A1*B1*EC1qg[idx]*gamma1gq[idx]*LQ4+ A1*B1*EC1qg[idx]*gamma1gq[idx]*LQ3*LF- 2.*A1*B1*EC1qq[idx]*gamma1qq[idx]*LQ4+ 2.*A1*B1*EC1qq[idx]*gamma1qq[idx]*LQ3*LF- 5./3.*A1*B1*beta0*gamma1qq[idx]*LQ5+ 7./6.*A1*B1*beta0*gamma1qq[idx]*LQ4*LF+ 1./2.*A1*B1*beta0*gamma1qq[idx]*LQ3*LF2+ 1./2.*A1*pow(B1,2)*gamma1qq[idx]*LQ5- 1./2.*A1*pow(B1,2)*gamma1qq[idx]*LQ4*LF- 1./2.*A1*A2*gamma1qq[idx]*LQ5+ 1./2.*A1*A2*gamma1qq[idx]*LQ4*LF+ 1./8.*pow(A1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ6- 1./4.*pow(A1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ5*LF+ 1./8.*pow(A1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ4*LF2- 1./4.*pow(A1,2)*gamma2qq[idx]*LQ5 + 1./4.*pow(A1,2)*gamma2qq[idx]*LQ4*LF+ 1./4.*pow(A1,2)*pow(gamma1qq[idx],2)*LQ6- 1./2.*pow(A1,2)*pow(gamma1qq[idx],2)*LQ5*LF+ 1./4.*pow(A1,2)*pow(gamma1qq[idx],2)*LQ4*LF2- 1./4.*pow(A1,2)*EC1qg[idx]*gamma1gq[idx]*LQ5+ 1./4.*pow(A1,2)*EC1qg[idx]*gamma1gq[idx]*LQ4*LF- 1./2.*pow(A1,2)*EC1qq[idx]*gamma1qq[idx]*LQ5+ 1./2.*pow(A1,2)*EC1qq[idx]*gamma1qq[idx]*LQ4*LF- 11./24.*pow(A1,2)*beta0*gamma1qq[idx]*LQ6+ 1./3.*pow(A1,2)*beta0*gamma1qq[idx]*LQ5*LF+ 1./8.*pow(A1,2)*beta0*gamma1qq[idx]*LQ4*LF2+ 1./4.*pow(A1,2)*B1*gamma1qq[idx]*LQ6- 1./4.*pow(A1,2)*B1*gamma1qq[idx]*LQ5*LF+ 1./24.*pow(A1,3)*gamma1qq[idx]*LQ7- 1./24.*pow(A1,3)*gamma1qq[idx]*LQ6*LF ;
	//complex <double> H4st_qg    =+ gamma4qg[idx]*LF- gamma4qg[idx]*LQ+ 1./2.*gamma2qg[idx]*gamma2qqbp[idx]*LF2+ 1./2.*gamma2qg[idx]*gamma2qqbp[idx]*LQ2- gamma2qg[idx]*gamma2qqbp[idx]*LQ*LF+ 1./2.*gamma2qg[idx]*gamma2qqp[idx]*LF2+ 1./2.*gamma2qg[idx]*gamma2qqp[idx]*LQ2- gamma2qg[idx]*gamma2qqp[idx]*LQ*LF+ 1./2.*gamma2qg[idx]*gamma2qqb[idx]*LF2+ 1./2.*gamma2qg[idx]*gamma2qqb[idx]*LQ2- gamma2qg[idx]*gamma2qqb[idx]*LQ*LF+ 1./2.*gamma2qg[idx]*gamma1gg[idx]*LF2+ 1./2.*gamma2qg[idx]*gamma1gg[idx]*LQ2- gamma2qg[idx]*gamma1gg[idx]*LQ*LF + gamma1qg[idx]*gamma3gg[idx]*LF2+ gamma1qg[idx]*gamma3gg[idx]*LQ2- gamma1qg[idx]*gamma3gg[idx]*LQ*LF+ 1./2.*gamma1qg[idx]*gamma1gg[idx]*gamma2gg[idx]*LF3- 1./2.*gamma1qg[idx]*gamma1gg[idx]*gamma2gg[idx]*LQ3+ 1./2.*gamma1qg[idx]*gamma1gg[idx]*gamma2gg[idx]*LQ2*LF- gamma1qg[idx]*gamma1gg[idx]*gamma2gg[idx]*LQ*LF2+ 1./24.*gamma1qg[idx]*pow(gamma1gg[idx],3)*LF4+ 1./24.*gamma1qg[idx]*pow(gamma1gg[idx],3)*LQ4- 1./6.*gamma1qg[idx]*pow(gamma1gg[idx],3)*LQ3*LF+ 1./4.*gamma1qg[idx]*pow(gamma1gg[idx],3)*LQ2*LF2- 1./6.*gamma1qg[idx]*pow(gamma1gg[idx],3)*LQ*LF3+ 1./2.*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LF3- 1./2.*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ3+ 5./2.*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ2*LF - 5./2.*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ*LF2+ pow(gamma1qg[idx],2)*gamma2gq[idx]*LF3- pow(gamma1qg[idx],2)*gamma2gq[idx]*LQ3+ 2.*pow(gamma1qg[idx],2)*gamma2gq[idx]*LQ2*LF- 2.*pow(gamma1qg[idx],2)*gamma2gq[idx]*LQ*LF2+ 5./12.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LF4+ 5./12.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ4- 5./3.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF+ 5./2.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF2- 5./3.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ*LF3+ gamma3qq[idx]*gamma1qg[idx]*LF2+ gamma3qq[idx]*gamma1qg[idx]*LQ2- 2.*gamma3qq[idx]*gamma1qg[idx]*LQ*LF+ 3./2.*gamma2qq[idx]*gamma2qg[idx]*LF2+ 3./2.*gamma2qq[idx]*gamma2qg[idx]*LQ2 - 3.*gamma2qq[idx]*gamma2qg[idx]*LQ*LF+ 1./2.*gamma2qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LF3- 1./2.*gamma2qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ3+ 3./2.*gamma2qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF- 3./2.*gamma2qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ*LF2+ 2.*gamma1qq[idx]*gamma3qg[idx]*LF2+ 2.*gamma1qq[idx]*gamma3qg[idx]*LQ2- 3.*gamma1qq[idx]*gamma3qg[idx]*LQ*LF+ gamma1qq[idx]*gamma2qg[idx]*gamma1gg[idx]*LQ2*LF- gamma1qq[idx]*gamma2qg[idx]*gamma1gg[idx]*LQ*LF2+ gamma1qq[idx]*gamma1qg[idx]*gamma2qqbp[idx]*LQ2*LF- gamma1qq[idx]*gamma1qg[idx]*gamma2qqbp[idx]*LQ*LF2+ gamma1qq[idx]*gamma1qg[idx]*gamma2qqp[idx]*LQ2*LF- gamma1qq[idx]*gamma1qg[idx]*gamma2qqp[idx]*LQ*LF2+ gamma1qq[idx]*gamma1qg[idx]*gamma2qqb[idx]*LQ2*LF - gamma1qq[idx]*gamma1qg[idx]*gamma2qqb[idx]*LQ*LF2+ 3./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2gg[idx]*LF3- 3./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2gg[idx]*LQ3+ 5./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2gg[idx]*LQ2*LF- 3.*gamma1qq[idx]*gamma1qg[idx]*gamma2gg[idx]*LQ*LF2+ 5./24.*gamma1qq[idx]*gamma1qg[idx]*pow(gamma1gg[idx],2)*LF4+ 5./24.*gamma1qq[idx]*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ4- 5./6.*gamma1qq[idx]*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ3*LF+ 5./4.*gamma1qq[idx]*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ2*LF2- 5./6.*gamma1qq[idx]*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ*LF3+ 7./12.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LF4+ 7./12.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4- 7./3.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3*LF+ 7./2.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF2- 7./3.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF3 + 3./2.*gamma1qq[idx]*gamma2qq[idx]*gamma1qg[idx]*LF3- 3./2.*gamma1qq[idx]*gamma2qq[idx]*gamma1qg[idx]*LQ3+ 11./2.*gamma1qq[idx]*gamma2qq[idx]*gamma1qg[idx]*LQ2*LF- 11./2.*gamma1qq[idx]*gamma2qq[idx]*gamma1qg[idx]*LQ*LF2+ 2.*pow(gamma1qq[idx],2)*gamma2qg[idx]*LF3- 2.*pow(gamma1qq[idx],2)*gamma2qg[idx]*LQ3+ 4.*pow(gamma1qq[idx],2)*gamma2qg[idx]*LQ2*LF- 9./2.*pow(gamma1qq[idx],2)*gamma2qg[idx]*LQ*LF2+ 11./24.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gg[idx]*LF4+ 11./24.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gg[idx]*LQ4- 11./6.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gg[idx]*LQ3*LF+ 11./4.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF2- 11./6.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gg[idx]*LQ*LF3+ 5./8.*pow(gamma1qq[idx],3)*gamma1qg[idx]*LF4+ 5./8.*pow(gamma1qq[idx],3)*gamma1qg[idx]*LQ4 - 5./2.*pow(gamma1qq[idx],3)*gamma1qg[idx]*LQ3*LF+ 15./4.*pow(gamma1qq[idx],3)*gamma1qg[idx]*LQ2*LF2- 5./2.*pow(gamma1qq[idx],3)*gamma1qg[idx]*LQ*LF3+ EC3qg[idx]*gamma1qq[idx]*LF- EC3qg[idx]*gamma1qq[idx]*LQ+ 3./2.*EC2qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+ 3./2.*EC2qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2- 3.*EC2qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+ EC2qg[idx]*gamma2qq[idx]*LF- EC2qg[idx]*gamma2qq[idx]*LQ+ EC2qg[idx]*gamma1qq[idx]*gamma1gg[idx]*LF2+ EC2qg[idx]*gamma1qq[idx]*gamma1gg[idx]*LQ2- 2.*EC2qg[idx]*gamma1qq[idx]*gamma1gg[idx]*LQ*LF+ 1./2.*EC2qg[idx]*pow(gamma1qq[idx],2)*LF2+ 1./2.*EC2qg[idx]*pow(gamma1qq[idx],2)*LQ2 - EC2qg[idx]*pow(gamma1qq[idx],2)*LQ*LF+ EC1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LF2+ EC1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ2- 3.*EC1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ*LF+ 2.*EC1qg[idx]*gamma1qg[idx]*gamma2gq[idx]*LF2+ 2.*EC1qg[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ2- 3.*EC1qg[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ*LF+ 5./3.*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF3- 5./3.*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3+ 5.*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF- 5.*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF2+ EC1qg[idx]*gamma3qq[idx]*LF- EC1qg[idx]*gamma3qq[idx]*LQ+ EC1qg[idx]*gamma2qq[idx]*gamma1gg[idx]*LF2+ EC1qg[idx]*gamma2qq[idx]*gamma1gg[idx]*LQ2 - 2.*EC1qg[idx]*gamma2qq[idx]*gamma1gg[idx]*LQ*LF+ EC1qg[idx]*gamma1qq[idx]*gamma2gg[idx]*LF2+ EC1qg[idx]*gamma1qq[idx]*gamma2gg[idx]*LQ2- 2.*EC1qg[idx]*gamma1qq[idx]*gamma2gg[idx]*LQ*LF+ 1./2.*EC1qg[idx]*gamma1qq[idx]*pow(gamma1gg[idx],2)*LF3- 1./2.*EC1qg[idx]*gamma1qq[idx]*pow(gamma1gg[idx],2)*LQ3+ 3./2.*EC1qg[idx]*gamma1qq[idx]*pow(gamma1gg[idx],2)*LQ2*LF- 3./2.*EC1qg[idx]*gamma1qq[idx]*pow(gamma1gg[idx],2)*LQ*LF2+ 4./3.*EC1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3- 4./3.*EC1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+ 4.*EC1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF- 4.*EC1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+ EC1qg[idx]*gamma1qq[idx]*gamma2qq[idx]*LF2+ EC1qg[idx]*gamma1qq[idx]*gamma2qq[idx]*LQ2- 2.*EC1qg[idx]*gamma1qq[idx]*gamma2qq[idx]*LQ*LF + 1./2.*EC1qg[idx]*pow(gamma1qq[idx],2)*gamma1gg[idx]*LF3- 1./2.*EC1qg[idx]*pow(gamma1qq[idx],2)*gamma1gg[idx]*LQ3+ 3./2.*EC1qg[idx]*pow(gamma1qq[idx],2)*gamma1gg[idx]*LQ2*LF- 3./2.*EC1qg[idx]*pow(gamma1qq[idx],2)*gamma1gg[idx]*LQ*LF2+ 1./6.*EC1qg[idx]*pow(gamma1qq[idx],3)*LF3- 1./6.*EC1qg[idx]*pow(gamma1qq[idx],3)*LQ3+ 1./2.*EC1qg[idx]*pow(gamma1qq[idx],3)*LQ2*LF- 1./2.*EC1qg[idx]*pow(gamma1qq[idx],3)*LQ*LF2+ 2.*EC1qg[idx]*EC2qg[idx]*gamma1gq[idx]*LF- 2.*EC1qg[idx]*EC2qg[idx]*gamma1gq[idx]*LQ+ pow(EC1qg[idx],2)*gamma2gq[idx]*LF- pow(EC1qg[idx],2)*gamma2gq[idx]*LQ+ 3./2.*pow(EC1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LF2+ 3./2.*pow(EC1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ2- 3.*pow(EC1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ*LF + 1./2.*pow(EC1qg[idx],2)*gamma1qq[idx]*gamma1gq[idx]*LF2+ 1./2.*pow(EC1qg[idx],2)*gamma1qq[idx]*gamma1gq[idx]*LQ2- pow(EC1qg[idx],2)*gamma1qq[idx]*gamma1gq[idx]*LQ*LF+ EC2qqbp[idx]*gamma1qq[idx]*gamma1qg[idx]*LF2+ EC2qqbp[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ2- 2.*EC2qqbp[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ*LF+ EC2qqp[idx]*gamma1qq[idx]*gamma1qg[idx]*LF2+ EC2qqp[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ2- 2.*EC2qqp[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ*LF+ EC2qqb[idx]*gamma1qq[idx]*gamma1qg[idx]*LF2+ EC2qqb[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ2- 2.*EC2qqb[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ*LF+ EC3qq[idx]*gamma1qg[idx]*LF- EC3qq[idx]*gamma1qg[idx]*LQ+ EC2qq[idx]*gamma2qg[idx]*LF - EC2qq[idx]*gamma2qg[idx]*LQ+ 1./2.*EC2qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LF2+ 1./2.*EC2qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ2- EC2qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ*LF+ 5./2.*EC2qq[idx]*gamma1qq[idx]*gamma1qg[idx]*LF2+ 5./2.*EC2qq[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ2- 5.*EC2qq[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ*LF+ EC2qq[idx]*EC1qg[idx]*gamma1gg[idx]*LF- EC2qq[idx]*EC1qg[idx]*gamma1gg[idx]*LQ+ EC2qq[idx]*EC1qg[idx]*gamma1qq[idx]*LF- EC2qq[idx]*EC1qg[idx]*gamma1qq[idx]*LQ+ EC1qq[idx]*gamma3qg[idx]*LF- EC1qq[idx]*gamma3qg[idx]*LQ- EC1qq[idx]*gamma2qg[idx]*gamma1gg[idx]*LQ*LF- EC1qq[idx]*gamma1qg[idx]*gamma2qqbp[idx]*LQ*LF - EC1qq[idx]*gamma1qg[idx]*gamma2qqp[idx]*LQ*LF- EC1qq[idx]*gamma1qg[idx]*gamma2qqb[idx]*LQ*LF+ EC1qq[idx]*gamma1qg[idx]*gamma2gg[idx]*LF2+ EC1qq[idx]*gamma1qg[idx]*gamma2gg[idx]*LQ2- EC1qq[idx]*gamma1qg[idx]*gamma2gg[idx]*LQ*LF+ 1./6.*EC1qq[idx]*gamma1qg[idx]*pow(gamma1gg[idx],2)*LF3- 1./6.*EC1qq[idx]*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ3+ 1./2.*EC1qq[idx]*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ2*LF- 1./2.*EC1qq[idx]*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ*LF2+ EC1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LF3- EC1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3+ 3.*EC1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF- 3.*EC1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF2+ 2.*EC1qq[idx]*gamma2qq[idx]*gamma1qg[idx]*LF2+ 2.*EC1qq[idx]*gamma2qq[idx]*gamma1qg[idx]*LQ2 - 5.*EC1qq[idx]*gamma2qq[idx]*gamma1qg[idx]*LQ*LF+ 3.*EC1qq[idx]*gamma1qq[idx]*gamma2qg[idx]*LF2+ 3.*EC1qq[idx]*gamma1qq[idx]*gamma2qg[idx]*LQ2- 5.*EC1qq[idx]*gamma1qq[idx]*gamma2qg[idx]*LQ*LF+ 7./6.*EC1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LF3- 7./6.*EC1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ3+ 7./2.*EC1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF- 7./2.*EC1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ*LF2+ 13./6.*EC1qq[idx]*pow(gamma1qq[idx],2)*gamma1qg[idx]*LF3- 13./6.*EC1qq[idx]*pow(gamma1qq[idx],2)*gamma1qg[idx]*LQ3+ 13./2.*EC1qq[idx]*pow(gamma1qq[idx],2)*gamma1qg[idx]*LQ2*LF- 13./2.*EC1qq[idx]*pow(gamma1qq[idx],2)*gamma1qg[idx]*LQ*LF2+ EC1qq[idx]*EC2qg[idx]*gamma1gg[idx]*LF- EC1qq[idx]*EC2qg[idx]*gamma1gg[idx]*LQ+ EC1qq[idx]*EC2qg[idx]*gamma1qq[idx]*LF - EC1qq[idx]*EC2qg[idx]*gamma1qq[idx]*LQ+ EC1qq[idx]*EC1qg[idx]*gamma2gg[idx]*LF- EC1qq[idx]*EC1qg[idx]*gamma2gg[idx]*LQ+ 1./2.*EC1qq[idx]*EC1qg[idx]*pow(gamma1gg[idx],2)*LF2+ 1./2.*EC1qq[idx]*EC1qg[idx]*pow(gamma1gg[idx],2)*LQ2- EC1qq[idx]*EC1qg[idx]*pow(gamma1gg[idx],2)*LQ*LF+ 3./2.*EC1qq[idx]*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+ 3./2.*EC1qq[idx]*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2- 3.*EC1qq[idx]*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+ EC1qq[idx]*EC1qg[idx]*gamma2qq[idx]*LF- EC1qq[idx]*EC1qg[idx]*gamma2qq[idx]*LQ+ EC1qq[idx]*EC1qg[idx]*gamma1qq[idx]*gamma1gg[idx]*LF2+ EC1qq[idx]*EC1qg[idx]*gamma1qq[idx]*gamma1gg[idx]*LQ2- 2.*EC1qq[idx]*EC1qg[idx]*gamma1qq[idx]*gamma1gg[idx]*LQ*LF+ 1./2.*EC1qq[idx]*EC1qg[idx]*pow(gamma1qq[idx],2)*LF2 + 1./2.*EC1qq[idx]*EC1qg[idx]*pow(gamma1qq[idx],2)*LQ2- EC1qq[idx]*EC1qg[idx]*pow(gamma1qq[idx],2)*LQ*LF+ EC1qq[idx]*EC2qqbp[idx]*gamma1qg[idx]*LF- EC1qq[idx]*EC2qqbp[idx]*gamma1qg[idx]*LQ+ EC1qq[idx]*EC2qqp[idx]*gamma1qg[idx]*LF- EC1qq[idx]*EC2qqp[idx]*gamma1qg[idx]*LQ+ EC1qq[idx]*EC2qqb[idx]*gamma1qg[idx]*LF- EC1qq[idx]*EC2qqb[idx]*gamma1qg[idx]*LQ+ 2.*EC1qq[idx]*EC2qq[idx]*gamma1qg[idx]*LF- 2.*EC1qq[idx]*EC2qq[idx]*gamma1qg[idx]*LQ+ pow(EC1qq[idx],2)*gamma2qg[idx]*LF- pow(EC1qq[idx],2)*gamma2qg[idx]*LQ+ 1./2.*pow(EC1qq[idx],2)*gamma1qg[idx]*gamma1gg[idx]*LF2+ 1./2.*pow(EC1qq[idx],2)*gamma1qg[idx]*gamma1gg[idx]*LQ2- pow(EC1qq[idx],2)*gamma1qg[idx]*gamma1gg[idx]*LQ*LF + 3./2.*pow(EC1qq[idx],2)*gamma1qq[idx]*gamma1qg[idx]*LF2+ 3./2.*pow(EC1qq[idx],2)*gamma1qq[idx]*gamma1qg[idx]*LQ2- 3.*pow(EC1qq[idx],2)*gamma1qq[idx]*gamma1qg[idx]*LQ*LF+ nf*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LF3- nf*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ3+ nf*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ2*LF- 2.*nf*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ*LF2+ 1./12.*nf*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LF4+ 1./12.*nf*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ4- 1./3.*nf*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF+ 1./2.*nf*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF2- 1./3.*nf*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ*LF3+ 1./2.*nf*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LF4+ 1./2.*nf*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4- 2.*nf*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3*LF + 3.*nf*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2 *LF2- 2.*nf*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF3+ nf*EC1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3- nf*EC1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+ 3.*nf*EC1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF- 3.*nf*EC1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+ 1./3.*nf*EC1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LF3- 1./3.*nf*EC1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3+ nf*EC1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF- nf*EC1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF2+ nf*EC1qq[idx]*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+ nf*EC1qq[idx]*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2- 2.*nf*EC1qq[idx]*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+ 1./2.*beta2*gamma1qg[idx]*LF2 - 1./2.*beta2*gamma1qg[idx]*LQ2+ beta1*gamma2qg[idx]*LF2- beta1*gamma2qg[idx]*LQ2+ 1./2.*beta1*gamma1qg[idx]*gamma1gg[idx]*LF3+ 1./2.*beta1*gamma1qg[idx]*gamma1gg[idx]*LQ3- 1./2.*beta1*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF- 1./2.*beta1*gamma1qg[idx]*gamma1gg[idx]*LQ*LF2+ 3./2.*beta1*gamma1qq[idx]*gamma1qg[idx]*LF3+ 3./2.*beta1*gamma1qq[idx]*gamma1qg[idx]*LQ3- 3./2.*beta1*gamma1qq[idx]*gamma1qg[idx]*LQ2*LF- 3./2.*beta1*gamma1qq[idx]*gamma1qg[idx]*LQ*LF2+ 1./2.*beta1*EC1qg[idx]*gamma1gg[idx]*LF2- 3./2.*beta1*EC1qg[idx]*gamma1gg[idx]*LQ2+ beta1*EC1qg[idx]*gamma1gg[idx]*LQ*LF+ 1./2.*beta1*EC1qg[idx]*gamma1qq[idx]*LF2 - 3./2.*beta1*EC1qg[idx]*gamma1qq[idx]*LQ2+ beta1*EC1qg[idx]*gamma1qq[idx]*LQ*LF+ beta1*EC1qq[idx]*gamma1qg[idx]*LF2- 3.*beta1*EC1qq[idx]*gamma1qg[idx]*LQ2+ 2.*beta1*EC1qq[idx]*gamma1qg[idx]*LQ*LF+ 3./2.*beta0*gamma3qg[idx]*LF2- 3./2.*beta0*gamma3qg[idx]*LQ2- beta0*gamma2qg[idx]*gamma1gg[idx]*LQ2*LF- 1./2.*beta0*gamma2qg[idx]*gamma1gg[idx]*LQ*LF2- beta0*gamma1qg[idx]*gamma2qqbp[idx]*LQ2*LF- 1./2.*beta0*gamma1qg[idx]*gamma2qqbp[idx]*LQ*LF2- beta0*gamma1qg[idx]*gamma2qqp[idx]*LQ2*LF- 1./2.*beta0*gamma1qg[idx]*gamma2qqp[idx]*LQ*LF2- beta0*gamma1qg[idx]*gamma2qqb[idx]*LQ2*LF- 1./2.*beta0*gamma1qg[idx]*gamma2qqb[idx]*LQ*LF2 + 3./2.*beta0*gamma1qg[idx]*gamma2gg[idx]*LF3+ 3./2.*beta0*gamma1qg[idx]*gamma2gg[idx]*LQ3- 1./2.*beta0*gamma1qg[idx]*gamma2gg[idx]*LQ2*LF- beta0*gamma1qg[idx]*gamma2gg[idx]*LQ*LF2+ 1./4.*beta0*gamma1qg[idx]*pow(gamma1gg[idx],2)*LF4- 1./4.*beta0*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ4+ 1./2.*beta0*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ3*LF- 1./2.*beta0*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ*LF3+ 3./4.*beta0*pow(gamma1qg[idx],2)*gamma1gq[idx]*LF4- 3./4.*beta0*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4+ 3./2.*beta0*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3*LF- 3./2.*beta0*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF3+ 3./2.*beta0*gamma2qq[idx]*gamma1qg[idx]*LF3+ 3./2.*beta0*gamma2qq[idx]*gamma1qg[idx]*LQ3- 5./2.*beta0*gamma2qq[idx]*gamma1qg[idx]*LQ2*LF - 2.*beta0*gamma2qq[idx]*gamma1qg[idx]*LQ*LF2+ 3.*beta0*gamma1qq[idx]*gamma2qg[idx]*LF3+ 3.*beta0*gamma1qq[idx]*gamma2qg[idx]*LQ3- 2.*beta0*gamma1qq[idx]*gamma2qg[idx]*LQ2*LF- 5./2.*beta0*gamma1qq[idx]*gamma2qg[idx]*LQ*LF2+ beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LF4- beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ4+ 2.*beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ3*LF- 2.*beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ*LF3+ 7./4.*beta0*pow(gamma1qq[idx],2)*gamma1qg[idx]*LF4- 7./4.*beta0*pow(gamma1qq[idx],2)*gamma1qg[idx]*LQ4+ 7./2.*beta0*pow(gamma1qq[idx],2)*gamma1qg[idx]*LQ3*LF- 7./2.*beta0*pow(gamma1qq[idx],2)*gamma1qg[idx]*LQ*LF3+ 1./2.*beta0*EC2qg[idx]*gamma1gg[idx]*LF2- 5./2.*beta0*EC2qg[idx]*gamma1gg[idx]*LQ2 + 2.*beta0*EC2qg[idx]*gamma1gg[idx]*LQ*LF+ 1./2.*beta0*EC2qg[idx]*gamma1qq[idx]*LF2- 5./2.*beta0*EC2qg[idx]*gamma1qq[idx]*LQ2+ 2.*beta0*EC2qg[idx]*gamma1qq[idx]*LQ*LF+ beta0*EC1qg[idx]*gamma2gg[idx]*LF2- 2.*beta0*EC1qg[idx]*gamma2gg[idx]*LQ2+ beta0*EC1qg[idx]*gamma2gg[idx]*LQ*LF+ 1./2.*beta0*EC1qg[idx]*pow(gamma1gg[idx],2)*LF3+ beta0*EC1qg[idx]*pow(gamma1gg[idx],2)*LQ3- 3./2.*beta0*EC1qg[idx]*pow(gamma1gg[idx],2)*LQ2*LF+ 3./2.*beta0*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3+ 3.*beta0*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3- 9./2.*beta0*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF+ beta0*EC1qg[idx]*gamma2qq[idx]*LF2- 2.*beta0*EC1qg[idx]*gamma2qq[idx]*LQ2 + beta0*EC1qg[idx]*gamma2qq[idx]*LQ*LF+ beta0*EC1qg[idx]*gamma1qq[idx]*gamma1gg[idx]*LF3+ 2.*beta0*EC1qg[idx]*gamma1qq[idx]*gamma1gg[idx]*LQ3- 3.*beta0*EC1qg[idx]*gamma1qq[idx]*gamma1gg[idx]*LQ2*LF+ 1./2.*beta0*EC1qg[idx]*pow(gamma1qq[idx],2)*LF3+ beta0*EC1qg[idx]*pow(gamma1qq[idx],2)*LQ3- 3./2.*beta0*EC1qg[idx]*pow(gamma1qq[idx],2)*LQ2*LF+ 1./2.*beta0*pow(EC1qg[idx],2)*gamma1gq[idx]*LF2- 5./2.*beta0*pow(EC1qg[idx],2)*gamma1gq[idx]*LQ2+ 2.*beta0*pow(EC1qg[idx],2)*gamma1gq[idx]*LQ*LF+ 1./2.*beta0*EC2qqbp[idx]*gamma1qg[idx]*LF2- 5./2.*beta0*EC2qqbp[idx]*gamma1qg[idx]*LQ2+ 2.*beta0*EC2qqbp[idx]*gamma1qg[idx]*LQ*LF+ 1./2.*beta0*EC2qqp[idx]*gamma1qg[idx]*LF2- 5./2.*beta0*EC2qqp[idx]*gamma1qg[idx]*LQ2 + 2.*beta0*EC2qqp[idx]*gamma1qg[idx]*LQ*LF+ 1./2.*beta0*EC2qqb[idx]*gamma1qg[idx]*LF2- 5./2.*beta0*EC2qqb[idx]*gamma1qg[idx]*LQ2+ 2.*beta0*EC2qqb[idx]*gamma1qg[idx]*LQ*LF+ beta0*EC2qq[idx]*gamma1qg[idx]*LF2- 5.*beta0*EC2qq[idx]*gamma1qg[idx]*LQ2+ 4.*beta0*EC2qq[idx]*gamma1qg[idx]*LQ*LF+ 2.*beta0*EC1qq[idx]*gamma2qg[idx]*LF2- 4.*beta0*EC1qq[idx]*gamma2qg[idx]*LQ2+ 2.*beta0*EC1qq[idx]*gamma2qg[idx]*LQ*LF+ beta0*EC1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LF3+ 2.*beta0*EC1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ3- 3.*beta0*EC1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF+ 3.*beta0*EC1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*LF3+ 6.*beta0*EC1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ3 - 9.*beta0*EC1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ2*LF+ 1./2.*beta0*EC1qq[idx]*EC1qg[idx]*gamma1gg[idx]*LF2- 5./2.*beta0*EC1qq[idx]*EC1qg[idx]*gamma1gg[idx]*LQ2+ 2.*beta0*EC1qq[idx]*EC1qg[idx]*gamma1gg[idx]*LQ*LF+ 1./2.*beta0*EC1qq[idx]*EC1qg[idx]*gamma1qq[idx]*LF2- 5./2.*beta0*EC1qq[idx]*EC1qg[idx]*gamma1qq[idx]*LQ2+ 2.*beta0*EC1qq[idx]*EC1qg[idx]*gamma1qq[idx]*LQ*LF+ 1./2.*beta0*pow(EC1qq[idx],2)*gamma1qg[idx]*LF2- 5./2.*beta0*pow(EC1qq[idx],2)*gamma1qg[idx]*LQ2+ 2.*beta0*pow(EC1qq[idx],2)*gamma1qg[idx]*LQ*LF+ 1./2.*beta0*nf*pow(gamma1qg[idx],2)*gamma1gq[idx]*LF4- 1./2.*beta0*nf*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4+ beta0*nf*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3*LF- beta0*nf*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF3+ beta0*nf*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3 + 2.*beta0*nf*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3- 3.*beta0*nf*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF+ 5./6.*beta0*beta1*gamma1qg[idx]*LF3- 5./6.*beta0*beta1*gamma1qg[idx]*LQ3+ pow(beta0,2)*gamma2qg[idx]*LF3- pow(beta0,2)*gamma2qg[idx]*LQ3+ 11./24.*pow(beta0,2)*gamma1qg[idx]*gamma1gg[idx]*LF4+ 11./24.*pow(beta0,2)*gamma1qg[idx]*gamma1gg[idx]*LQ4- 1./3.*pow(beta0,2)*gamma1qg[idx]*gamma1gg[idx]*LQ3*LF- 1./4.*pow(beta0,2)*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF2- 1./3.*pow(beta0,2)*gamma1qg[idx]*gamma1gg[idx]*LQ*LF3+ 11./8.*pow(beta0,2)*gamma1qq[idx]*gamma1qg[idx]*LF4+ 11./8.*pow(beta0,2)*gamma1qq[idx]*gamma1qg[idx]*LQ4- pow(beta0,2)*gamma1qq[idx]*gamma1qg[idx]*LQ3*LF- 3./4.*pow(beta0,2)*gamma1qq[idx]*gamma1qg[idx]*LQ2*LF2 - pow(beta0,2)*gamma1qq[idx]*gamma1qg[idx]*LQ*LF3+ 1./3.*pow(beta0,2)*EC1qg[idx]*gamma1gg[idx]*LF3- 11./6.*pow(beta0,2)*EC1qg[idx]*gamma1gg[idx]*LQ3+ pow(beta0,2)*EC1qg[idx]*gamma1gg[idx]*LQ2*LF+ 1./2.*pow(beta0,2)*EC1qg[idx]*gamma1gg[idx]*LQ*LF2+ 1./3.*pow(beta0,2)*EC1qg[idx]*gamma1qq[idx]*LF3- 11./6.*pow(beta0,2)*EC1qg[idx]*gamma1qq[idx]*LQ3+ pow(beta0,2)*EC1qg[idx]*gamma1qq[idx]*LQ2*LF+ 1./2.*pow(beta0,2)*EC1qg[idx]*gamma1qq[idx]*LQ*LF2+ 2./3.*pow(beta0,2)*EC1qq[idx]*gamma1qg[idx]*LF3- 11./3.*pow(beta0,2)*EC1qq[idx]*gamma1qg[idx]*LQ3+ 2.*pow(beta0,2)*EC1qq[idx]*gamma1qg[idx]*LQ2*LF+ pow(beta0,2)*EC1qq[idx]*gamma1qg[idx]*LQ*LF2+ 1./4.*pow(beta0,3)*gamma1qg[idx]*LF4- 1./4.*pow(beta0,3)*gamma1qg[idx]*LQ4 + B3*gamma1qg[idx]*LQ2- B3*gamma1qg[idx]*LQ*LF+ B2*gamma2qg[idx]*LQ2- B2*gamma2qg[idx]*LQ*LF- 1./2.*B2*gamma1qg[idx]*gamma1gg[idx]*LQ3+ B2*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF- 1./2.*B2*gamma1qg[idx]*gamma1gg[idx]*LQ*LF2- 3./2.*B2*gamma1qq[idx]*gamma1qg[idx]*LQ3+ 3.*B2*gamma1qq[idx]*gamma1qg[idx]*LQ2*LF- 3./2.*B2*gamma1qq[idx]*gamma1qg[idx]*LQ*LF2+ B2*EC1qg[idx]*gamma1gg[idx]*LQ2- B2*EC1qg[idx]*gamma1gg[idx]*LQ*LF+ B2*EC1qg[idx]*gamma1qq[idx]*LQ2- B2*EC1qg[idx]*gamma1qq[idx]*LQ*LF+ 2.*B2*EC1qq[idx]*gamma1qg[idx]*LQ2 - 2.*B2*EC1qq[idx]*gamma1qg[idx]*LQ*LF+ 3./2.*B2*beta0*gamma1qg[idx]*LQ3- B2*beta0*gamma1qg[idx]*LQ2*LF- 1./2.*B2*beta0*gamma1qg[idx]*LQ*LF2+ B1*gamma3qg[idx]*LQ2- B1*gamma3qg[idx]*LQ*LF+ B1*gamma2qg[idx]*gamma1gg[idx]*LQ2*LF+ B1*gamma1qg[idx]*gamma2qqbp[idx]*LQ2*LF+ B1*gamma1qg[idx]*gamma2qqp[idx]*LQ2*LF+ B1*gamma1qg[idx]*gamma2qqb[idx]*LQ2*LF- B1*gamma1qg[idx]*gamma2gg[idx]*LQ3+ B1*gamma1qg[idx]*gamma2gg[idx]*LQ2*LF- B1*gamma1qg[idx]*gamma2gg[idx]*LQ*LF2+ 1./6.*B1*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ4- 1./2.*B1*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ3*LF + 1./2.*B1*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ2*LF2- 1./6.*B1*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ*LF3+ 1./2.*B1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4- 3./2.*B1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3*LF+ 3./2.*B1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF2- 1./2.*B1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF3- B1*gamma2qq[idx]*gamma1qg[idx]*LQ3+ 3.*B1*gamma2qq[idx]*gamma1qg[idx]*LQ2*LF- B1*gamma2qq[idx]*gamma1qg[idx]*LQ*LF2- 2.*B1*gamma1qq[idx]*gamma2qg[idx]*LQ3+ 3.*B1*gamma1qq[idx]*gamma2qg[idx]*LQ2*LF- 2.*B1*gamma1qq[idx]*gamma2qg[idx]*LQ*LF2+ 2./3.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ4- 2.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ3*LF+ 2.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF2 - 2./3.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ*LF3+ 7./6.*B1*pow(gamma1qq[idx],2)*gamma1qg[idx]*LQ4- 7./2.*B1*pow(gamma1qq[idx],2)*gamma1qg[idx]*LQ3*LF+ 7./2.*B1*pow(gamma1qq[idx],2)*gamma1qg[idx]*LQ2*LF2- 7./6.*B1*pow(gamma1qq[idx],2)*gamma1qg[idx]*LQ*LF3+ B1*EC2qg[idx]*gamma1gg[idx]*LQ2- B1*EC2qg[idx]*gamma1gg[idx]*LQ*LF+ B1*EC2qg[idx]*gamma1qq[idx]*LQ2- B1*EC2qg[idx]*gamma1qq[idx]*LQ*LF+ B1*EC1qg[idx]*gamma2gg[idx]*LQ2- B1*EC1qg[idx]*gamma2gg[idx]*LQ*LF- 1./2.*B1*EC1qg[idx]*pow(gamma1gg[idx],2)*LQ3+ B1*EC1qg[idx]*pow(gamma1gg[idx],2)*LQ2*LF- 1./2.*B1*EC1qg[idx]*pow(gamma1gg[idx],2)*LQ*LF2- 3./2.*B1*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3 + 3.*B1*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF- 3./2.*B1*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+ B1*EC1qg[idx]*gamma2qq[idx]*LQ2- B1*EC1qg[idx]*gamma2qq[idx]*LQ*LF- B1*EC1qg[idx]*gamma1qq[idx]*gamma1gg[idx]*LQ3+ 2.*B1*EC1qg[idx]*gamma1qq[idx]*gamma1gg[idx]*LQ2*LF- B1*EC1qg[idx]*gamma1qq[idx]*gamma1gg[idx]*LQ*LF2- 1./2.*B1*EC1qg[idx]*pow(gamma1qq[idx],2)*LQ3+ B1*EC1qg[idx]*pow(gamma1qq[idx],2)*LQ2*LF- 1./2.*B1*EC1qg[idx]*pow(gamma1qq[idx],2)*LQ*LF2+ B1*pow(EC1qg[idx],2)*gamma1gq[idx]*LQ2- B1*pow(EC1qg[idx],2)*gamma1gq[idx]*LQ*LF+ B1*EC2qqbp[idx]*gamma1qg[idx]*LQ2- B1*EC2qqbp[idx]*gamma1qg[idx]*LQ*LF+ B1*EC2qqp[idx]*gamma1qg[idx]*LQ2 - B1*EC2qqp[idx]*gamma1qg[idx]*LQ*LF+ B1*EC2qqb[idx]*gamma1qg[idx]*LQ2- B1*EC2qqb[idx]*gamma1qg[idx]*LQ*LF+ 2.*B1*EC2qq[idx]*gamma1qg[idx]*LQ2- 2.*B1*EC2qq[idx]*gamma1qg[idx]*LQ*LF+ 2.*B1*EC1qq[idx]*gamma2qg[idx]*LQ2- 2.*B1*EC1qq[idx]*gamma2qg[idx]*LQ*LF- B1*EC1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ3+ 2.*B1*EC1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF- B1*EC1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ*LF2- 3.*B1*EC1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ3+ 6.*B1*EC1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ2*LF- 3.*B1*EC1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ*LF2+ B1*EC1qq[idx]*EC1qg[idx]*gamma1gg[idx]*LQ2- B1*EC1qq[idx]*EC1qg[idx]*gamma1gg[idx]*LQ*LF + B1*EC1qq[idx]*EC1qg[idx]*gamma1qq[idx]*LQ2- B1*EC1qq[idx]*EC1qg[idx]*gamma1qq[idx]*LQ*LF+ B1*pow(EC1qq[idx],2)*gamma1qg[idx]*LQ2- B1*pow(EC1qq[idx],2)*gamma1qg[idx]*LQ*LF+ 1./3.*B1*nf*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4- B1*nf*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3*LF+ B1*nf*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF2- 1./3.*B1*nf*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF3- B1*nf*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+ 2.*B1*nf*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF- B1*nf*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+ B1*beta1*gamma1qg[idx]*LQ3- 1./2.*B1*beta1*gamma1qg[idx]*LQ2*LF- 1./2.*B1*beta1*gamma1qg[idx]*LQ*LF2+ 3./2.*B1*beta0*gamma2qg[idx]*LQ3 - 1./2.*B1*beta0*gamma2qg[idx]*LQ2*LF- B1*beta0*gamma2qg[idx]*LQ*LF2- 3./4.*B1*beta0*gamma1qg[idx]*gamma1gg[idx]*LQ4+ B1*beta0*gamma1qg[idx]*gamma1gg[idx]*LQ3*LF+ 1./4.*B1*beta0*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF2- 1./2.*B1*beta0*gamma1qg[idx]*gamma1gg[idx]*LQ*LF3- 9./4.*B1*beta0*gamma1qq[idx]*gamma1qg[idx]*LQ4+ 3.*B1*beta0*gamma1qq[idx]*gamma1qg[idx]*LQ3*LF+ 3./4.*B1*beta0*gamma1qq[idx]*gamma1qg[idx]*LQ2*LF2- 3./2.*B1*beta0*gamma1qq[idx]*gamma1qg[idx]*LQ*LF3+ 2.*B1*beta0*EC1qg[idx]*gamma1gg[idx]*LQ3- 3./2.*B1*beta0*EC1qg[idx]*gamma1gg[idx]*LQ2*LF- 1./2.*B1*beta0*EC1qg[idx]*gamma1gg[idx]*LQ*LF2+ 2.*B1*beta0*EC1qg[idx]*gamma1qq[idx]*LQ3- 3./2.*B1*beta0*EC1qg[idx]*gamma1qq[idx]*LQ2*LF - 1./2.*B1*beta0*EC1qg[idx]*gamma1qq[idx]*LQ*LF2+ 4.*B1*beta0*EC1qq[idx]*gamma1qg[idx]*LQ3- 3.*B1*beta0*EC1qq[idx]*gamma1qg[idx]*LQ2*LF- B1*beta0*EC1qq[idx]*gamma1qg[idx]*LQ*LF2+ 11./12.*B1*pow(beta0,2)*gamma1qg[idx]*LQ4- 1./3.*B1*pow(beta0,2)*gamma1qg[idx]*LQ3*LF- 1./4.*B1*pow(beta0,2)*gamma1qg[idx]*LQ2*LF2- 1./3.*B1*pow(beta0,2)*gamma1qg[idx]*LQ*LF3- B1*B2*gamma1qg[idx]*LQ3+ B1*B2*gamma1qg[idx]*LQ2*LF- 1./2.*pow(B1,2)*gamma2qg[idx]*LQ3+ 1./2.*pow(B1,2)*gamma2qg[idx]*LQ2*LF+ 1./4.*pow(B1,2)*gamma1qg[idx]*gamma1gg[idx]*LQ4- 1./2.*pow(B1,2)*gamma1qg[idx]*gamma1gg[idx]*LQ3*LF+ 1./4.*pow(B1,2)*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF2 + 3./4.*pow(B1,2)*gamma1qq[idx]*gamma1qg[idx]*LQ4- 3./2.*pow(B1,2)*gamma1qq[idx]*gamma1qg[idx]*LQ3*LF+ 3./4.*pow(B1,2)*gamma1qq[idx]*gamma1qg[idx]*LQ2*LF2- 1./2.*pow(B1,2)*EC1qg[idx]*gamma1gg[idx]*LQ3+ 1./2.*pow(B1,2)*EC1qg[idx]*gamma1gg[idx]*LQ2*LF- 1./2.*pow(B1,2)*EC1qg[idx]*gamma1qq[idx]*LQ3+ 1./2.*pow(B1,2)*EC1qg[idx]*gamma1qq[idx]*LQ2*LF- pow(B1,2)*EC1qq[idx]*gamma1qg[idx]*LQ3+ pow(B1,2)*EC1qq[idx]*gamma1qg[idx]*LQ2*LF- 3./4.*pow(B1,2)*beta0*gamma1qg[idx]*LQ4+ 1./2.*pow(B1,2)*beta0*gamma1qg[idx]*LQ3*LF+ 1./4.*pow(B1,2)*beta0*gamma1qg[idx]*LQ2*LF2+ 1./6.*pow(B1,3)*gamma1qg[idx]*LQ4- 1./6.*pow(B1,3)*gamma1qg[idx]*LQ3*LF+ 1./2.*A3*gamma1qg[idx]*LQ3 - 1./2.*A3*gamma1qg[idx]*LQ2*LF+ 1./2.*A2*gamma2qg[idx]*LQ3- 1./2.*A2*gamma2qg[idx]*LQ2*LF- 1./4.*A2*gamma1qg[idx]*gamma1gg[idx]*LQ4+ 1./2.*A2*gamma1qg[idx]*gamma1gg[idx]*LQ3*LF- 1./4.*A2*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF2- 3./4.*A2*gamma1qq[idx]*gamma1qg[idx]*LQ4+ 3./2.*A2*gamma1qq[idx]*gamma1qg[idx]*LQ3*LF- 3./4.*A2*gamma1qq[idx]*gamma1qg[idx]*LQ2*LF2+ 1./2.*A2*EC1qg[idx]*gamma1gg[idx]*LQ3- 1./2.*A2*EC1qg[idx]*gamma1gg[idx]*LQ2*LF+ 1./2.*A2*EC1qg[idx]*gamma1qq[idx]*LQ3- 1./2.*A2*EC1qg[idx]*gamma1qq[idx]*LQ2*LF+ A2*EC1qq[idx]*gamma1qg[idx]*LQ3- A2*EC1qq[idx]*gamma1qg[idx]*LQ2*LF + 11./12.*A2*beta0*gamma1qg[idx]*LQ4- 2./3.*A2*beta0*gamma1qg[idx]*LQ3*LF- 1./4.*A2*beta0*gamma1qg[idx]*LQ2*LF2- 1./2.*A2*B1*gamma1qg[idx]*LQ4+ 1./2.*A2*B1*gamma1qg[idx]*LQ3*LF+ 1./2.*A1*gamma3qg[idx]*LQ3- 1./2.*A1*gamma3qg[idx]*LQ2*LF+ 1./2.*A1*gamma2qg[idx]*gamma1gg[idx]*LQ3*LF+ 1./2.*A1*gamma1qg[idx]*gamma2qqbp[idx]*LQ3*LF+ 1./2.*A1*gamma1qg[idx]*gamma2qqp[idx]*LQ3*LF+ 1./2.*A1*gamma1qg[idx]*gamma2qqb[idx]*LQ3*LF- 1./2.*A1*gamma1qg[idx]*gamma2gg[idx]*LQ4+ 1./2.*A1*gamma1qg[idx]*gamma2gg[idx]*LQ3*LF- 1./2.*A1*gamma1qg[idx]*gamma2gg[idx]*LQ2*LF2+ 1./12.*A1*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ5 - 1./4.*A1*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ4*LF+ 1./4.*A1*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ3*LF2- 1./12.*A1*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ2*LF3+ 1./4.*A1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ5- 3./4.*A1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4*LF+ 3./4.*A1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3*LF2- 1./4.*A1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF3- 1./2.*A1*gamma2qq[idx]*gamma1qg[idx]*LQ4+ 3./2.*A1*gamma2qq[idx]*gamma1qg[idx]*LQ3*LF- 1./2.*A1*gamma2qq[idx]*gamma1qg[idx]*LQ2*LF2- A1*gamma1qq[idx]*gamma2qg[idx]*LQ4+ 3./2.*A1*gamma1qq[idx]*gamma2qg[idx]*LQ3*LF- A1*gamma1qq[idx]*gamma2qg[idx]*LQ2*LF2+ 1./3.*A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ5- A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ4*LF + A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ3*LF2- 1./3.*A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF3+ 7./12.*A1*pow(gamma1qq[idx],2)*gamma1qg[idx]*LQ5- 7./4.*A1*pow(gamma1qq[idx],2)*gamma1qg[idx]*LQ4*LF+ 7./4.*A1*pow(gamma1qq[idx],2)*gamma1qg[idx]*LQ3*LF2- 7./12.*A1*pow(gamma1qq[idx],2)*gamma1qg[idx]*LQ2*LF3+ 1./2.*A1*EC2qg[idx]*gamma1gg[idx]*LQ3- 1./2.*A1*EC2qg[idx]*gamma1gg[idx]*LQ2*LF+ 1./2.*A1*EC2qg[idx]*gamma1qq[idx]*LQ3- 1./2.*A1*EC2qg[idx]*gamma1qq[idx]*LQ2*LF+ 1./2.*A1*EC1qg[idx]*gamma2gg[idx]*LQ3- 1./2.*A1*EC1qg[idx]*gamma2gg[idx]*LQ2*LF- 1./4.*A1*EC1qg[idx]*pow(gamma1gg[idx],2)*LQ4+ 1./2.*A1*EC1qg[idx]*pow(gamma1gg[idx],2)*LQ3*LF- 1./4.*A1*EC1qg[idx]*pow(gamma1gg[idx],2)*LQ2*LF2 - 3./4.*A1*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4+ 3./2.*A1*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF- 3./4.*A1*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2+ 1./2.*A1*EC1qg[idx]*gamma2qq[idx]*LQ3- 1./2.*A1*EC1qg[idx]*gamma2qq[idx]*LQ2*LF- 1./2.*A1*EC1qg[idx]*gamma1qq[idx]*gamma1gg[idx]*LQ4+ A1*EC1qg[idx]*gamma1qq[idx]*gamma1gg[idx]*LQ3*LF- 1./2.*A1*EC1qg[idx]*gamma1qq[idx]*gamma1gg[idx]*LQ2*LF2- 1./4.*A1*EC1qg[idx]*pow(gamma1qq[idx],2)*LQ4+ 1./2.*A1*EC1qg[idx]*pow(gamma1qq[idx],2)*LQ3*LF- 1./4.*A1*EC1qg[idx]*pow(gamma1qq[idx],2)*LQ2*LF2+ 1./2.*A1*pow(EC1qg[idx],2)*gamma1gq[idx]*LQ3- 1./2.*A1*pow(EC1qg[idx],2)*gamma1gq[idx]*LQ2*LF+ 1./2.*A1*EC2qqbp[idx]*gamma1qg[idx]*LQ3- 1./2.*A1*EC2qqbp[idx]*gamma1qg[idx]*LQ2*LF + 1./2.*A1*EC2qqp[idx]*gamma1qg[idx]*LQ3- 1./2.*A1*EC2qqp[idx]*gamma1qg[idx]*LQ2*LF+ 1./2.*A1*EC2qqb[idx]*gamma1qg[idx]*LQ3- 1./2.*A1*EC2qqb[idx]*gamma1qg[idx]*LQ2*LF+ A1*EC2qq[idx]*gamma1qg[idx]*LQ3- A1*EC2qq[idx]*gamma1qg[idx]*LQ2*LF+ A1*EC1qq[idx]*gamma2qg[idx]*LQ3- A1*EC1qq[idx]*gamma2qg[idx]*LQ2*LF- 1./2.*A1*EC1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ4+ A1*EC1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ3*LF- 1./2.*A1*EC1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF2- 3./2.*A1*EC1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ4+ 3.*A1*EC1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ3*LF- 3./2.*A1*EC1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ2*LF2+ 1./2.*A1*EC1qq[idx]*EC1qg[idx]*gamma1gg[idx]*LQ3 - 1./2.*A1*EC1qq[idx]*EC1qg[idx]*gamma1gg[idx]*LQ2*LF+ 1./2.*A1*EC1qq[idx]*EC1qg[idx]*gamma1qq[idx]*LQ3- 1./2.*A1*EC1qq[idx]*EC1qg[idx]*gamma1qq[idx]*LQ2*LF+ 1./2.*A1*pow(EC1qq[idx],2)*gamma1qg[idx]*LQ3- 1./2.*A1*pow(EC1qq[idx],2)*gamma1qg[idx]*LQ2*LF+ 1./6.*A1*nf*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ5- 1./2.*A1*nf*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4*LF+ 1./2.*A1*nf*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3*LF2- 1./6.*A1*nf*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF3- 1./2.*A1*nf*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4+ A1*nf*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF- 1./2.*A1*nf*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2+ 7./12.*A1*beta1*gamma1qg[idx]*LQ4- 1./3.*A1*beta1*gamma1qg[idx]*LQ3*LF- 1./4.*A1*beta1*gamma1qg[idx]*LQ2*LF2 + 5./6.*A1*beta0*gamma2qg[idx]*LQ4- 1./3.*A1*beta0*gamma2qg[idx]*LQ3*LF- 1./2.*A1*beta0*gamma2qg[idx]*LQ2*LF2- 5./12.*A1*beta0*gamma1qg[idx]*gamma1gg[idx]*LQ5+ 7./12.*A1*beta0*gamma1qg[idx]*gamma1gg[idx]*LQ4*LF+ 1./12.*A1*beta0*gamma1qg[idx]*gamma1gg[idx]*LQ3*LF2- 1./4.*A1*beta0*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF3- 5./4.*A1*beta0*gamma1qq[idx]*gamma1qg[idx]*LQ5+ 7./4.*A1*beta0*gamma1qq[idx]*gamma1qg[idx]*LQ4*LF+ 1./4.*A1*beta0*gamma1qq[idx]*gamma1qg[idx]*LQ3*LF2- 3./4.*A1*beta0*gamma1qq[idx]*gamma1qg[idx]*LQ2*LF3+ 13./12.*A1*beta0*EC1qg[idx]*gamma1gg[idx]*LQ4- 5./6.*A1*beta0*EC1qg[idx]*gamma1gg[idx]*LQ3*LF- 1./4.*A1*beta0*EC1qg[idx]*gamma1gg[idx]*LQ2*LF2+ 13./12.*A1*beta0*EC1qg[idx]*gamma1qq[idx]*LQ4 - 5./6.*A1*beta0*EC1qg[idx]*gamma1qq[idx]*LQ3*LF- 1./4.*A1*beta0*EC1qg[idx]*gamma1qq[idx]*LQ2*LF2+ 13./6.*A1*beta0*EC1qq[idx]*gamma1qg[idx]*LQ4- 5./3.*A1*beta0*EC1qq[idx]*gamma1qg[idx]*LQ3*LF- 1./2.*A1*beta0*EC1qq[idx]*gamma1qg[idx]*LQ2*LF2+ 7./12.*A1*pow(beta0,2)*gamma1qg[idx]*LQ5- 1./4.*A1*pow(beta0,2)*gamma1qg[idx]*LQ4*LF- 1./6.*A1*pow(beta0,2)*gamma1qg[idx]*LQ3*LF2- 1./6.*A1*pow(beta0,2)*gamma1qg[idx]*LQ2*LF3- 1./2.*A1*B2*gamma1qg[idx]*LQ4+ 1./2.*A1*B2*gamma1qg[idx]*LQ3*LF- 1./2.*A1*B1*gamma2qg[idx]*LQ4+ 1./2.*A1*B1*gamma2qg[idx]*LQ3*LF+ 1./4.*A1*B1*gamma1qg[idx]*gamma1gg[idx]*LQ5- 1./2.*A1*B1*gamma1qg[idx]*gamma1gg[idx]*LQ4*LF + 1./4.*A1*B1*gamma1qg[idx]*gamma1gg[idx]*LQ3*LF2+ 3./4.*A1*B1*gamma1qq[idx]*gamma1qg[idx]*LQ5- 3./2.*A1*B1*gamma1qq[idx]*gamma1qg[idx]*LQ4*LF+ 3./4.*A1*B1*gamma1qq[idx]*gamma1qg[idx]*LQ3*LF2- 1./2.*A1*B1*EC1qg[idx]*gamma1gg[idx]*LQ4+ 1./2.*A1*B1*EC1qg[idx]*gamma1gg[idx]*LQ3*LF- 1./2.*A1*B1*EC1qg[idx]*gamma1qq[idx]*LQ4+ 1./2.*A1*B1*EC1qg[idx]*gamma1qq[idx]*LQ3*LF- A1*B1*EC1qq[idx]*gamma1qg[idx]*LQ4+ A1*B1*EC1qq[idx]*gamma1qg[idx]*LQ3*LF- 5./6.*A1*B1*beta0*gamma1qg[idx]*LQ5+ 7./12.*A1*B1*beta0*gamma1qg[idx]*LQ4*LF+ 1./4.*A1*B1*beta0*gamma1qg[idx]*LQ3*LF2+ 1./4.*A1*pow(B1,2)*gamma1qg[idx]*LQ5- 1./4.*A1*pow(B1,2)*gamma1qg[idx]*LQ4*LF - 1./4.*A1*A2*gamma1qg[idx]*LQ5+ 1./4.*A1*A2*gamma1qg[idx]*LQ4*LF- 1./8.*pow(A1,2)*gamma2qg[idx]*LQ5+ 1./8.*pow(A1,2)*gamma2qg[idx]*LQ4*LF+ 1./16.*pow(A1,2)*gamma1qg[idx]*gamma1gg[idx]*LQ6- 1./8.*pow(A1,2)*gamma1qg[idx]*gamma1gg[idx]*LQ5*LF+ 1./16.*pow(A1,2)*gamma1qg[idx]*gamma1gg[idx]*LQ4*LF2+ 3./16.*pow(A1,2)*gamma1qq[idx]*gamma1qg[idx]*LQ6- 3./8.*pow(A1,2)*gamma1qq[idx]*gamma1qg[idx]*LQ5*LF+ 3./16.*pow(A1,2)*gamma1qq[idx]*gamma1qg[idx]*LQ4*LF2- 1./8.*pow(A1,2)*EC1qg[idx]*gamma1gg[idx]*LQ5+ 1./8.*pow(A1,2)*EC1qg[idx]*gamma1gg[idx]*LQ4*LF- 1./8.*pow(A1,2)*EC1qg[idx]*gamma1qq[idx]*LQ5+ 1./8.*pow(A1,2)*EC1qg[idx]*gamma1qq[idx]*LQ4*LF- 1./4.*pow(A1,2)*EC1qq[idx]*gamma1qg[idx]*LQ5 + 1./4.*pow(A1,2)*EC1qq[idx]*gamma1qg[idx]*LQ4*LF- 11./48.*pow(A1,2)*beta0*gamma1qg[idx]*LQ6+ 1./6.*pow(A1,2)*beta0*gamma1qg[idx]*LQ5*LF+ 1./16.*pow(A1,2)*beta0*gamma1qg[idx]*LQ4*LF2+ 1./8.*pow(A1,2)*B1*gamma1qg[idx]*LQ6- 1./8.*pow(A1,2)*B1*gamma1qg[idx]*LQ5*LF+ 1./48.*pow(A1,3)*gamma1qg[idx]*LQ7- 1./48.*pow(A1,3)*gamma1qg[idx]*LQ6*LF ;
	//complex <double> H4st_qq    =+ 1./2.*gamma2qqp[idx]*gamma2qqbp[idx]*LF2+ 1./2.*gamma2qqp[idx]*gamma2qqbp[idx]*LQ2- gamma2qqp[idx]*gamma2qqbp[idx]*LQ*LF+ 1./2.*gamma2qqp[idx]*gamma2qqbp[idx]*LF2+ 1./2.*gamma2qqp[idx]*gamma2qqbp[idx]*LQ2- gamma2qqp[idx]*gamma2qqbp[idx]*LQ*LF+ gamma4qqb[idx]*LF- gamma4qqb[idx]*LQ+ 1./2.*gamma2qg[idx]*gamma1gq[idx]*LF2+ 1./2.*gamma2qg[idx]*gamma1gq[idx]*LQ2- gamma2qg[idx]*gamma1gq[idx]*LQ*LF+ gamma1qg[idx]*gamma3gq[idx]*LF2+ gamma1qg[idx]*gamma3gq[idx]*LQ2- gamma1qg[idx]*gamma3gq[idx]*LQ*LF + 1./2.*gamma1qg[idx]*gamma2gq[idx]*gamma1gg[idx]*LF3- 1./2.*gamma1qg[idx]*gamma2gq[idx]*gamma1gg[idx]*LQ3+ 1./2.*gamma1qg[idx]*gamma2gq[idx]*gamma1gg[idx]*LQ2*LF- gamma1qg[idx]*gamma2gq[idx]*gamma1gg[idx]*LQ*LF2+ 1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LF3- 1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ3+ 1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ2*LF- gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ*LF2+ 1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LF3- 1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ3+ 1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ2*LF- gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ*LF2+ gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LF3- gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ3+ 2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ2*LF - 5./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ*LF2+ 1./24.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LF4+ 1./24.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LQ4- 1./6.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LQ3*LF+ 1./4.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LQ2*LF2- 1./6.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LQ*LF3+ 1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LF4+ 1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ4- pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ3*LF+ 3./2.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2*LF2- pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF3+ 2.*gamma2qq[idx]*gamma2qqb[idx]*LF2+ 2.*gamma2qq[idx]*gamma2qqb[idx]*LQ2- 4.*gamma2qq[idx]*gamma2qqb[idx]*LQ*LF+ gamma2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3 - gamma2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+ 2.*gamma2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF- 5./2.*gamma2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+ 2.*gamma1qq[idx]*gamma3qqb[idx]*LF2+ 2.*gamma1qq[idx]*gamma3qqb[idx]*LQ2- 3.*gamma1qq[idx]*gamma3qqb[idx]*LQ*LF+ gamma1qq[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ2*LF- gamma1qq[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ*LF2+ 3./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LF3- 3./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ3+ 5./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ2*LF- 3.*gamma1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ*LF2+ 1./4.*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF4+ 1./4.*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4- gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF + 3./2.*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]* gamma1gg[idx]*LQ2*LF2- gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF3+ pow(gamma1qq[idx],2)*gamma2qqb[idx]*LF3- pow(gamma1qq[idx],2)*gamma2qqb[idx]*LQ3+ 3.*pow(gamma1qq[idx],2)*gamma2qqb[idx]*LQ2*LF- 7./2.*pow(gamma1qq[idx],2)*gamma2qqb[idx]*LQ*LF2+ 17./24.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LF4+ 17./24.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ4- 17./6.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF+ 17./4.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2- 17./6.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3+ EC2qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LF2+ EC2qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2- 2.*EC2qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ*LF + EC1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LF2+ EC1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ2- 2.*EC1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ*LF+ EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LF3- EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ3+ 3.*EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ2*LF- 3.*EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ*LF2+ EC1qg[idx]*gamma2qq[idx]*gamma1gq[idx]*LF2+ EC1qg[idx]*gamma2qq[idx]*gamma1gq[idx]*LQ2- 2.*EC1qg[idx]*gamma2qq[idx]*gamma1gq[idx]*LQ*LF+ EC1qg[idx]*gamma1qq[idx]*gamma2gq[idx]*LF2+ EC1qg[idx]*gamma1qq[idx]*gamma2gq[idx]*LQ2- 2.*EC1qg[idx]*gamma1qq[idx]*gamma2gq[idx]*LQ*LF+ 1./2.*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*gamma1gg[idx]*LF3- 1./2.*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3 + 3./2.*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*gamma1gg[idx]* LQ2*LF- 3./2.*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF2+ EC1qg[idx]*pow(gamma1qq[idx],2)*gamma1gq[idx]*LF3- EC1qg[idx]*pow(gamma1qq[idx],2)*gamma1gq[idx]*LQ3+ 3.*EC1qg[idx]*pow(gamma1qq[idx],2)*gamma1gq[idx]*LQ2*LF- 3.*EC1qg[idx]*pow(gamma1qq[idx],2)*gamma1gq[idx]*LQ*LF2+ pow(EC1qg[idx],2)*pow(gamma1gq[idx],2)*LF2+ pow(EC1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2- 2.*pow(EC1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF+ EC3qqb[idx]*gamma1qq[idx]*LF- EC3qqb[idx]*gamma1qq[idx]*LQ+ 1./2.*EC2qqb[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+ 1./2.*EC2qqb[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2- EC2qqb[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF + EC2qqb[idx]*gamma2qq[idx]*LF- EC2qqb[idx]*gamma2qq[idx]*LQ+ 1./2.*EC2qqb[idx]*pow(gamma1qq[idx],2)*LF2+ 1./2.*EC2qqb[idx]*pow(gamma1qq[idx],2)*LQ2- EC2qqb[idx]*pow(gamma1qq[idx],2)*LQ*LF+ EC2qqb[idx]*EC1qg[idx]*gamma1gq[idx]*LF- EC2qqb[idx]*EC1qg[idx]*gamma1gq[idx]*LQ+ EC2qq[idx]*gamma2qqb[idx]*LF- EC2qq[idx]*gamma2qqb[idx]*LQ+ 1./2.*EC2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+ 1./2.*EC2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2- EC2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+ EC2qq[idx]*EC1qg[idx]*gamma1gq[idx]*LF- EC2qq[idx]*EC1qg[idx]*gamma1gq[idx]*LQ+ EC1qq[idx]*gamma3qqb[idx]*LF - EC1qq[idx]*gamma3qqb[idx]*LQ- EC1qq[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ*LF+ EC1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LF2+ EC1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ2- EC1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ*LF+ 1./6.*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF3- 1./6.*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3+ 1./2.*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF- 1./2.*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF2+ 2.*EC1qq[idx]*gamma1qq[idx]*gamma2qqb[idx]*LF2+ 2.*EC1qq[idx]*gamma1qq[idx]*gamma2qqb[idx]*LQ2- 5.*EC1qq[idx]*gamma1qq[idx]*gamma2qqb[idx]*LQ*LF+ 4./3.*EC1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3- 4./3.*EC1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+ 4.*EC1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF - 4.*EC1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ* LF2+ EC1qq[idx]*EC2qg[idx]*gamma1gq[idx]*LF- EC1qq[idx]*EC2qg[idx]*gamma1gq[idx]*LQ+ EC1qq[idx]*EC1qg[idx]*gamma2gq[idx]*LF- EC1qq[idx]*EC1qg[idx]*gamma2gq[idx]*LQ+ 1./2.*EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF2+ 1./2.*EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2- EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF+ 3./2.*EC1qq[idx]*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LF2+ 3./2.*EC1qq[idx]*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2- 3.*EC1qq[idx]*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ*LF+ EC1qq[idx]*EC2qqb[idx]*gamma1qq[idx]*LF- EC1qq[idx]*EC2qqb[idx]*gamma1qq[idx]*LQ+ pow(EC1qq[idx],2)*gamma2qqb[idx]*LF - pow(EC1qq[idx],2)*gamma2qqb[idx]*LQ+ 1./2.*pow(EC1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LF2+ 1./2.*pow(EC1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ2- pow(EC1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+ 1./12.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LF4+ 1./12.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ4- 1./3.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ3*LF+ 1./2.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2*LF2- 1./3.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF3+ beta1*gamma2qqb[idx]*LF2- beta1*gamma2qqb[idx]*LQ2+ 1./2.*beta1*gamma1qg[idx]*gamma1gq[idx]*LF3+ 1./2.*beta1*gamma1qg[idx]*gamma1gq[idx]*LQ3- 1./2.*beta1*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF- 1./2.*beta1*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2 + 1./2.*beta1*EC1qg[idx]*gamma1gq[idx]*LF2- 3./2.*beta1*EC1qg[idx]*gamma1gq[idx]*LQ2+ beta1*EC1qg[idx]*gamma1gq[idx]*LQ*LF+ 3./2.*beta0*gamma3qqb[idx]*LF2- 3./2.*beta0*gamma3qqb[idx]*LQ2- beta0*gamma2qg[idx]*gamma1gq[idx]*LQ2*LF- 1./2.*beta0*gamma2qg[idx]*gamma1gq[idx]*LQ*LF2+ 3./2.*beta0*gamma1qg[idx]*gamma2gq[idx]*LF3+ 3./2.*beta0*gamma1qg[idx]*gamma2gq[idx]*LQ3- 1./2.*beta0*gamma1qg[idx]*gamma2gq[idx]*LQ2*LF- beta0*gamma1qg[idx]*gamma2gq[idx]*LQ*LF2+ 1./4.*beta0*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF4- 1./4.*beta0*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4+ 1./2.*beta0*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF- 1./2.*beta0*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF3 + 3./2.*beta0*gamma1qq[idx]*gamma2qqb[idx]*LF3+ 3./2.*beta0*gamma1qq[idx]*gamma2qqb[idx]*LQ3- 5./2.*beta0*gamma1qq[idx]*gamma2qqb[idx]*LQ2*LF- 2.*beta0*gamma1qq[idx]*gamma2qqb[idx]*LQ*LF2+ 5./4.*beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF4- 5./4.*beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4+ 5./2.*beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF- 5./2.*beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3+ 1./2.*beta0*EC2qg[idx]*gamma1gq[idx]*LF2- 5./2.*beta0*EC2qg[idx]*gamma1gq[idx]*LQ2+ 2.*beta0*EC2qg[idx]*gamma1gq[idx]*LQ*LF+ beta0*EC1qg[idx]*gamma2gq[idx]*LF2- 2.*beta0*EC1qg[idx]*gamma2gq[idx]*LQ2+ beta0*EC1qg[idx]*gamma2gq[idx]*LQ*LF+ 1./2.*beta0*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF3 + beta0*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3- 3./2.*beta0*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF+ 3./2.*beta0*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LF3+ 3.*beta0*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ3- 9./2.*beta0*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2*LF+ 1./2.*beta0*EC2qqb[idx]*gamma1qq[idx]*LF2- 5./2.*beta0*EC2qqb[idx]*gamma1qq[idx]*LQ2+ 2.*beta0*EC2qqb[idx]*gamma1qq[idx]*LQ*LF+ 2.*beta0*EC1qq[idx]*gamma2qqb[idx]*LF2- 4.*beta0*EC1qq[idx]*gamma2qqb[idx]*LQ2+ 2.*beta0*EC1qq[idx]*gamma2qqb[idx]*LQ*LF+ beta0*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3+ 2.*beta0*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3- 3.*beta0*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF+ 1./2.*beta0*EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*LF2 - 5./2.*beta0*EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*LQ2+ 2.*beta0*EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*LQ*LF+ pow(beta0,2)*gamma2qqb[idx]*LF3- pow(beta0,2)*gamma2qqb[idx]*LQ3+ 11./24.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LF4+ 11./24.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LQ4- 1./3.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF- 1./4.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2- 1./3.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3+ 1./3.*pow(beta0,2)*EC1qg[idx]*gamma1gq[idx]*LF3- 11./6.*pow(beta0,2)*EC1qg[idx]*gamma1gq[idx]*LQ3+ pow(beta0,2)*EC1qg[idx]*gamma1gq[idx]*LQ2*LF+ 1./2.*pow(beta0,2)*EC1qg[idx]*gamma1gq[idx]*LQ*LF2+ B2*gamma2qqb[idx]*LQ2- B2*gamma2qqb[idx]*LQ*LF - 1./2.*B2*gamma1qg[idx]*gamma1gq[idx]*LQ3+ B2*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF- 1./2.*B2*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+ B2*EC1qg[idx]*gamma1gq[idx]*LQ2- B2*EC1qg[idx]*gamma1gq[idx]*LQ*LF+ B1*gamma3qqb[idx]*LQ2- B1*gamma3qqb[idx]*LQ*LF+ B1*gamma2qg[idx]*gamma1gq[idx]*LQ2*LF- B1*gamma1qg[idx]*gamma2gq[idx]*LQ3+ B1*gamma1qg[idx]*gamma2gq[idx]*LQ2*LF- B1*gamma1qg[idx]*gamma2gq[idx]*LQ*LF2+ 1./6.*B1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4- 1./2.*B1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF+ 1./2.*B1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF2- 1./6.*B1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF3 - B1*gamma1qq[idx]*gamma2qqb[idx]*LQ3+ 3.*B1*gamma1qq[idx]*gamma2qqb[idx]*LQ2*LF- B1*gamma1qq[idx]*gamma2qqb[idx]*LQ*LF2+ 5./6.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4- 5./2.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF+ 5./2.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2- 5./6.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3+ B1*EC2qg[idx]*gamma1gq[idx]*LQ2- B1*EC2qg[idx]*gamma1gq[idx]*LQ*LF+ B1*EC1qg[idx]*gamma2gq[idx]*LQ2- B1*EC1qg[idx]*gamma2gq[idx]*LQ*LF- 1./2.*B1*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3+ B1*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF- 1./2.*B1*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF2- 3./2.*B1*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ3 + 3.*B1*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2*LF- 3./2.*B1*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ*LF2+ B1*EC2qqb[idx]*gamma1qq[idx]*LQ2- B1*EC2qqb[idx]*gamma1qq[idx]*LQ*LF+ 2.*B1*EC1qq[idx]*gamma2qqb[idx]*LQ2- 2.*B1*EC1qq[idx]*gamma2qqb[idx]*LQ*LF- B1*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+ 2.*B1*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF- B1*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+ B1*EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*LQ2- B1*EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*LQ*LF+ 3./2.*B1*beta0*gamma2qqb[idx]*LQ3- 1./2.*B1*beta0*gamma2qqb[idx]*LQ2*LF- B1*beta0*gamma2qqb[idx]*LQ*LF2- 3./4.*B1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ4 + B1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF+ 1./4.*B1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2- 1./2.*B1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3+ 2.*B1*beta0*EC1qg[idx]*gamma1gq[idx]*LQ3- 3./2.*B1*beta0*EC1qg[idx]*gamma1gq[idx]*LQ2*LF- 1./2.*B1*beta0*EC1qg[idx]*gamma1gq[idx]*LQ*LF2- 1./2.*pow(B1,2)*gamma2qqb[idx]*LQ3+ 1./2.*pow(B1,2)*gamma2qqb[idx]*LQ2*LF+ 1./4.*pow(B1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ4- 1./2.*pow(B1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF+ 1./4.*pow(B1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2- 1./2.*pow(B1,2)*EC1qg[idx]*gamma1gq[idx]*LQ3+ 1./2.*pow(B1,2)*EC1qg[idx]*gamma1gq[idx]*LQ2*LF+ 1./2.*A2*gamma2qqb[idx]*LQ3- 1./2.*A2*gamma2qqb[idx]*LQ2*LF - 1./4.*A2*gamma1qg[idx]*gamma1gq[idx]*LQ4+ 1./2.*A2*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF- 1./4.*A2*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2+ 1./2.*A2*EC1qg[idx]*gamma1gq[idx]*LQ3- 1./2.*A2*EC1qg[idx]*gamma1gq[idx]*LQ2*LF+ 1./2.*A1*gamma3qqb[idx]*LQ3- 1./2.*A1*gamma3qqb[idx]*LQ2*LF+ 1./2.*A1*gamma2qg[idx]*gamma1gq[idx]*LQ3*LF- 1./2.*A1*gamma1qg[idx]*gamma2gq[idx]*LQ4+ 1./2.*A1*gamma1qg[idx]*gamma2gq[idx]*LQ3*LF- 1./2.*A1*gamma1qg[idx]*gamma2gq[idx]*LQ2*LF2+ 1./12.*A1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ5- 1./4.*A1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4*LF+ 1./4.*A1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF2- 1./12.*A1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF3 - 1./2.*A1*gamma1qq[idx]*gamma2qqb[idx]*LQ4+ 3./2.*A1*gamma1qq[idx]*gamma2qqb[idx]*LQ3*LF- 1./2.*A1*gamma1qq[idx]*gamma2qqb[idx]*LQ2*LF2+ 5./12.*A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ5- 5./4.*A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4*LF+ 5./4.*A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF2- 5./12.*A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF3+ 1./2.*A1*EC2qg[idx]*gamma1gq[idx]*LQ3- 1./2.*A1*EC2qg[idx]*gamma1gq[idx]*LQ2*LF+ 1./2.*A1*EC1qg[idx]*gamma2gq[idx]*LQ3- 1./2.*A1*EC1qg[idx]*gamma2gq[idx]*LQ2*LF- 1./4.*A1*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4+ 1./2.*A1*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF- 1./4.*A1*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF2- 3./4.*A1*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ4 + 3./2.*A1*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ3*LF- 3./4.*A1*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2*LF2+ 1./2.*A1*EC2qqb[idx]*gamma1qq[idx]*LQ3- 1./2.*A1*EC2qqb[idx]*gamma1qq[idx]*LQ2*LF+ A1*EC1qq[idx]*gamma2qqb[idx]*LQ3- A1*EC1qq[idx]*gamma2qqb[idx]*LQ2*LF- 1./2.*A1*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4+ A1*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF- 1./2.*A1*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2+ 1./2.*A1*EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*LQ3- 1./2.*A1*EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*LQ2*LF+ 5./6.*A1*beta0*gamma2qqb[idx]*LQ4- 1./3.*A1*beta0*gamma2qqb[idx]*LQ3*LF- 1./2.*A1*beta0*gamma2qqb[idx]*LQ2*LF2- 5./12.*A1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ5 + 7./12.*A1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ4*LF+ 1./12.*A1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF2- 1./4.*A1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF3+ 13./12.*A1*beta0*EC1qg[idx]*gamma1gq[idx]*LQ4- 5./6.*A1*beta0*EC1qg[idx]*gamma1gq[idx]*LQ3*LF- 1./4.*A1*beta0*EC1qg[idx]*gamma1gq[idx]*LQ2*LF2- 1./2.*A1*B1*gamma2qqb[idx]*LQ4+ 1./2.*A1*B1*gamma2qqb[idx]*LQ3*LF+ 1./4.*A1*B1*gamma1qg[idx]*gamma1gq[idx]*LQ5- 1./2.*A1*B1*gamma1qg[idx]*gamma1gq[idx]*LQ4*LF+ 1./4.*A1*B1*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF2- 1./2.*A1*B1*EC1qg[idx]*gamma1gq[idx]*LQ4+ 1./2.*A1*B1*EC1qg[idx]*gamma1gq[idx]*LQ3*LF- 1./8.*pow(A1,2)*gamma2qqb[idx]*LQ5+ 1./8.*pow(A1,2)*gamma2qqb[idx]*LQ4*LF  + 1./16.*pow(A1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ6- 1./8.*pow(A1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ5*LF+ 1./16.*pow(A1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ4*LF2- 1./8.*pow(A1,2)*EC1qg[idx]*gamma1gq[idx]*LQ5+ 1./8.*pow(A1,2)*EC1qg[idx]*gamma1gq[idx]*LQ4*LF ;
	//complex <double> H4st_qqp   =+ gamma4qqbp[idx]*LF- gamma4qqbp[idx]*LQ+ gamma2qqb[idx]*gamma2qqp[idx]*LF2+ gamma2qqb[idx]*gamma2qqp[idx]*LQ2- 2.*gamma2qqb[idx]*gamma2qqp[idx]*LQ*LF+ 1./2.*gamma2qg[idx]*gamma1gq[idx]*LF2+ 1./2.*gamma2qg[idx]*gamma1gq[idx]*LQ2- gamma2qg[idx]*gamma1gq[idx]*LQ*LF+ gamma1qg[idx]*gamma3gq[idx]*LF2+ gamma1qg[idx]*gamma3gq[idx]*LQ2- gamma1qg[idx]*gamma3gq[idx]*LQ*LF+ 1./2.*gamma1qg[idx]*gamma2gq[idx]*gamma1gg[idx]*LF3- 1./2.*gamma1qg[idx]*gamma2gq[idx]*gamma1gg[idx]*LQ3+ 1./2.*gamma1qg[idx]*gamma2gq[idx]*gamma1gg[idx]*LQ2*LF - gamma1qg[idx]*gamma2gq[idx]*gamma1gg[idx]*LQ*LF2+ gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LF3- gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ3+ 2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ2*LF- 5./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ*LF2+ 1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LF3- 1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ3+ 1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ2*LF- gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ*LF2+ 1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LF3- 1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ3+ 1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ2*LF- gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ*LF2+ 1./24.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LF4+ 1./24.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LQ4 - 1./6.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)* LQ3*LF+ 1./4.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LQ2*LF2- 1./6.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LQ*LF3+ 1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LF4+ 1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ4- pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ3*LF+ 3./2.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2*LF2- pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF3+ 2.*gamma2qq[idx]*gamma2qqbp[idx]*LF2+ 2.*gamma2qq[idx]*gamma2qqbp[idx]*LQ2- 4.*gamma2qq[idx]*gamma2qqbp[idx]*LQ*LF+ gamma2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3- gamma2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+ 2.*gamma2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF - 5./2.*gamma2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+ 2.*gamma1qq[idx]*gamma3qqbp[idx]*LF2+ 2.*gamma1qq[idx]*gamma3qqbp[idx]*LQ2- 3.*gamma1qq[idx]*gamma3qqbp[idx]*LQ*LF+ gamma1qq[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ2*LF- gamma1qq[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ*LF2+ 3./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LF3- 3./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ3+ 5./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ2*LF- 3.*gamma1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ*LF2+ 1./4.*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF4+ 1./4.*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4- gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF+ 3./2.*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF2- gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF3 + pow(gamma1qq[idx],2)*gamma2qqbp[idx]*LF3- pow(gamma1qq[idx],2)*gamma2qqbp[idx]*LQ3+ 3.*pow(gamma1qq[idx],2)*gamma2qqbp[idx]*LQ2*LF- 7./2.*pow(gamma1qq[idx],2)*gamma2qqbp[idx]*LQ*LF2+ 17./24.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LF4+ 17./24.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ4- 17./6.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF+ 17./4.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2- 17./6.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3+ EC2qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LF2+ EC2qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2- 2.*EC2qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ*LF+ EC1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LF2+ EC1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ2- 2.*EC1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ*LF + EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LF3- EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ3+ 3.*EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ2*LF- 3.*EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ*LF2+ EC1qg[idx]*gamma2qq[idx]*gamma1gq[idx]*LF2+ EC1qg[idx]*gamma2qq[idx]*gamma1gq[idx]*LQ2- 2.*EC1qg[idx]*gamma2qq[idx]*gamma1gq[idx]*LQ*LF+ EC1qg[idx]*gamma1qq[idx]*gamma2gq[idx]*LF2+ EC1qg[idx]*gamma1qq[idx]*gamma2gq[idx]*LQ2- 2.*EC1qg[idx]*gamma1qq[idx]*gamma2gq[idx]*LQ*LF+ 1./2.*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*gamma1gg[idx]*LF3- 1./2.*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3+ 3./2.*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF- 3./2.*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF2+ EC1qg[idx]*pow(gamma1qq[idx],2)*gamma1gq[idx]*LF3 - EC1qg[idx]*pow(gamma1qq[idx],2)*gamma1gq[idx]*LQ3+ 3.*EC1qg[idx]*pow(gamma1qq[idx],2)*gamma1gq[idx]*LQ2*LF- 3.*EC1qg[idx]*pow(gamma1qq[idx],2)*gamma1gq[idx]*LQ*LF2+ pow(EC1qg[idx],2)*pow(gamma1gq[idx],2)*LF2+ pow(EC1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2- 2.*pow(EC1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF+ EC3qqbp[idx]*gamma1qq[idx]*LF- EC3qqbp[idx]*gamma1qq[idx]*LQ+ 1./2.*EC2qqbp[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+ 1./2.*EC2qqbp[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2- EC2qqbp[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+ EC2qqbp[idx]*gamma2qq[idx]*LF- EC2qqbp[idx]*gamma2qq[idx]*LQ+ 1./2.*EC2qqbp[idx]*pow(gamma1qq[idx],2)*LF2+ 1./2.*EC2qqbp[idx]*pow(gamma1qq[idx],2)*LQ2 - EC2qqbp[idx]*pow(gamma1qq[idx],2)*LQ*LF+ EC2qqbp[idx]*EC1qg[idx]*gamma1gq[idx]*LF- EC2qqbp[idx]*EC1qg[idx]*gamma1gq[idx]*LQ+ EC2qq[idx]*gamma2qqbp[idx]*LF- EC2qq[idx]*gamma2qqbp[idx]*LQ+ 1./2.*EC2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+ 1./2.*EC2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2- EC2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+ EC2qq[idx]*EC1qg[idx]*gamma1gq[idx]*LF- EC2qq[idx]*EC1qg[idx]*gamma1gq[idx]*LQ+ EC1qq[idx]*gamma3qqbp[idx]*LF- EC1qq[idx]*gamma3qqbp[idx]*LQ- EC1qq[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ*LF+ EC1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LF2+ EC1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ2 - EC1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ*LF+ 1./6.*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF3- 1./6.*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3+ 1./2.*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF- 1./2.*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF2+ 2.*EC1qq[idx]*gamma1qq[idx]*gamma2qqbp[idx]*LF2+ 2.*EC1qq[idx]*gamma1qq[idx]*gamma2qqbp[idx]*LQ2- 5.*EC1qq[idx]*gamma1qq[idx]*gamma2qqbp[idx]*LQ*LF+ 4./3.*EC1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3- 4./3.*EC1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+ 4.*EC1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF- 4.*EC1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+ EC1qq[idx]*EC2qg[idx]*gamma1gq[idx]*LF- EC1qq[idx]*EC2qg[idx]*gamma1gq[idx]*LQ+ EC1qq[idx]*EC1qg[idx]*gamma2gq[idx]*LF - EC1qq[idx]*EC1qg[idx]*gamma2gq[idx]*LQ+ 1./2.*EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF2+ 1./2.*EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2- EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF+ 3./2.*EC1qq[idx]*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LF2+ 3./2.*EC1qq[idx]*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2- 3.*EC1qq[idx]*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ*LF+ EC1qq[idx]*EC2qqbp[idx]*gamma1qq[idx]*LF- EC1qq[idx]*EC2qqbp[idx]*gamma1qq[idx]*LQ+ pow(EC1qq[idx],2)*gamma2qqbp[idx]*LF- pow(EC1qq[idx],2)*gamma2qqbp[idx]*LQ+ 1./2.*pow(EC1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LF2+ 1./2.*pow(EC1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ2- pow(EC1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+ 1./12.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LF4 + 1./12.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)* LQ4- 1./3.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ3*LF+ 1./2.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2*LF2- 1./3.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF3+ beta1*gamma2qqbp[idx]*LF2- beta1*gamma2qqbp[idx]*LQ2+ 1./2.*beta1*gamma1qg[idx]*gamma1gq[idx]*LF3+ 1./2.*beta1*gamma1qg[idx]*gamma1gq[idx]*LQ3- 1./2.*beta1*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF- 1./2.*beta1*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+ 1./2.*beta1*EC1qg[idx]*gamma1gq[idx]*LF2- 3./2.*beta1*EC1qg[idx]*gamma1gq[idx]*LQ2+ beta1*EC1qg[idx]*gamma1gq[idx]*LQ*LF+ 3./2.*beta0*gamma3qqbp[idx]*LF2 - 3./2.*beta0*gamma3qqbp[idx]*LQ2- beta0*gamma2qg[idx]*gamma1gq[idx]*LQ2*LF- 1./2.*beta0*gamma2qg[idx]*gamma1gq[idx]*LQ*LF2+ 3./2.*beta0*gamma1qg[idx]*gamma2gq[idx]*LF3+ 3./2.*beta0*gamma1qg[idx]*gamma2gq[idx]*LQ3- 1./2.*beta0*gamma1qg[idx]*gamma2gq[idx]*LQ2*LF- beta0*gamma1qg[idx]*gamma2gq[idx]*LQ*LF2+ 1./4.*beta0*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF4- 1./4.*beta0*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4+ 1./2.*beta0*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF- 1./2.*beta0*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF3+ 3./2.*beta0*gamma1qq[idx]*gamma2qqbp[idx]*LF3+ 3./2.*beta0*gamma1qq[idx]*gamma2qqbp[idx]*LQ3- 5./2.*beta0*gamma1qq[idx]*gamma2qqbp[idx]*LQ2*LF- 2.*beta0*gamma1qq[idx]*gamma2qqbp[idx]*LQ*LF2 + 5./4.*beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]* LF4- 5./4.*beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4+ 5./2.*beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF- 5./2.*beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3+ 1./2.*beta0*EC2qg[idx]*gamma1gq[idx]*LF2- 5./2.*beta0*EC2qg[idx]*gamma1gq[idx]*LQ2+ 2.*beta0*EC2qg[idx]*gamma1gq[idx]*LQ*LF+ beta0*EC1qg[idx]*gamma2gq[idx]*LF2- 2.*beta0*EC1qg[idx]*gamma2gq[idx]*LQ2+ beta0*EC1qg[idx]*gamma2gq[idx]*LQ*LF+ 1./2.*beta0*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF3+ beta0*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3- 3./2.*beta0*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF+ 3./2.*beta0*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LF3 + 3.*beta0*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ3- 9./2.*beta0*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2*LF+ 1./2.*beta0*EC2qqbp[idx]*gamma1qq[idx]*LF2- 5./2.*beta0*EC2qqbp[idx]*gamma1qq[idx]*LQ2+ 2.*beta0*EC2qqbp[idx]*gamma1qq[idx]*LQ*LF+ 2.*beta0*EC1qq[idx]*gamma2qqbp[idx]*LF2- 4.*beta0*EC1qq[idx]*gamma2qqbp[idx]*LQ2+ 2.*beta0*EC1qq[idx]*gamma2qqbp[idx]*LQ*LF+ beta0*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3+ 2.*beta0*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3- 3.*beta0*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF+ 1./2.*beta0*EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*LF2- 5./2.*beta0*EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*LQ2+ 2.*beta0*EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*LQ*LF+ pow(beta0,2)*gamma2qqbp[idx]*LF3 - pow(beta0,2)*gamma2qqbp[idx]*LQ3+ 11./24.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LF4+ 11./24.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LQ4- 1./3.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF- 1./4.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2- 1./3.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3+ 1./3.*pow(beta0,2)*EC1qg[idx]*gamma1gq[idx]*LF3- 11./6.*pow(beta0,2)*EC1qg[idx]*gamma1gq[idx]*LQ3+ pow(beta0,2)*EC1qg[idx]*gamma1gq[idx]*LQ2*LF+ 1./2.*pow(beta0,2)*EC1qg[idx]*gamma1gq[idx]*LQ*LF2+ B2*gamma2qqbp[idx]*LQ2- B2*gamma2qqbp[idx]*LQ*LF- 1./2.*B2*gamma1qg[idx]*gamma1gq[idx]*LQ3+ B2*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF- 1./2.*B2*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2 + B2*EC1qg[idx]*gamma1gq[idx]*LQ2- B2*EC1qg[idx]*gamma1gq[idx]*LQ*LF+ B1*gamma3qqbp[idx]*LQ2- B1*gamma3qqbp[idx]*LQ*LF+ B1*gamma2qg[idx]*gamma1gq[idx]*LQ2*LF- B1*gamma1qg[idx]*gamma2gq[idx]*LQ3+ B1*gamma1qg[idx]*gamma2gq[idx]*LQ2*LF- B1*gamma1qg[idx]*gamma2gq[idx]*LQ*LF2+ 1./6.*B1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4- 1./2.*B1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF+ 1./2.*B1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF2- 1./6.*B1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF3- B1*gamma1qq[idx]*gamma2qqbp[idx]*LQ3+ 3.*B1*gamma1qq[idx]*gamma2qqbp[idx]*LQ2*LF- B1*gamma1qq[idx]*gamma2qqbp[idx]*LQ*LF2 + 5./6.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4- 5./2.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF+ 5./2.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2- 5./6.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3+ B1*EC2qg[idx]*gamma1gq[idx]*LQ2- B1*EC2qg[idx]*gamma1gq[idx]*LQ*LF+ B1*EC1qg[idx]*gamma2gq[idx]*LQ2- B1*EC1qg[idx]*gamma2gq[idx]*LQ*LF- 1./2.*B1*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3+ B1*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF- 1./2.*B1*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF2- 3./2.*B1*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ3+ 3.*B1*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2*LF- 3./2.*B1*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ*LF2+ B1*EC2qqbp[idx]*gamma1qq[idx]*LQ2 - B1*EC2qqbp[idx]*gamma1qq[idx]*LQ*LF+ 2.*B1*EC1qq[idx]*gamma2qqbp[idx]*LQ2- 2.*B1*EC1qq[idx]*gamma2qqbp[idx]*LQ*LF- B1*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+ 2.*B1*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF- B1*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+ B1*EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*LQ2- B1*EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*LQ*LF+ 3./2.*B1*beta0*gamma2qqbp[idx]*LQ3- 1./2.*B1*beta0*gamma2qqbp[idx]*LQ2*LF- B1*beta0*gamma2qqbp[idx]*LQ*LF2- 3./4.*B1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ4+ B1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF+ 1./4.*B1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2- 1./2.*B1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3 + 2.*B1*beta0*EC1qg[idx]*gamma1gq[idx]*LQ3- 3./2.*B1*beta0*EC1qg[idx]*gamma1gq[idx]*LQ2*LF- 1./2.*B1*beta0*EC1qg[idx]*gamma1gq[idx]*LQ*LF2- 1./2.*pow(B1,2)*gamma2qqbp[idx]*LQ3+ 1./2.*pow(B1,2)*gamma2qqbp[idx]*LQ2*LF+ 1./4.*pow(B1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ4- 1./2.*pow(B1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF+ 1./4.*pow(B1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2- 1./2.*pow(B1,2)*EC1qg[idx]*gamma1gq[idx]*LQ3+ 1./2.*pow(B1,2)*EC1qg[idx]*gamma1gq[idx]*LQ2*LF+ 1./2.*A2*gamma2qqbp[idx]*LQ3- 1./2.*A2*gamma2qqbp[idx]*LQ2*LF- 1./4.*A2*gamma1qg[idx]*gamma1gq[idx]*LQ4+ 1./2.*A2*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF- 1./4.*A2*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2 + 1./2.*A2*EC1qg[idx]*gamma1gq[idx]*LQ3- 1./2.*A2*EC1qg[idx]*gamma1gq[idx]*LQ2*LF+ 1./2.*A1*gamma3qqbp[idx]*LQ3- 1./2.*A1*gamma3qqbp[idx]*LQ2*LF+ 1./2.*A1*gamma2qg[idx]*gamma1gq[idx]*LQ3*LF- 1./2.*A1*gamma1qg[idx]*gamma2gq[idx]*LQ4+ 1./2.*A1*gamma1qg[idx]*gamma2gq[idx]*LQ3*LF- 1./2.*A1*gamma1qg[idx]*gamma2gq[idx]*LQ2*LF2+ 1./12.*A1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ5- 1./4.*A1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4*LF+ 1./4.*A1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF2- 1./12.*A1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF3- 1./2.*A1*gamma1qq[idx]*gamma2qqbp[idx]*LQ4+ 3./2.*A1*gamma1qq[idx]*gamma2qqbp[idx]*LQ3*LF- 1./2.*A1*gamma1qq[idx]*gamma2qqbp[idx]*LQ2*LF2 + 5./12.*A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ5- 5./4.*A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4*LF+ 5./4.*A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF2- 5./12.*A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF3+ 1./2.*A1*EC2qg[idx]*gamma1gq[idx]*LQ3- 1./2.*A1*EC2qg[idx]*gamma1gq[idx]*LQ2*LF+ 1./2.*A1*EC1qg[idx]*gamma2gq[idx]*LQ3- 1./2.*A1*EC1qg[idx]*gamma2gq[idx]*LQ2*LF- 1./4.*A1*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4+ 1./2.*A1*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF- 1./4.*A1*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF2- 3./4.*A1*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ4+ 3./2.*A1*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ3*LF- 3./4.*A1*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2*LF2+ 1./2.*A1*EC2qqbp[idx]*gamma1qq[idx]*LQ3 - 1./2.*A1*EC2qqbp[idx]*gamma1qq[idx]*LQ2*LF+ A1*EC1qq[idx]*gamma2qqbp[idx]*LQ3- A1*EC1qq[idx]*gamma2qqbp[idx]*LQ2*LF- 1./2.*A1*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4+ A1*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF- 1./2.*A1*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2+ 1./2.*A1*EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*LQ3- 1./2.*A1*EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*LQ2*LF+ 5./6.*A1*beta0*gamma2qqbp[idx]*LQ4- 1./3.*A1*beta0*gamma2qqbp[idx]*LQ3*LF- 1./2.*A1*beta0*gamma2qqbp[idx]*LQ2*LF2- 5./12.*A1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ5+ 7./12.*A1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ4*LF+ 1./12.*A1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF2- 1./4.*A1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF3 + 13./12.*A1*beta0*EC1qg[idx]*gamma1gq[idx]*LQ4- 5./6.*A1*beta0*EC1qg[idx]*gamma1gq[idx]*LQ3*LF- 1./4.*A1*beta0*EC1qg[idx]*gamma1gq[idx]*LQ2*LF2- 1./2.*A1*B1*gamma2qqbp[idx]*LQ4+ 1./2.*A1*B1*gamma2qqbp[idx]*LQ3*LF+ 1./4.*A1*B1*gamma1qg[idx]*gamma1gq[idx]*LQ5- 1./2.*A1*B1*gamma1qg[idx]*gamma1gq[idx]*LQ4*LF+ 1./4.*A1*B1*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF2- 1./2.*A1*B1*EC1qg[idx]*gamma1gq[idx]*LQ4+ 1./2.*A1*B1*EC1qg[idx]*gamma1gq[idx]*LQ3*LF- 1./8.*pow(A1,2)*gamma2qqbp[idx]*LQ5+ 1./8.*pow(A1,2)*gamma2qqbp[idx]*LQ4*LF+ 1./16.*pow(A1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ6- 1./8.*pow(A1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ5*LF+ 1./16.*pow(A1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ4*LF2 - 1./8.*pow(A1,2)*EC1qg[idx]*gamma1gq[idx]*LQ5+ 1./8.*pow(A1,2)*EC1qg[idx]*gamma1gq[idx]*LQ4*LF ;
	//complex <double> H4st_qqbp  =+ gamma4qqp[idx]*LF- gamma4qqp[idx]*LQ+ gamma2qqb[idx]*gamma2qqbp[idx]*LF2+ gamma2qqb[idx]*gamma2qqbp[idx]*LQ2- 2.*gamma2qqb[idx]*gamma2qqbp[idx]*LQ*LF+ 1./2.*gamma2qg[idx]*gamma1gq[idx]*LF2+ 1./2.*gamma2qg[idx]*gamma1gq[idx]*LQ2- gamma2qg[idx]*gamma1gq[idx]*LQ*LF+ gamma1qg[idx]*gamma3gq[idx]*LF2+ gamma1qg[idx]*gamma3gq[idx]*LQ2- gamma1qg[idx]*gamma3gq[idx]*LQ*LF+ 1./2.*gamma1qg[idx]*gamma2gq[idx]*gamma1gg[idx]*LF3- 1./2.*gamma1qg[idx]*gamma2gq[idx]*gamma1gg[idx]*LQ3+ 1./2.*gamma1qg[idx]*gamma2gq[idx]*gamma1gg[idx]*LQ2*LF - gamma1qg[idx]*gamma2gq[idx]*gamma1gg[idx]*LQ*LF2+ 1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LF3- 1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ3+ 1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ2*LF- gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ*LF2+ gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LF3- gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ3+ 2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ2*LF- 5./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ*LF2+ 1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LF3- 1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ3+ 1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ2*LF- gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ*LF2+ 1./24.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LF4+ 1./24.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LQ4 - 1./6.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)* LQ3*LF+ 1./4.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LQ2*LF2- 1./6.*gamma1qg[idx]*gamma1gq[idx]*pow(gamma1gg[idx],2)*LQ*LF3+ 1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LF4+ 1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ4- pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ3*LF+ 3./2.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2*LF2- pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF3+ 2.*gamma2qq[idx]*gamma2qqp[idx]*LF2+ 2.*gamma2qq[idx]*gamma2qqp[idx]*LQ2- 4.*gamma2qq[idx]*gamma2qqp[idx]*LQ*LF+ gamma2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3- gamma2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+ 2.*gamma2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF - 5./2.*gamma2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+ 2.*gamma1qq[idx]*gamma3qqp[idx]*LF2+ 2.*gamma1qq[idx]*gamma3qqp[idx]*LQ2- 3.*gamma1qq[idx]*gamma3qqp[idx]*LQ*LF+ gamma1qq[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ2*LF- gamma1qq[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ*LF2+ 3./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LF3- 3./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ3+ 5./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ2*LF- 3.*gamma1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ*LF2+ 1./4.*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF4+ 1./4.*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4- gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF+ 3./2.*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF2- gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF3 + pow(gamma1qq[idx],2)*gamma2qqp[idx]*LF3- pow(gamma1qq[idx],2)*gamma2qqp[idx]*LQ3+ 3.*pow(gamma1qq[idx],2)*gamma2qqp[idx]*LQ2*LF- 7./2.*pow(gamma1qq[idx],2)*gamma2qqp[idx]*LQ*LF2+ 17./24.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LF4+ 17./24.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ4- 17./6.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF+ 17./4.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2- 17./6.*pow(gamma1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3+ EC2qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LF2+ EC2qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2- 2.*EC2qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ*LF+ EC1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LF2+ EC1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ2- 2.*EC1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ*LF + EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LF3- EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ3+ 3.*EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ2*LF- 3.*EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ*LF2+ EC1qg[idx]*gamma2qq[idx]*gamma1gq[idx]*LF2+ EC1qg[idx]*gamma2qq[idx]*gamma1gq[idx]*LQ2- 2.*EC1qg[idx]*gamma2qq[idx]*gamma1gq[idx]*LQ*LF+ EC1qg[idx]*gamma1qq[idx]*gamma2gq[idx]*LF2+ EC1qg[idx]*gamma1qq[idx]*gamma2gq[idx]*LQ2- 2.*EC1qg[idx]*gamma1qq[idx]*gamma2gq[idx]*LQ*LF+ 1./2.*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*gamma1gg[idx]*LF3- 1./2.*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3+ 3./2.*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF- 3./2.*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF2+ EC1qg[idx]*pow(gamma1qq[idx],2)*gamma1gq[idx]*LF3 - EC1qg[idx]*pow(gamma1qq[idx],2)*gamma1gq[idx]*LQ3+ 3.*EC1qg[idx]*pow(gamma1qq[idx],2)*gamma1gq[idx]*LQ2*LF- 3.*EC1qg[idx]*pow(gamma1qq[idx],2)*gamma1gq[idx]*LQ*LF2+ pow(EC1qg[idx],2)*pow(gamma1gq[idx],2)*LF2+ pow(EC1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2- 2.*pow(EC1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF+ EC3qqp[idx]*gamma1qq[idx]*LF- EC3qqp[idx]*gamma1qq[idx]*LQ+ 1./2.*EC2qqp[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+ 1./2.*EC2qqp[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2- EC2qqp[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+ EC2qqp[idx]*gamma2qq[idx]*LF- EC2qqp[idx]*gamma2qq[idx]*LQ+ 1./2.*EC2qqp[idx]*pow(gamma1qq[idx],2)*LF2+ 1./2.*EC2qqp[idx]*pow(gamma1qq[idx],2)*LQ2 - EC2qqp[idx]*pow(gamma1qq[idx],2)*LQ*LF+ EC2qqp[idx]*EC1qg[idx]*gamma1gq[idx]*LF- EC2qqp[idx]*EC1qg[idx]*gamma1gq[idx]*LQ+ EC2qq[idx]*gamma2qqp[idx]*LF- EC2qq[idx]*gamma2qqp[idx]*LQ+ 1./2.*EC2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+ 1./2.*EC2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2- EC2qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+ EC2qq[idx]*EC1qg[idx]*gamma1gq[idx]*LF- EC2qq[idx]*EC1qg[idx]*gamma1gq[idx]*LQ+ EC1qq[idx]*gamma3qqp[idx]*LF- EC1qq[idx]*gamma3qqp[idx]*LQ- EC1qq[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ*LF+ EC1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LF2+ EC1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ2 - EC1qq[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ*LF+ 1./6.*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF3- 1./6.*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3+ 1./2.*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF- 1./2.*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF2+ 2.*EC1qq[idx]*gamma1qq[idx]*gamma2qqp[idx]*LF2+ 2.*EC1qq[idx]*gamma1qq[idx]*gamma2qqp[idx]*LQ2- 5.*EC1qq[idx]*gamma1qq[idx]*gamma2qqp[idx]*LQ*LF+ 4./3.*EC1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3- 4./3.*EC1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+ 4.*EC1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF- 4.*EC1qq[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+ EC1qq[idx]*EC2qg[idx]*gamma1gq[idx]*LF- EC1qq[idx]*EC2qg[idx]*gamma1gq[idx]*LQ+ EC1qq[idx]*EC1qg[idx]*gamma2gq[idx]*LF - EC1qq[idx]*EC1qg[idx]*gamma2gq[idx]*LQ+ 1./2.*EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF2+ 1./2.*EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2- EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF+ 3./2.*EC1qq[idx]*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LF2+ 3./2.*EC1qq[idx]*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2- 3.*EC1qq[idx]*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ*LF+ EC1qq[idx]*EC2qqp[idx]*gamma1qq[idx]*LF- EC1qq[idx]*EC2qqp[idx]*gamma1qq[idx]*LQ+ pow(EC1qq[idx],2)*gamma2qqp[idx]*LF- pow(EC1qq[idx],2)*gamma2qqp[idx]*LQ+ 1./2.*pow(EC1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LF2+ 1./2.*pow(EC1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ2- pow(EC1qq[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+ 1./12.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LF4 + 1./12.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)* LQ4- 1./3.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ3*LF+ 1./2.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2*LF2- 1./3.*nf*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF3+ beta1*gamma2qqp[idx]*LF2- beta1*gamma2qqp[idx]*LQ2+ 1./2.*beta1*gamma1qg[idx]*gamma1gq[idx]*LF3+ 1./2.*beta1*gamma1qg[idx]*gamma1gq[idx]*LQ3- 1./2.*beta1*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF- 1./2.*beta1*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+ 1./2.*beta1*EC1qg[idx]*gamma1gq[idx]*LF2- 3./2.*beta1*EC1qg[idx]*gamma1gq[idx]*LQ2+ beta1*EC1qg[idx]*gamma1gq[idx]*LQ*LF+ 3./2.*beta0*gamma3qqp[idx]*LF2 - 3./2.*beta0*gamma3qqp[idx]*LQ2- beta0*gamma2qg[idx]*gamma1gq[idx]*LQ2*LF- 1./2.*beta0*gamma2qg[idx]*gamma1gq[idx]*LQ*LF2+ 3./2.*beta0*gamma1qg[idx]*gamma2gq[idx]*LF3+ 3./2.*beta0*gamma1qg[idx]*gamma2gq[idx]*LQ3- 1./2.*beta0*gamma1qg[idx]*gamma2gq[idx]*LQ2*LF- beta0*gamma1qg[idx]*gamma2gq[idx]*LQ*LF2+ 1./4.*beta0*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF4- 1./4.*beta0*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4+ 1./2.*beta0*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF- 1./2.*beta0*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF3+ 3./2.*beta0*gamma1qq[idx]*gamma2qqp[idx]*LF3+ 3./2.*beta0*gamma1qq[idx]*gamma2qqp[idx]*LQ3- 5./2.*beta0*gamma1qq[idx]*gamma2qqp[idx]*LQ2*LF- 2.*beta0*gamma1qq[idx]*gamma2qqp[idx]*LQ*LF2 + 5./4.*beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]* LF4- 5./4.*beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4+ 5./2.*beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF- 5./2.*beta0*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3+ 1./2.*beta0*EC2qg[idx]*gamma1gq[idx]*LF2- 5./2.*beta0*EC2qg[idx]*gamma1gq[idx]*LQ2+ 2.*beta0*EC2qg[idx]*gamma1gq[idx]*LQ*LF+ beta0*EC1qg[idx]*gamma2gq[idx]*LF2- 2.*beta0*EC1qg[idx]*gamma2gq[idx]*LQ2+ beta0*EC1qg[idx]*gamma2gq[idx]*LQ*LF+ 1./2.*beta0*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF3+ beta0*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3- 3./2.*beta0*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF+ 3./2.*beta0*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LF3 + 3.*beta0*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ3- 9./2.*beta0*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2*LF+ 1./2.*beta0*EC2qqp[idx]*gamma1qq[idx]*LF2- 5./2.*beta0*EC2qqp[idx]*gamma1qq[idx]*LQ2+ 2.*beta0*EC2qqp[idx]*gamma1qq[idx]*LQ*LF+ 2.*beta0*EC1qq[idx]*gamma2qqp[idx]*LF2- 4.*beta0*EC1qq[idx]*gamma2qqp[idx]*LQ2+ 2.*beta0*EC1qq[idx]*gamma2qqp[idx]*LQ*LF+ beta0*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3+ 2.*beta0*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3- 3.*beta0*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF+ 1./2.*beta0*EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*LF2- 5./2.*beta0*EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*LQ2+ 2.*beta0*EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*LQ*LF+ pow(beta0,2)*gamma2qqp[idx]*LF3 - pow(beta0,2)*gamma2qqp[idx]*LQ3+ 11./24.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LF4+ 11./24.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LQ4- 1./3.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF- 1./4.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2- 1./3.*pow(beta0,2)*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3+ 1./3.*pow(beta0,2)*EC1qg[idx]*gamma1gq[idx]*LF3- 11./6.*pow(beta0,2)*EC1qg[idx]*gamma1gq[idx]*LQ3+ pow(beta0,2)*EC1qg[idx]*gamma1gq[idx]*LQ2*LF+ 1./2.*pow(beta0,2)*EC1qg[idx]*gamma1gq[idx]*LQ*LF2+ B2*gamma2qqp[idx]*LQ2- B2*gamma2qqp[idx]*LQ*LF- 1./2.*B2*gamma1qg[idx]*gamma1gq[idx]*LQ3+ B2*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF- 1./2.*B2*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2 + B2*EC1qg[idx]*gamma1gq[idx]*LQ2- B2*EC1qg[idx]*gamma1gq[idx]*LQ*LF+ B1*gamma3qqp[idx]*LQ2- B1*gamma3qqp[idx]*LQ*LF+ B1*gamma2qg[idx]*gamma1gq[idx]*LQ2*LF- B1*gamma1qg[idx]*gamma2gq[idx]*LQ3+ B1*gamma1qg[idx]*gamma2gq[idx]*LQ2*LF- B1*gamma1qg[idx]*gamma2gq[idx]*LQ*LF2+ 1./6.*B1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4- 1./2.*B1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF+ 1./2.*B1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF2- 1./6.*B1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF3- B1*gamma1qq[idx]*gamma2qqp[idx]*LQ3+ 3.*B1*gamma1qq[idx]*gamma2qqp[idx]*LQ2*LF- B1*gamma1qq[idx]*gamma2qqp[idx]*LQ*LF2 + 5./6.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4- 5./2.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF+ 5./2.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2- 5./6.*B1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3+ B1*EC2qg[idx]*gamma1gq[idx]*LQ2- B1*EC2qg[idx]*gamma1gq[idx]*LQ*LF+ B1*EC1qg[idx]*gamma2gq[idx]*LQ2- B1*EC1qg[idx]*gamma2gq[idx]*LQ*LF- 1./2.*B1*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3+ B1*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF- 1./2.*B1*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF2- 3./2.*B1*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ3+ 3.*B1*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2*LF- 3./2.*B1*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ*LF2+ B1*EC2qqp[idx]*gamma1qq[idx]*LQ2 - B1*EC2qqp[idx]*gamma1qq[idx]*LQ*LF+ 2.*B1*EC1qq[idx]*gamma2qqp[idx]*LQ2- 2.*B1*EC1qq[idx]*gamma2qqp[idx]*LQ*LF- B1*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+ 2.*B1*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF- B1*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+ B1*EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*LQ2- B1*EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*LQ*LF+ 3./2.*B1*beta0*gamma2qqp[idx]*LQ3- 1./2.*B1*beta0*gamma2qqp[idx]*LQ2*LF- B1*beta0*gamma2qqp[idx]*LQ*LF2- 3./4.*B1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ4+ B1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF+ 1./4.*B1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2- 1./2.*B1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ*LF3 + 2.*B1*beta0*EC1qg[idx]*gamma1gq[idx]*LQ3- 3./2.*B1*beta0*EC1qg[idx]*gamma1gq[idx]*LQ2*LF- 1./2.*B1*beta0*EC1qg[idx]*gamma1gq[idx]*LQ*LF2- 1./2.*pow(B1,2)*gamma2qqp[idx]*LQ3+ 1./2.*pow(B1,2)*gamma2qqp[idx]*LQ2*LF+ 1./4.*pow(B1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ4- 1./2.*pow(B1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF+ 1./4.*pow(B1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2- 1./2.*pow(B1,2)*EC1qg[idx]*gamma1gq[idx]*LQ3+ 1./2.*pow(B1,2)*EC1qg[idx]*gamma1gq[idx]*LQ2*LF+ 1./2.*A2*gamma2qqp[idx]*LQ3- 1./2.*A2*gamma2qqp[idx]*LQ2*LF- 1./4.*A2*gamma1qg[idx]*gamma1gq[idx]*LQ4+ 1./2.*A2*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF- 1./4.*A2*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2 + 1./2.*A2*EC1qg[idx]*gamma1gq[idx]*LQ3- 1./2.*A2*EC1qg[idx]*gamma1gq[idx]*LQ2*LF+ 1./2.*A1*gamma3qqp[idx]*LQ3- 1./2.*A1*gamma3qqp[idx]*LQ2*LF+ 1./2.*A1*gamma2qg[idx]*gamma1gq[idx]*LQ3*LF- 1./2.*A1*gamma1qg[idx]*gamma2gq[idx]*LQ4+ 1./2.*A1*gamma1qg[idx]*gamma2gq[idx]*LQ3*LF- 1./2.*A1*gamma1qg[idx]*gamma2gq[idx]*LQ2*LF2+ 1./12.*A1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ5- 1./4.*A1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4*LF+ 1./4.*A1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF2- 1./12.*A1*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF3- 1./2.*A1*gamma1qq[idx]*gamma2qqp[idx]*LQ4+ 3./2.*A1*gamma1qq[idx]*gamma2qqp[idx]*LQ3*LF- 1./2.*A1*gamma1qq[idx]*gamma2qqp[idx]*LQ2*LF2 + 5./12.*A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ5- 5./4.*A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4*LF+ 5./4.*A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF2- 5./12.*A1*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF3+ 1./2.*A1*EC2qg[idx]*gamma1gq[idx]*LQ3- 1./2.*A1*EC2qg[idx]*gamma1gq[idx]*LQ2*LF+ 1./2.*A1*EC1qg[idx]*gamma2gq[idx]*LQ3- 1./2.*A1*EC1qg[idx]*gamma2gq[idx]*LQ2*LF- 1./4.*A1*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ4+ 1./2.*A1*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF- 1./4.*A1*EC1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF2- 3./4.*A1*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ4+ 3./2.*A1*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ3*LF- 3./4.*A1*EC1qg[idx]*gamma1qq[idx]*gamma1gq[idx]*LQ2*LF2+ 1./2.*A1*EC2qqp[idx]*gamma1qq[idx]*LQ3 - 1./2.*A1*EC2qqp[idx]*gamma1qq[idx]*LQ2*LF+ A1*EC1qq[idx]*gamma2qqp[idx]*LQ3- A1*EC1qq[idx]*gamma2qqp[idx]*LQ2*LF- 1./2.*A1*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4+ A1*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF- 1./2.*A1*EC1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2+ 1./2.*A1*EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*LQ3- 1./2.*A1*EC1qq[idx]*EC1qg[idx]*gamma1gq[idx]*LQ2*LF+ 5./6.*A1*beta0*gamma2qqp[idx]*LQ4- 1./3.*A1*beta0*gamma2qqp[idx]*LQ3*LF- 1./2.*A1*beta0*gamma2qqp[idx]*LQ2*LF2- 5./12.*A1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ5+ 7./12.*A1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ4*LF+ 1./12.*A1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF2- 1./4.*A1*beta0*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF3 + 13./12.*A1*beta0*EC1qg[idx]*gamma1gq[idx]*LQ4- 5./6.*A1*beta0*EC1qg[idx]*gamma1gq[idx]*LQ3*LF- 1./4.*A1*beta0*EC1qg[idx]*gamma1gq[idx]*LQ2*LF2- 1./2.*A1*B1*gamma2qqp[idx]*LQ4+ 1./2.*A1*B1*gamma2qqp[idx]*LQ3*LF+ 1./4.*A1*B1*gamma1qg[idx]*gamma1gq[idx]*LQ5- 1./2.*A1*B1*gamma1qg[idx]*gamma1gq[idx]*LQ4*LF+ 1./4.*A1*B1*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF2- 1./2.*A1*B1*EC1qg[idx]*gamma1gq[idx]*LQ4+ 1./2.*A1*B1*EC1qg[idx]*gamma1gq[idx]*LQ3*LF- 1./8.*pow(A1,2)*gamma2qqp[idx]*LQ5+ 1./8.*pow(A1,2)*gamma2qqp[idx]*LQ4*LF+ 1./16.*pow(A1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ6- 1./8.*pow(A1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ5*LF+ 1./16.*pow(A1,2)*gamma1qg[idx]*gamma1gq[idx]*LQ4*LF2 - 1./8.*pow(A1,2)*EC1qg[idx]*gamma1gq[idx]*LQ5+ 1./8.*pow(A1,2)*EC1qg[idx]*gamma1gq[idx]*LQ4*LF ;
	//complex <double> H4st_gg    =+ pow(gamma2qg[idx],2)*LF2+ pow(gamma2qg[idx],2)*LQ2- 2.*pow(gamma2qg[idx],2)*LQ*LF+ 2.*gamma1qg[idx]*gamma3qg[idx]*LF2+ 2.*gamma1qg[idx]*gamma3qg[idx]*LQ2- 4.*gamma1qg[idx]*gamma3qg[idx]*LQ*LF+ gamma1qg[idx]*gamma2qg[idx]*gamma1gg[idx]*LF3- gamma1qg[idx]*gamma2qg[idx]*gamma1gg[idx]*LQ3+ 5.*gamma1qg[idx]*gamma2qg[idx]*gamma1gg[idx]*LQ2*LF- 5.*gamma1qg[idx]*gamma2qg[idx]*gamma1gg[idx]*LQ*LF2+ 2.*pow(gamma1qg[idx],2)*gamma2qqbp[idx]*LQ2*LF- 2.*pow(gamma1qg[idx],2)*gamma2qqbp[idx]*LQ*LF2+ 2.*pow(gamma1qg[idx],2)*gamma2qqp[idx]*LQ2*LF- 2.*pow(gamma1qg[idx],2)*gamma2qqp[idx]*LQ*LF2 + 2.*pow(gamma1qg[idx],2)*gamma2qqb[idx]*LQ2*LF- 2.*pow(gamma1qg[idx],2)*gamma2qqb[idx]*LQ*LF2+ 2.*pow(gamma1qg[idx],2)*gamma2gg[idx]*LF3- 2.*pow(gamma1qg[idx],2)*gamma2gg[idx]*LQ3+ 4.*pow(gamma1qg[idx],2)*gamma2gg[idx]*LQ2*LF- 4.*pow(gamma1qg[idx],2)*gamma2gg[idx]*LQ*LF2+ 7./12.*pow(gamma1qg[idx],2)*pow(gamma1gg[idx],2)*LF4+ 7./12.*pow(gamma1qg[idx],2)*pow(gamma1gg[idx],2)*LQ4- 7./3.*pow(gamma1qg[idx],2)*pow(gamma1gg[idx],2)*LQ3*LF+ 7./2.*pow(gamma1qg[idx],2)*pow(gamma1gg[idx],2)*LQ2*LF2- 7./3.*pow(gamma1qg[idx],2)*pow(gamma1gg[idx],2)*LQ*LF3+ 2.*gamma2qq[idx]*pow(gamma1qg[idx],2)*LQ2*LF- 2.*gamma2qq[idx]*pow(gamma1qg[idx],2)*LQ*LF2+ 3.*gamma1qq[idx]*gamma1qg[idx]*gamma2qg[idx]*LF3- 3.*gamma1qq[idx]*gamma1qg[idx]*gamma2qg[idx]*LQ3 + 7.*gamma1qq[idx]*gamma1qg[idx]*gamma2qg[idx]*LQ2*LF- 7.*gamma1qq[idx]*gamma1qg[idx]*gamma2qg[idx]*LQ*LF2+ 5./6.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gg[idx]*LF4+ 5./6.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gg[idx]*LQ4- 10./3.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gg[idx]*LQ3*LF+ 5.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gg[idx]*LQ2*LF2- 10./3.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gg[idx]*LQ*LF3+ 7./12.*pow(gamma1qq[idx],2)*pow(gamma1qg[idx],2)*LF4+ 7./12.*pow(gamma1qq[idx],2)*pow(gamma1qg[idx],2)*LQ4- 7./3.*pow(gamma1qq[idx],2)*pow(gamma1qg[idx],2)*LQ3*LF+ 7./2.*pow(gamma1qq[idx],2)*pow(gamma1qg[idx],2)*LQ2*LF2- 7./3.*pow(gamma1qq[idx],2)*pow(gamma1qg[idx],2)*LQ*LF3+ 2.*EC3qg[idx]*gamma1qg[idx]*LF- 2.*EC3qg[idx]*gamma1qg[idx]*LQ+ 2.*EC2qg[idx]*gamma2qg[idx]*LF - 2.*EC2qg[idx]*gamma2qg[idx]*LQ+ 3.*EC2qg[idx]*gamma1qg[idx]*gamma1gg[idx]*LF2+ 3.*EC2qg[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ2- 6.*EC2qg[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ*LF+ EC2qg[idx]*gamma1qq[idx]*gamma1qg[idx]*LF2+ EC2qg[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ2- 2.*EC2qg[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ*LF+ 2.*EC1qg[idx]*gamma3qg[idx]*LF- 2.*EC1qg[idx]*gamma3qg[idx]*LQ+ 2.*EC1qg[idx]*gamma2qg[idx]*gamma1gg[idx]*LF2+ 2.*EC1qg[idx]*gamma2qg[idx]*gamma1gg[idx]*LQ2- 6.*EC1qg[idx]*gamma2qg[idx]*gamma1gg[idx]*LQ*LF- 2.*EC1qg[idx]*gamma1qg[idx]*gamma2qqbp[idx]*LQ*LF- 2.*EC1qg[idx]*gamma1qg[idx]*gamma2qqp[idx]*LQ*LF- 2.*EC1qg[idx]*gamma1qg[idx]*gamma2qqb[idx]*LQ*LF + 4.*EC1qg[idx]*gamma1qg[idx]*gamma2gg[idx]*LF2+ 4.*EC1qg[idx]*gamma1qg[idx]*gamma2gg[idx]*LQ2- 6.*EC1qg[idx]*gamma1qg[idx]*gamma2gg[idx]*LQ*LF+ 7./3.*EC1qg[idx]*gamma1qg[idx]*pow(gamma1gg[idx],2)*LF3- 7./3.*EC1qg[idx]*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ3+ 7.*EC1qg[idx]*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ2*LF- 7.*EC1qg[idx]*gamma1qg[idx]*pow(gamma1gg[idx],2)*LQ*LF2- 2.*EC1qg[idx]*gamma2qq[idx]*gamma1qg[idx]*LQ*LF+ 2.*EC1qg[idx]*gamma1qq[idx]*gamma2qg[idx]*LF2+ 2.*EC1qg[idx]*gamma1qq[idx]*gamma2qg[idx]*LQ2- 2.*EC1qg[idx]*gamma1qq[idx]*gamma2qg[idx]*LQ*LF+ 4./3.*EC1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LF3- 4./3.*EC1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ3+ 4.*EC1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF- 4.*EC1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ*LF2 + 1./3.*EC1qg[idx]*pow(gamma1qq[idx],2)*gamma1qg[idx]*LF3- 1./3.*EC1qg[idx]*pow(gamma1qq[idx],2)*gamma1qg[idx]*LQ3+ EC1qg[idx]*pow(gamma1qq[idx],2)*gamma1qg[idx]*LQ2*LF- EC1qg[idx]*pow(gamma1qq[idx],2)*gamma1qg[idx]*LQ*LF2+ 4.*EC1qg[idx]*EC2qg[idx]*gamma1gg[idx]*LF- 4.*EC1qg[idx]*EC2qg[idx]*gamma1gg[idx]*LQ+ 2.*pow(EC1qg[idx],2)*gamma2gg[idx]*LF- 2.*pow(EC1qg[idx],2)*gamma2gg[idx]*LQ+ 2.*pow(EC1qg[idx],2)*pow(gamma1gg[idx],2)*LF2+ 2.*pow(EC1qg[idx],2)*pow(gamma1gg[idx],2)*LQ2- 4.*pow(EC1qg[idx],2)*pow(gamma1gg[idx],2)*LQ*LF+ 2.*EC2qqbp[idx]*pow(gamma1qg[idx],2)*LF2+ 2.*EC2qqbp[idx]*pow(gamma1qg[idx],2)*LQ2- 4.*EC2qqbp[idx]*pow(gamma1qg[idx],2)*LQ*LF+ 2.*EC2qqbp[idx]*EC1qg[idx]*gamma1qg[idx]*LF - 2.*EC2qqbp[idx]*EC1qg[idx]*gamma1qg[idx]*LQ+ 2.*EC2qqp[idx]*pow(gamma1qg[idx],2)*LF2+ 2.*EC2qqp[idx]*pow(gamma1qg[idx],2)*LQ2- 4.*EC2qqp[idx]*pow(gamma1qg[idx],2)*LQ*LF+ 2.*EC2qqp[idx]*EC1qg[idx]*gamma1qg[idx]*LF- 2.*EC2qqp[idx]*EC1qg[idx]*gamma1qg[idx]*LQ+ 2.*EC2qqb[idx]*pow(gamma1qg[idx],2)*LF2+ 2.*EC2qqb[idx]*pow(gamma1qg[idx],2)*LQ2- 4.*EC2qqb[idx]*pow(gamma1qg[idx],2)*LQ*LF+ 2.*EC2qqb[idx]*EC1qg[idx]*gamma1qg[idx]*LF- 2.*EC2qqb[idx]*EC1qg[idx]*gamma1qg[idx]*LQ+ 2.*EC2qq[idx]*pow(gamma1qg[idx],2)*LF2+ 2.*EC2qq[idx]*pow(gamma1qg[idx],2)*LQ2- 4.*EC2qq[idx]*pow(gamma1qg[idx],2)*LQ*LF+ 2.*EC2qq[idx]*EC1qg[idx]*gamma1qg[idx]*LF - 2.*EC2qq[idx]*EC1qg[idx]*gamma1qg[idx]*LQ+ 4.*EC1qq[idx]*gamma1qg[idx]*gamma2qg[idx]*LF2+ 4.*EC1qq[idx]*gamma1qg[idx]*gamma2qg[idx]*LQ2- 8.*EC1qq[idx]*gamma1qg[idx]*gamma2qg[idx]*LQ*LF+ 2.*EC1qq[idx]*pow(gamma1qg[idx],2)*gamma1gg[idx]*LF3- 2.*EC1qq[idx]*pow(gamma1qg[idx],2)*gamma1gg[idx]*LQ3+ 6.*EC1qq[idx]*pow(gamma1qg[idx],2)*gamma1gg[idx]*LQ2*LF- 6.*EC1qq[idx]*pow(gamma1qg[idx],2)*gamma1gg[idx]*LQ*LF2+ 2.*EC1qq[idx]*gamma1qq[idx]*pow(gamma1qg[idx],2)*LF3- 2.*EC1qq[idx]*gamma1qq[idx]*pow(gamma1qg[idx],2)*LQ3+ 6.*EC1qq[idx]*gamma1qq[idx]*pow(gamma1qg[idx],2)*LQ2*LF- 6.*EC1qq[idx]*gamma1qq[idx]*pow(gamma1qg[idx],2)*LQ*LF2+ 2.*EC1qq[idx]*EC2qg[idx]*gamma1qg[idx]*LF- 2.*EC1qq[idx]*EC2qg[idx]*gamma1qg[idx]*LQ+ 2.*EC1qq[idx]*EC1qg[idx]*gamma2qg[idx]*LF - 2.*EC1qq[idx]*EC1qg[idx]*gamma2qg[idx]*LQ+ 3.*EC1qq[idx]*EC1qg[idx]*gamma1qg[idx]*gamma1gg[idx]*LF2+ 3.*EC1qq[idx]*EC1qg[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ2- 6.*EC1qq[idx]*EC1qg[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ*LF+ EC1qq[idx]*EC1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*LF2+ EC1qq[idx]*EC1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ2- 2.*EC1qq[idx]*EC1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ*LF+ pow(EC1qq[idx],2)*pow(gamma1qg[idx],2)*LF2+ pow(EC1qq[idx],2)*pow(gamma1qg[idx],2)*LQ2- 2.*pow(EC1qq[idx],2)*pow(gamma1qg[idx],2)*LQ*LF+ 2./3.*nf*pow(gamma1qg[idx],3)*gamma1gq[idx]*LF4+ 2./3.*nf*pow(gamma1qg[idx],3)*gamma1gq[idx]*LQ4- 8./3.*nf*pow(gamma1qg[idx],3)*gamma1gq[idx]*LQ3*LF+ 4.*nf*pow(gamma1qg[idx],3)*gamma1gq[idx]*LQ2*LF2- 8./3.*nf*pow(gamma1qg[idx],3)*gamma1gq[idx]*LQ*LF3 + 8./3.*nf*EC1qg[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LF3- 8./3.*nf*EC1qg[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3+ 8.*nf*EC1qg[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF- 8.*nf*EC1qg[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF2+ 2.*nf*pow(EC1qg[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LF2+ 2.*nf*pow(EC1qg[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ2- 4.*nf*pow(EC1qg[idx],2)*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+ beta1*pow(gamma1qg[idx],2)*LF3+ beta1*pow(gamma1qg[idx],2)*LQ3- beta1*pow(gamma1qg[idx],2)*LQ2*LF- beta1*pow(gamma1qg[idx],2)*LQ*LF2+ beta1*EC1qg[idx]*gamma1qg[idx]*LF2- 3.*beta1*EC1qg[idx]*gamma1qg[idx]*LQ2+ 2.*beta1*EC1qg[idx]*gamma1qg[idx]*LQ*LF+ 3.*beta0*gamma1qg[idx]*gamma2qg[idx]*LF3 + 3.*beta0*gamma1qg[idx]*gamma2qg[idx]*LQ3- 3.*beta0*gamma1qg[idx]*gamma2qg[idx]*LQ2*LF- 3.*beta0*gamma1qg[idx]*gamma2qg[idx]*LQ*LF2+ 3./2.*beta0*pow(gamma1qg[idx],2)*gamma1gg[idx]*LF4- 3./2.*beta0*pow(gamma1qg[idx],2)*gamma1gg[idx]*LQ4+ 3.*beta0*pow(gamma1qg[idx],2)*gamma1gg[idx]*LQ3*LF- 3.*beta0*pow(gamma1qg[idx],2)*gamma1gg[idx]*LQ*LF3+ 3./2.*beta0*gamma1qq[idx]*pow(gamma1qg[idx],2)*LF4- 3./2.*beta0*gamma1qq[idx]*pow(gamma1qg[idx],2)*LQ4+ 3.*beta0*gamma1qq[idx]*pow(gamma1qg[idx],2)*LQ3*LF- 3.*beta0*gamma1qq[idx]*pow(gamma1qg[idx],2)*LQ*LF3+ beta0*EC2qg[idx]*gamma1qg[idx]*LF2- 5.*beta0*EC2qg[idx]*gamma1qg[idx]*LQ2+ 4.*beta0*EC2qg[idx]*gamma1qg[idx]*LQ*LF+ 2.*beta0*EC1qg[idx]*gamma2qg[idx]*LF2 - 4.*beta0*EC1qg[idx]*gamma2qg[idx]*LQ2+ 2.*beta0*EC1qg[idx]*gamma2qg[idx]*LQ*LF+ 3.*beta0*EC1qg[idx]*gamma1qg[idx]*gamma1gg[idx]*LF3+ 6.*beta0*EC1qg[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ3- 9.*beta0*EC1qg[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF+ beta0*EC1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*LF3+ 2.*beta0*EC1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ3- 3.*beta0*EC1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ2*LF+ beta0*pow(EC1qg[idx],2)*gamma1gg[idx]*LF2- 5.*beta0*pow(EC1qg[idx],2)*gamma1gg[idx]*LQ2+ 4.*beta0*pow(EC1qg[idx],2)*gamma1gg[idx]*LQ*LF+ 2.*beta0*EC1qq[idx]*pow(gamma1qg[idx],2)*LF3+ 4.*beta0*EC1qq[idx]*pow(gamma1qg[idx],2)*LQ3- 6.*beta0*EC1qq[idx]*pow(gamma1qg[idx],2)*LQ2*LF+ beta0*EC1qq[idx]*EC1qg[idx]*gamma1qg[idx]*LF2 - 5.*beta0*EC1qq[idx]*EC1qg[idx]*gamma1qg[idx]*LQ2+ 4.*beta0*EC1qq[idx]*EC1qg[idx]*gamma1qg[idx]*LQ*LF+ 11./12.*pow(beta0,2)*pow(gamma1qg[idx],2)*LF4+ 11./12.*pow(beta0,2)*pow(gamma1qg[idx],2)*LQ4- 2./3.*pow(beta0,2)*pow(gamma1qg[idx],2)*LQ3*LF- 1./2.*pow(beta0,2)*pow(gamma1qg[idx],2)*LQ2*LF2- 2./3.*pow(beta0,2)*pow(gamma1qg[idx],2)*LQ*LF3+ 2./3.*pow(beta0,2)*EC1qg[idx]*gamma1qg[idx]*LF3- 11./3.*pow(beta0,2)*EC1qg[idx]*gamma1qg[idx]*LQ3+ 2.*pow(beta0,2)*EC1qg[idx]*gamma1qg[idx]*LQ2*LF+ pow(beta0,2)*EC1qg[idx]*gamma1qg[idx]*LQ*LF2- B2*pow(gamma1qg[idx],2)*LQ3+ 2.*B2*pow(gamma1qg[idx],2)*LQ2*LF- B2*pow(gamma1qg[idx],2)*LQ*LF2+ 2.*B2*EC1qg[idx]*gamma1qg[idx]*LQ2 - 2.*B2*EC1qg[idx]*gamma1qg[idx]*LQ*LF- 2.*B1*gamma1qg[idx]*gamma2qg[idx]*LQ3+ 4.*B1*gamma1qg[idx]*gamma2qg[idx]*LQ2*LF- 2.*B1*gamma1qg[idx]*gamma2qg[idx]*LQ*LF2+ B1*pow(gamma1qg[idx],2)*gamma1gg[idx]*LQ4- 3.*B1*pow(gamma1qg[idx],2)*gamma1gg[idx]*LQ3*LF+ 3.*B1*pow(gamma1qg[idx],2)*gamma1gg[idx]*LQ2*LF2- B1*pow(gamma1qg[idx],2)*gamma1gg[idx]*LQ*LF3+ B1*gamma1qq[idx]*pow(gamma1qg[idx],2)*LQ4- 3.*B1*gamma1qq[idx]*pow(gamma1qg[idx],2)*LQ3*LF+ 3.*B1*gamma1qq[idx]*pow(gamma1qg[idx],2)*LQ2*LF2- B1*gamma1qq[idx]*pow(gamma1qg[idx],2)*LQ*LF3+ 2.*B1*EC2qg[idx]*gamma1qg[idx]*LQ2- 2.*B1*EC2qg[idx]*gamma1qg[idx]*LQ*LF+ 2.*B1*EC1qg[idx]*gamma2qg[idx]*LQ2 - 2.*B1*EC1qg[idx]*gamma2qg[idx]*LQ*LF- 3.*B1*EC1qg[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ3+ 6.*B1*EC1qg[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF- 3.*B1*EC1qg[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ*LF2- B1*EC1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ3+ 2.*B1*EC1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ2*LF- B1*EC1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ*LF2+ 2.*B1*pow(EC1qg[idx],2)*gamma1gg[idx]*LQ2- 2.*B1*pow(EC1qg[idx],2)*gamma1gg[idx]*LQ*LF- 2.*B1*EC1qq[idx]*pow(gamma1qg[idx],2)*LQ3+ 4.*B1*EC1qq[idx]*pow(gamma1qg[idx],2)*LQ2*LF- 2.*B1*EC1qq[idx]*pow(gamma1qg[idx],2)*LQ*LF2+ 2.*B1*EC1qq[idx]*EC1qg[idx]*gamma1qg[idx]*LQ2- 2.*B1*EC1qq[idx]*EC1qg[idx]*gamma1qg[idx]*LQ*LF- 3./2.*B1*beta0*pow(gamma1qg[idx],2)*LQ4 + 2.*B1*beta0*pow(gamma1qg[idx],2)*LQ3*LF+ 1./2.*B1*beta0*pow(gamma1qg[idx],2)*LQ2*LF2- B1*beta0*pow(gamma1qg[idx],2)*LQ*LF3+ 4.*B1*beta0*EC1qg[idx]*gamma1qg[idx]*LQ3- 3.*B1*beta0*EC1qg[idx]*gamma1qg[idx]*LQ2*LF- B1*beta0*EC1qg[idx]*gamma1qg[idx]*LQ*LF2+ 1./2.*pow(B1,2)*pow(gamma1qg[idx],2)*LQ4- pow(B1,2)*pow(gamma1qg[idx],2)*LQ3*LF+ 1./2.*pow(B1,2)*pow(gamma1qg[idx],2)*LQ2*LF2- pow(B1,2)*EC1qg[idx]*gamma1qg[idx]*LQ3+ pow(B1,2)*EC1qg[idx]*gamma1qg[idx]*LQ2*LF- 1./2.*A2*pow(gamma1qg[idx],2)*LQ4+ A2*pow(gamma1qg[idx],2)*LQ3*LF- 1./2.*A2*pow(gamma1qg[idx],2)*LQ2*LF2+ A2*EC1qg[idx]*gamma1qg[idx]*LQ3 - A2*EC1qg[idx]*gamma1qg[idx]*LQ2*LF- A1*gamma1qg[idx]*gamma2qg[idx]*LQ4+ 2.*A1*gamma1qg[idx]*gamma2qg[idx]*LQ3*LF- A1*gamma1qg[idx]*gamma2qg[idx]*LQ2*LF2+ 1./2.*A1*pow(gamma1qg[idx],2)*gamma1gg[idx]*LQ5- 3./2.*A1*pow(gamma1qg[idx],2)*gamma1gg[idx]*LQ4*LF+ 3./2.*A1*pow(gamma1qg[idx],2)*gamma1gg[idx]*LQ3*LF2- 1./2.*A1*pow(gamma1qg[idx],2)*gamma1gg[idx]*LQ2*LF3+ 1./2.*A1*gamma1qq[idx]*pow(gamma1qg[idx],2)*LQ5- 3./2.*A1*gamma1qq[idx]*pow(gamma1qg[idx],2)*LQ4*LF+ 3./2.*A1*gamma1qq[idx]*pow(gamma1qg[idx],2)*LQ3*LF2- 1./2.*A1*gamma1qq[idx]*pow(gamma1qg[idx],2)*LQ2*LF3+ A1*EC2qg[idx]*gamma1qg[idx]*LQ3- A1*EC2qg[idx]*gamma1qg[idx]*LQ2*LF+ A1*EC1qg[idx]*gamma2qg[idx]*LQ3 - A1*EC1qg[idx]*gamma2qg[idx]*LQ2*LF- 3./2.*A1*EC1qg[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ4+ 3.*A1*EC1qg[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ3*LF- 3./2.*A1*EC1qg[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ2*LF2- 1./2.*A1*EC1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ4+ A1*EC1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ3*LF- 1./2.*A1*EC1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ2*LF2+ A1*pow(EC1qg[idx],2)*gamma1gg[idx]*LQ3- A1*pow(EC1qg[idx],2)*gamma1gg[idx]*LQ2*LF- A1*EC1qq[idx]*pow(gamma1qg[idx],2)*LQ4+ 2.*A1*EC1qq[idx]*pow(gamma1qg[idx],2)*LQ3*LF- A1*EC1qq[idx]*pow(gamma1qg[idx],2)*LQ2*LF2+ A1*EC1qq[idx]*EC1qg[idx]*gamma1qg[idx]*LQ3- A1*EC1qq[idx]*EC1qg[idx]*gamma1qg[idx]*LQ2*LF- 5./6.*A1*beta0*pow(gamma1qg[idx],2)*LQ5 + 7./6.*A1*beta0*pow(gamma1qg[idx],2)*LQ4*LF+ 1./6.*A1*beta0*pow(gamma1qg[idx],2)*LQ3*LF2- 1./2.*A1*beta0*pow(gamma1qg[idx],2)*LQ2*LF3+ 13./6.*A1*beta0*EC1qg[idx]*gamma1qg[idx]*LQ4- 5./3.*A1*beta0*EC1qg[idx]*gamma1qg[idx]*LQ3*LF- 1./2.*A1*beta0*EC1qg[idx]*gamma1qg[idx]*LQ2*LF2+ 1./2.*A1*B1*pow(gamma1qg[idx],2)*LQ5- A1*B1*pow(gamma1qg[idx],2)*LQ4*LF+ 1./2.*A1*B1*pow(gamma1qg[idx],2)*LQ3*LF2- A1*B1*EC1qg[idx]*gamma1qg[idx]*LQ4+ A1*B1*EC1qg[idx]*gamma1qg[idx]*LQ3*LF+ 1./8.*pow(A1,2)*pow(gamma1qg[idx],2)*LQ6- 1./4.*pow(A1,2)*pow(gamma1qg[idx],2)*LQ5*LF+ 1./8.*pow(A1,2)*pow(gamma1qg[idx],2)*LQ4*LF2- 1./4.*pow(A1,2)*EC1qg[idx]*gamma1qg[idx]*LQ5 + 1./4.*pow(A1,2)*EC1qg[idx]*gamma1qg[idx]*LQ4*LF ;
	//complex <double> H4st_qbg   =+ gamma2qg[idx]*gamma2qqb[idx]*LF2+ gamma2qg[idx]*gamma2qqb[idx]*LQ2- 2.*gamma2qg[idx]*gamma2qqb[idx]*LQ*LF+ gamma1qg[idx]*gamma3qqb[idx]*LF2+ gamma1qg[idx]*gamma3qqb[idx]*LQ2- 2.*gamma1qg[idx]*gamma3qqb[idx]*LQ*LF+ 1./2.*gamma1qg[idx]*gamma1gg[idx]*gamma2qqb[idx]*LF3- 1./2.*gamma1qg[idx]*gamma1gg[idx]*gamma2qqb[idx]*LQ3+ 3./2.*gamma1qg[idx]*gamma1gg[idx]*gamma2qqb[idx]*LQ2*LF- 3./2.*gamma1qg[idx]*gamma1gg[idx]*gamma2qqb[idx]*LQ*LF2+ 1./2.*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LF3- 1./2.*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ3+ 5./2.*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ2*LF- 5./2.*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ*LF2 + pow(gamma1qg[idx],2)*gamma2gq[idx]*LF3- pow(gamma1qg[idx],2)*gamma2gq[idx]*LQ3+ 2.*pow(gamma1qg[idx],2)*gamma2gq[idx]*LQ2*LF- 2.*pow(gamma1qg[idx],2)*gamma2gq[idx]*LQ*LF2+ 5./12.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LF4+ 5./12.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ4- 5./3.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF+ 5./2.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF2- 5./3.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ*LF3+ 1./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2qqb[idx]*LF3- 1./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2qqb[idx]*LQ3+ 5./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2qqb[idx]*LQ2*LF- 5./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2qqb[idx]*LQ*LF2+ 7./12.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LF4+ 7./12.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4 - 7./3.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]* LQ3*LF+ 7./2.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF2- 7./3.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF3+ EC2qg[idx]*gamma2qqb[idx]*LF- EC2qg[idx]*gamma2qqb[idx]*LQ+ 3./2.*EC2qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+ 3./2.*EC2qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2- 3.*EC2qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+ EC1qg[idx]*gamma3qqb[idx]*LF- EC1qg[idx]*gamma3qqb[idx]*LQ+ EC1qg[idx]*gamma1gg[idx]*gamma2qqb[idx]*LF2+ EC1qg[idx]*gamma1gg[idx]*gamma2qqb[idx]*LQ2- 2.*EC1qg[idx]*gamma1gg[idx]*gamma2qqb[idx]*LQ*LF+ EC1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LF2 + EC1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ2- 3.*EC1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ*LF+ 2.*EC1qg[idx]*gamma1qg[idx]*gamma2gq[idx]*LF2+ 2.*EC1qg[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ2- 3.*EC1qg[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ*LF+ 5./3.*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF3- 5./3.*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3+ 5.*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF- 5.*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF2- EC1qg[idx]*gamma1qq[idx]*gamma2qqb[idx]*LQ*LF+ 4./3.*EC1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3- 4./3.*EC1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+ 4.*EC1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF- 4.*EC1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+ 2.*EC1qg[idx]*EC2qg[idx]*gamma1gq[idx]*LF - 2.*EC1qg[idx]*EC2qg[idx]*gamma1gq[idx]*LQ+ pow(EC1qg[idx],2)*gamma2gq[idx]*LF- pow(EC1qg[idx],2)*gamma2gq[idx]*LQ+ 3./2.*pow(EC1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LF2+ 3./2.*pow(EC1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ2- 3.*pow(EC1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ*LF+ 1./2.*pow(EC1qg[idx],2)*gamma1qq[idx]*gamma1gq[idx]*LF2+ 1./2.*pow(EC1qg[idx],2)*gamma1qq[idx]*gamma1gq[idx]*LQ2- pow(EC1qg[idx],2)*gamma1qq[idx]*gamma1gq[idx]*LQ*LF+ EC3qqb[idx]*gamma1qg[idx]*LF- EC3qqb[idx]*gamma1qg[idx]*LQ+ EC2qqb[idx]*gamma2qg[idx]*LF- EC2qqb[idx]*gamma2qg[idx]*LQ+ 1./2.*EC2qqb[idx]*gamma1qg[idx]*gamma1gg[idx]*LF2+ 1./2.*EC2qqb[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ2 - EC2qqb[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ*LF+ 1./2.*EC2qqb[idx]*gamma1qq[idx]*gamma1qg[idx]*LF2+ 1./2.*EC2qqb[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ2- EC2qqb[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ*LF+ EC2qqb[idx]*EC1qg[idx]*gamma1gg[idx]*LF- EC2qqb[idx]*EC1qg[idx]*gamma1gg[idx]*LQ+ 2.*EC1qq[idx]*gamma1qg[idx]*gamma2qqb[idx]*LF2+ 2.*EC1qq[idx]*gamma1qg[idx]*gamma2qqb[idx]*LQ2- 4.*EC1qq[idx]*gamma1qg[idx]*gamma2qqb[idx]*LQ*LF+ EC1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LF3- EC1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3+ 3.*EC1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF- 3.*EC1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF2+ EC1qq[idx]*EC1qg[idx]*gamma2qqb[idx]*LF- EC1qq[idx]*EC1qg[idx]*gamma2qqb[idx]*LQ + 3./2.*EC1qq[idx]*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+ 3./2.*EC1qq[idx]*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2- 3.*EC1qq[idx]*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+ EC1qq[idx]*EC2qqb[idx]*gamma1qg[idx]*LF- EC1qq[idx]*EC2qqb[idx]*gamma1qg[idx]*LQ+ 3./2.*beta0*gamma1qg[idx]*gamma2qqb[idx]*LF3+ 3./2.*beta0*gamma1qg[idx]*gamma2qqb[idx]*LQ3- 3./2.*beta0*gamma1qg[idx]*gamma2qqb[idx]*LQ2*LF- 3./2.*beta0*gamma1qg[idx]*gamma2qqb[idx]*LQ*LF2+ 3./4.*beta0*pow(gamma1qg[idx],2)*gamma1gq[idx]*LF4- 3./4.*beta0*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4+ 3./2.*beta0*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3*LF- 3./2.*beta0*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF3+ beta0*EC1qg[idx]*gamma2qqb[idx]*LF2- 2.*beta0*EC1qg[idx]*gamma2qqb[idx]*LQ2 + beta0*EC1qg[idx]*gamma2qqb[idx]*LQ*LF+ 3./2.*beta0*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3+ 3.*beta0*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3- 9./2.*beta0*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF+ 1./2.*beta0*pow(EC1qg[idx],2)*gamma1gq[idx]*LF2- 5./2.*beta0*pow(EC1qg[idx],2)*gamma1gq[idx]*LQ2+ 2.*beta0*pow(EC1qg[idx],2)*gamma1gq[idx]*LQ*LF+ 1./2.*beta0*EC2qqb[idx]*gamma1qg[idx]*LF2- 5./2.*beta0*EC2qqb[idx]*gamma1qg[idx]*LQ2+ 2.*beta0*EC2qqb[idx]*gamma1qg[idx]*LQ*LF- B1*gamma1qg[idx]*gamma2qqb[idx]*LQ3+ 2.*B1*gamma1qg[idx]*gamma2qqb[idx]*LQ2*LF- B1*gamma1qg[idx]*gamma2qqb[idx]*LQ*LF2+ 1./2.*B1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4- 3./2.*B1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3*LF + 3./2.*B1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF2- 1./2.*B1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF3+ B1*EC1qg[idx]*gamma2qqb[idx]*LQ2- B1*EC1qg[idx]*gamma2qqb[idx]*LQ*LF- 3./2.*B1*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+ 3.*B1*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF- 3./2.*B1*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+ B1*pow(EC1qg[idx],2)*gamma1gq[idx]*LQ2- B1*pow(EC1qg[idx],2)*gamma1gq[idx]*LQ*LF+ B1*EC2qqb[idx]*gamma1qg[idx]*LQ2- B1*EC2qqb[idx]*gamma1qg[idx]*LQ*LF- 1./2.*A1*gamma1qg[idx]*gamma2qqb[idx]*LQ4+ A1*gamma1qg[idx]*gamma2qqb[idx]*LQ3*LF- 1./2.*A1*gamma1qg[idx]*gamma2qqb[idx]*LQ2*LF2+ 1./4.*A1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ5 - 3./4.*A1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4*LF+ 3./4.*A1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3*LF2- 1./4.*A1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF3+ 1./2.*A1*EC1qg[idx]*gamma2qqb[idx]*LQ3- 1./2.*A1*EC1qg[idx]*gamma2qqb[idx]*LQ2*LF- 3./4.*A1*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4+ 3./2.*A1*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF- 3./4.*A1*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2+ 1./2.*A1*pow(EC1qg[idx],2)*gamma1gq[idx]*LQ3- 1./2.*A1*pow(EC1qg[idx],2)*gamma1gq[idx]*LQ2*LF+ 1./2.*A1*EC2qqb[idx]*gamma1qg[idx]*LQ3- 1./2.*A1*EC2qqb[idx]*gamma1qg[idx]*LQ2*LF ;
	//complex <double> H4st_qpg   =+ gamma2qg[idx]*gamma2qqp[idx]*LF2+ gamma2qg[idx]*gamma2qqp[idx]*LQ2- 2.*gamma2qg[idx]*gamma2qqp[idx]*LQ*LF+ gamma1qg[idx]*gamma3qqp[idx]*LF2+ gamma1qg[idx]*gamma3qqp[idx]*LQ2- 2.*gamma1qg[idx]*gamma3qqp[idx]*LQ*LF+ 1./2.*gamma1qg[idx]*gamma1gg[idx]*gamma2qqp[idx]*LF3- 1./2.*gamma1qg[idx]*gamma1gg[idx]*gamma2qqp[idx]*LQ3+ 3./2.*gamma1qg[idx]*gamma1gg[idx]*gamma2qqp[idx]*LQ2*LF- 3./2.*gamma1qg[idx]*gamma1gg[idx]*gamma2qqp[idx]*LQ*LF2+ 1./2.*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LF3- 1./2.*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ3+ 5./2.*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ2*LF- 5./2.*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ*LF2 + pow(gamma1qg[idx],2)*gamma2gq[idx]*LF3- pow(gamma1qg[idx],2)*gamma2gq[idx]*LQ3+ 2.*pow(gamma1qg[idx],2)*gamma2gq[idx]*LQ2*LF- 2.*pow(gamma1qg[idx],2)*gamma2gq[idx]*LQ*LF2+ 5./12.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LF4+ 5./12.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ4- 5./3.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF+ 5./2.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF2- 5./3.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ*LF3+ 1./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2qqp[idx]*LF3- 1./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2qqp[idx]*LQ3+ 5./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2qqp[idx]*LQ2*LF- 5./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2qqp[idx]*LQ*LF2+ 7./12.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LF4+ 7./12.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4 - 7./3.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]* LQ3*LF+ 7./2.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF2- 7./3.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF3+ EC2qg[idx]*gamma2qqp[idx]*LF- EC2qg[idx]*gamma2qqp[idx]*LQ+ 3./2.*EC2qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+ 3./2.*EC2qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2- 3.*EC2qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+ EC1qg[idx]*gamma3qqp[idx]*LF- EC1qg[idx]*gamma3qqp[idx]*LQ+ EC1qg[idx]*gamma1gg[idx]*gamma2qqp[idx]*LF2+ EC1qg[idx]*gamma1gg[idx]*gamma2qqp[idx]*LQ2- 2.*EC1qg[idx]*gamma1gg[idx]*gamma2qqp[idx]*LQ*LF+ EC1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LF2 + EC1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ2- 3.*EC1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ*LF+ 2.*EC1qg[idx]*gamma1qg[idx]*gamma2gq[idx]*LF2+ 2.*EC1qg[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ2- 3.*EC1qg[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ*LF+ 5./3.*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF3- 5./3.*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3+ 5.*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF- 5.*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF2- EC1qg[idx]*gamma1qq[idx]*gamma2qqp[idx]*LQ*LF+ 4./3.*EC1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3- 4./3.*EC1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+ 4.*EC1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF- 4.*EC1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+ 2.*EC1qg[idx]*EC2qg[idx]*gamma1gq[idx]*LF - 2.*EC1qg[idx]*EC2qg[idx]*gamma1gq[idx]*LQ+ pow(EC1qg[idx],2)*gamma2gq[idx]*LF- pow(EC1qg[idx],2)*gamma2gq[idx]*LQ+ 3./2.*pow(EC1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LF2+ 3./2.*pow(EC1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ2- 3.*pow(EC1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ*LF+ 1./2.*pow(EC1qg[idx],2)*gamma1qq[idx]*gamma1gq[idx]*LF2+ 1./2.*pow(EC1qg[idx],2)*gamma1qq[idx]*gamma1gq[idx]*LQ2- pow(EC1qg[idx],2)*gamma1qq[idx]*gamma1gq[idx]*LQ*LF+ EC3qqp[idx]*gamma1qg[idx]*LF- EC3qqp[idx]*gamma1qg[idx]*LQ+ EC2qqp[idx]*gamma2qg[idx]*LF- EC2qqp[idx]*gamma2qg[idx]*LQ+ 1./2.*EC2qqp[idx]*gamma1qg[idx]*gamma1gg[idx]*LF2+ 1./2.*EC2qqp[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ2 - EC2qqp[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ*LF+ 1./2.*EC2qqp[idx]*gamma1qq[idx]*gamma1qg[idx]*LF2+ 1./2.*EC2qqp[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ2- EC2qqp[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ*LF+ EC2qqp[idx]*EC1qg[idx]*gamma1gg[idx]*LF- EC2qqp[idx]*EC1qg[idx]*gamma1gg[idx]*LQ+ 2.*EC1qq[idx]*gamma1qg[idx]*gamma2qqp[idx]*LF2+ 2.*EC1qq[idx]*gamma1qg[idx]*gamma2qqp[idx]*LQ2- 4.*EC1qq[idx]*gamma1qg[idx]*gamma2qqp[idx]*LQ*LF+ EC1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LF3- EC1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3+ 3.*EC1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF- 3.*EC1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF2+ EC1qq[idx]*EC1qg[idx]*gamma2qqp[idx]*LF- EC1qq[idx]*EC1qg[idx]*gamma2qqp[idx]*LQ + 3./2.*EC1qq[idx]*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+ 3./2.*EC1qq[idx]*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2- 3.*EC1qq[idx]*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+ EC1qq[idx]*EC2qqp[idx]*gamma1qg[idx]*LF- EC1qq[idx]*EC2qqp[idx]*gamma1qg[idx]*LQ+ 3./2.*beta0*gamma1qg[idx]*gamma2qqp[idx]*LF3+ 3./2.*beta0*gamma1qg[idx]*gamma2qqp[idx]*LQ3- 3./2.*beta0*gamma1qg[idx]*gamma2qqp[idx]*LQ2*LF- 3./2.*beta0*gamma1qg[idx]*gamma2qqp[idx]*LQ*LF2+ 3./4.*beta0*pow(gamma1qg[idx],2)*gamma1gq[idx]*LF4- 3./4.*beta0*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4+ 3./2.*beta0*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3*LF- 3./2.*beta0*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF3+ beta0*EC1qg[idx]*gamma2qqp[idx]*LF2- 2.*beta0*EC1qg[idx]*gamma2qqp[idx]*LQ2 + beta0*EC1qg[idx]*gamma2qqp[idx]*LQ*LF+ 3./2.*beta0*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3+ 3.*beta0*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3- 9./2.*beta0*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF+ 1./2.*beta0*pow(EC1qg[idx],2)*gamma1gq[idx]*LF2- 5./2.*beta0*pow(EC1qg[idx],2)*gamma1gq[idx]*LQ2+ 2.*beta0*pow(EC1qg[idx],2)*gamma1gq[idx]*LQ*LF+ 1./2.*beta0*EC2qqp[idx]*gamma1qg[idx]*LF2- 5./2.*beta0*EC2qqp[idx]*gamma1qg[idx]*LQ2+ 2.*beta0*EC2qqp[idx]*gamma1qg[idx]*LQ*LF- B1*gamma1qg[idx]*gamma2qqp[idx]*LQ3+ 2.*B1*gamma1qg[idx]*gamma2qqp[idx]*LQ2*LF- B1*gamma1qg[idx]*gamma2qqp[idx]*LQ*LF2+ 1./2.*B1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4- 3./2.*B1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3*LF + 3./2.*B1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF2- 1./2.*B1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF3+ B1*EC1qg[idx]*gamma2qqp[idx]*LQ2- B1*EC1qg[idx]*gamma2qqp[idx]*LQ*LF- 3./2.*B1*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+ 3.*B1*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF- 3./2.*B1*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+ B1*pow(EC1qg[idx],2)*gamma1gq[idx]*LQ2- B1*pow(EC1qg[idx],2)*gamma1gq[idx]*LQ*LF+ B1*EC2qqp[idx]*gamma1qg[idx]*LQ2- B1*EC2qqp[idx]*gamma1qg[idx]*LQ*LF- 1./2.*A1*gamma1qg[idx]*gamma2qqp[idx]*LQ4+ A1*gamma1qg[idx]*gamma2qqp[idx]*LQ3*LF- 1./2.*A1*gamma1qg[idx]*gamma2qqp[idx]*LQ2*LF2+ 1./4.*A1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ5 - 3./4.*A1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4*LF+ 3./4.*A1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3*LF2- 1./4.*A1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF3+ 1./2.*A1*EC1qg[idx]*gamma2qqp[idx]*LQ3- 1./2.*A1*EC1qg[idx]*gamma2qqp[idx]*LQ2*LF- 3./4.*A1*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4+ 3./2.*A1*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF- 3./4.*A1*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2+ 1./2.*A1*pow(EC1qg[idx],2)*gamma1gq[idx]*LQ3- 1./2.*A1*pow(EC1qg[idx],2)*gamma1gq[idx]*LQ2*LF+ 1./2.*A1*EC2qqp[idx]*gamma1qg[idx]*LQ3- 1./2.*A1*EC2qqp[idx]*gamma1qg[idx]*LQ2*LF ;
	//complex <double> H4st_qbpg  =+ gamma2qg[idx]*gamma2qqbp[idx]*LF2+ gamma2qg[idx]*gamma2qqbp[idx]*LQ2- 2.*gamma2qg[idx]*gamma2qqbp[idx]*LQ*LF+ gamma1qg[idx]*gamma3qqbp[idx]*LF2+ gamma1qg[idx]*gamma3qqbp[idx]*LQ2- 2.*gamma1qg[idx]*gamma3qqbp[idx]*LQ*LF+ 1./2.*gamma1qg[idx]*gamma1gg[idx]*gamma2qqbp[idx]*LF3- 1./2.*gamma1qg[idx]*gamma1gg[idx]*gamma2qqbp[idx]*LQ3+ 3./2.*gamma1qg[idx]*gamma1gg[idx]*gamma2qqbp[idx]*LQ2*LF- 3./2.*gamma1qg[idx]*gamma1gg[idx]*gamma2qqbp[idx]*LQ*LF2+ 1./2.*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LF3- 1./2.*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ3+ 5./2.*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ2*LF- 5./2.*gamma1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ*LF2 + pow(gamma1qg[idx],2)*gamma2gq[idx]*LF3- pow(gamma1qg[idx],2)*gamma2gq[idx]*LQ3+ 2.*pow(gamma1qg[idx],2)*gamma2gq[idx]*LQ2*LF- 2.*pow(gamma1qg[idx],2)*gamma2gq[idx]*LQ*LF2+ 5./12.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LF4+ 5./12.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ4- 5./3.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ3*LF+ 5./2.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF2- 5./3.*pow(gamma1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ*LF3+ 1./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2qqbp[idx]*LF3- 1./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2qqbp[idx]*LQ3+ 5./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2qqbp[idx]*LQ2*LF- 5./2.*gamma1qq[idx]*gamma1qg[idx]*gamma2qqbp[idx]*LQ*LF2+ 7./12.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LF4+ 7./12.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4 - 7./3.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]* LQ3*LF+ 7./2.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF2- 7./3.*gamma1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF3+ EC2qg[idx]*gamma2qqbp[idx]*LF- EC2qg[idx]*gamma2qqbp[idx]*LQ+ 3./2.*EC2qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+ 3./2.*EC2qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2- 3.*EC2qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+ EC1qg[idx]*gamma3qqbp[idx]*LF- EC1qg[idx]*gamma3qqbp[idx]*LQ+ EC1qg[idx]*gamma1gg[idx]*gamma2qqbp[idx]*LF2+ EC1qg[idx]*gamma1gg[idx]*gamma2qqbp[idx]*LQ2- 2.*EC1qg[idx]*gamma1gg[idx]*gamma2qqbp[idx]*LQ*LF+ EC1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LF2 + EC1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ2- 3.*EC1qg[idx]*gamma2qg[idx]*gamma1gq[idx]*LQ*LF+ 2.*EC1qg[idx]*gamma1qg[idx]*gamma2gq[idx]*LF2+ 2.*EC1qg[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ2- 3.*EC1qg[idx]*gamma1qg[idx]*gamma2gq[idx]*LQ*LF+ 5./3.*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LF3- 5./3.*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ3+ 5.*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ2*LF- 5.*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*gamma1gg[idx]*LQ*LF2- EC1qg[idx]*gamma1qq[idx]*gamma2qqbp[idx]*LQ*LF+ 4./3.*EC1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3- 4./3.*EC1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+ 4.*EC1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF- 4.*EC1qg[idx]*gamma1qq[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+ 2.*EC1qg[idx]*EC2qg[idx]*gamma1gq[idx]*LF - 2.*EC1qg[idx]*EC2qg[idx]*gamma1gq[idx]*LQ+ pow(EC1qg[idx],2)*gamma2gq[idx]*LF- pow(EC1qg[idx],2)*gamma2gq[idx]*LQ+ 3./2.*pow(EC1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LF2+ 3./2.*pow(EC1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ2- 3.*pow(EC1qg[idx],2)*gamma1gq[idx]*gamma1gg[idx]*LQ*LF+ 1./2.*pow(EC1qg[idx],2)*gamma1qq[idx]*gamma1gq[idx]*LF2+ 1./2.*pow(EC1qg[idx],2)*gamma1qq[idx]*gamma1gq[idx]*LQ2- pow(EC1qg[idx],2)*gamma1qq[idx]*gamma1gq[idx]*LQ*LF+ EC3qqbp[idx]*gamma1qg[idx]*LF- EC3qqbp[idx]*gamma1qg[idx]*LQ+ EC2qqbp[idx]*gamma2qg[idx]*LF- EC2qqbp[idx]*gamma2qg[idx]*LQ+ 1./2.*EC2qqbp[idx]*gamma1qg[idx]*gamma1gg[idx]*LF2+ 1./2.*EC2qqbp[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ2 - EC2qqbp[idx]*gamma1qg[idx]*gamma1gg[idx]*LQ*LF+ 1./2.*EC2qqbp[idx]*gamma1qq[idx]*gamma1qg[idx]*LF2+ 1./2.*EC2qqbp[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ2- EC2qqbp[idx]*gamma1qq[idx]*gamma1qg[idx]*LQ*LF+ EC2qqbp[idx]*EC1qg[idx]*gamma1gg[idx]*LF- EC2qqbp[idx]*EC1qg[idx]*gamma1gg[idx]*LQ+ 2.*EC1qq[idx]*gamma1qg[idx]*gamma2qqbp[idx]*LF2+ 2.*EC1qq[idx]*gamma1qg[idx]*gamma2qqbp[idx]*LQ2- 4.*EC1qq[idx]*gamma1qg[idx]*gamma2qqbp[idx]*LQ*LF+ EC1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LF3- EC1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3+ 3.*EC1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF- 3.*EC1qq[idx]*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF2+ EC1qq[idx]*EC1qg[idx]*gamma2qqbp[idx]*LF- EC1qq[idx]*EC1qg[idx]*gamma2qqbp[idx]*LQ + 3./2.*EC1qq[idx]*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+ 3./2.*EC1qq[idx]*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2- 3.*EC1qq[idx]*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+ EC1qq[idx]*EC2qqbp[idx]*gamma1qg[idx]*LF- EC1qq[idx]*EC2qqbp[idx]*gamma1qg[idx]*LQ+ 3./2.*beta0*gamma1qg[idx]*gamma2qqbp[idx]*LF3+ 3./2.*beta0*gamma1qg[idx]*gamma2qqbp[idx]*LQ3- 3./2.*beta0*gamma1qg[idx]*gamma2qqbp[idx]*LQ2*LF- 3./2.*beta0*gamma1qg[idx]*gamma2qqbp[idx]*LQ*LF2+ 3./4.*beta0*pow(gamma1qg[idx],2)*gamma1gq[idx]*LF4- 3./4.*beta0*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4+ 3./2.*beta0*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3*LF- 3./2.*beta0*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF3+ beta0*EC1qg[idx]*gamma2qqbp[idx]*LF2- 2.*beta0*EC1qg[idx]*gamma2qqbp[idx]*LQ2 + beta0*EC1qg[idx]*gamma2qqbp[idx]*LQ*LF+ 3./2.*beta0*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LF3+ 3.*beta0*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3- 9./2.*beta0*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF+ 1./2.*beta0*pow(EC1qg[idx],2)*gamma1gq[idx]*LF2- 5./2.*beta0*pow(EC1qg[idx],2)*gamma1gq[idx]*LQ2+ 2.*beta0*pow(EC1qg[idx],2)*gamma1gq[idx]*LQ*LF+ 1./2.*beta0*EC2qqbp[idx]*gamma1qg[idx]*LF2- 5./2.*beta0*EC2qqbp[idx]*gamma1qg[idx]*LQ2+ 2.*beta0*EC2qqbp[idx]*gamma1qg[idx]*LQ*LF- B1*gamma1qg[idx]*gamma2qqbp[idx]*LQ3+ 2.*B1*gamma1qg[idx]*gamma2qqbp[idx]*LQ2*LF- B1*gamma1qg[idx]*gamma2qqbp[idx]*LQ*LF2+ 1./2.*B1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4- 3./2.*B1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3*LF + 3./2.*B1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF2- 1./2.*B1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ*LF3+ B1*EC1qg[idx]*gamma2qqbp[idx]*LQ2- B1*EC1qg[idx]*gamma2qqbp[idx]*LQ*LF- 3./2.*B1*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3+ 3.*B1*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF- 3./2.*B1*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF2+ B1*pow(EC1qg[idx],2)*gamma1gq[idx]*LQ2- B1*pow(EC1qg[idx],2)*gamma1gq[idx]*LQ*LF+ B1*EC2qqbp[idx]*gamma1qg[idx]*LQ2- B1*EC2qqbp[idx]*gamma1qg[idx]*LQ*LF- 1./2.*A1*gamma1qg[idx]*gamma2qqbp[idx]*LQ4+ A1*gamma1qg[idx]*gamma2qqbp[idx]*LQ3*LF- 1./2.*A1*gamma1qg[idx]*gamma2qqbp[idx]*LQ2*LF2+ 1./4.*A1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ5 - 3./4.*A1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ4*LF+ 3./4.*A1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ3*LF2- 1./4.*A1*pow(gamma1qg[idx],2)*gamma1gq[idx]*LQ2*LF3+ 1./2.*A1*EC1qg[idx]*gamma2qqbp[idx]*LQ3- 1./2.*A1*EC1qg[idx]*gamma2qqbp[idx]*LQ2*LF- 3./4.*A1*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ4+ 3./2.*A1*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ3*LF- 3./4.*A1*EC1qg[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2*LF2+ 1./2.*A1*pow(EC1qg[idx],2)*gamma1gq[idx]*LQ3- 1./2.*A1*pow(EC1qg[idx],2)*gamma1gq[idx]*LQ2*LF+ 1./2.*A1*EC2qqbp[idx]*gamma1qg[idx]*LQ3- 1./2.*A1*EC2qqbp[idx]*gamma1qg[idx]*LQ2*LF ;
	//complex <double> H4st_qbq   =+ pow(gamma2qqb[idx],2)*LF2+ pow(gamma2qqb[idx],2)*LQ2- 2.*pow(gamma2qqb[idx],2)*LQ*LF+ gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LF3- gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ3+ 3.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ2*LF- 3.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ*LF2+ 1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LF4+ 1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ4- pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ3*LF+ 3./2.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2*LF2- pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF3+ 2.*EC1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LF2+ 2.*EC1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ2 - 4.*EC1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ*LF+ EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LF3- EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ3+ 3.*EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ2*LF- 3.*EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ*LF2+ pow(EC1qg[idx],2)*pow(gamma1gq[idx],2)*LF2+ pow(EC1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2- 2.*pow(EC1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF+ 2.*EC2qqb[idx]*gamma2qqb[idx]*LF- 2.*EC2qqb[idx]*gamma2qqb[idx]*LQ+ EC2qqb[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+ EC2qqb[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2- 2.*EC2qqb[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+ 2.*EC2qqb[idx]*EC1qg[idx]*gamma1gq[idx]*LF- 2.*EC2qqb[idx]*EC1qg[idx]*gamma1gq[idx]*LQ ;
	//complex <double> H4st_qbqp  =+ gamma2qqb[idx]*gamma2qqbp[idx]*LF2+ gamma2qqb[idx]*gamma2qqbp[idx]*LQ2- 2.*gamma2qqb[idx]*gamma2qqbp[idx]*LQ*LF+ 1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LF3- 1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ3+ 3./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ2*LF- 3./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ*LF2+ 1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LF3- 1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ3+ 3./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ2*LF- 3./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ*LF2+ 1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LF4+ 1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ4- pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ3*LF + 3./2.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2* LF2- pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF3+ EC1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LF2+ EC1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ2- 2.*EC1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ*LF+ EC1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LF2+ EC1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ2- 2.*EC1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ*LF+ EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LF3- EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ3+ 3.*EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ2*LF- 3.*EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ*LF2+ pow(EC1qg[idx],2)*pow(gamma1gq[idx],2)*LF2+ pow(EC1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2 - 2.*pow(EC1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF+ EC2qqbp[idx]*gamma2qqb[idx]*LF- EC2qqbp[idx]*gamma2qqb[idx]*LQ+ 1./2.*EC2qqbp[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+ 1./2.*EC2qqbp[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2- EC2qqbp[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+ EC2qqbp[idx]*EC1qg[idx]*gamma1gq[idx]*LF- EC2qqbp[idx]*EC1qg[idx]*gamma1gq[idx]*LQ+ EC2qqb[idx]*gamma2qqbp[idx]*LF- EC2qqb[idx]*gamma2qqbp[idx]*LQ+ 1./2.*EC2qqb[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+ 1./2.*EC2qqb[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2- EC2qqb[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+ EC2qqb[idx]*EC1qg[idx]*gamma1gq[idx]*LF- EC2qqb[idx]*EC1qg[idx]*gamma1gq[idx]*LQ ;
	//complex <double> H4st_qbqbp =+ gamma2qqb[idx]*gamma2qqp[idx]*LF2+ gamma2qqb[idx]*gamma2qqp[idx]*LQ2- 2.*gamma2qqb[idx]*gamma2qqp[idx]*LQ*LF+ 1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LF3- 1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ3+ 3./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ2*LF- 3./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ*LF2+ 1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LF3- 1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ3+ 3./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ2*LF- 3./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ*LF2+ 1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LF4+ 1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ4- pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ3*LF + 3./2.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2* LF2- pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF3+ EC1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LF2+ EC1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ2- 2.*EC1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ*LF+ EC1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LF2+ EC1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ2- 2.*EC1qg[idx]*gamma1gq[idx]*gamma2qqb[idx]*LQ*LF+ EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LF3- EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ3+ 3.*EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ2*LF- 3.*EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ*LF2+ pow(EC1qg[idx],2)*pow(gamma1gq[idx],2)*LF2+ pow(EC1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2 - 2.*pow(EC1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF+ EC2qqp[idx]*gamma2qqb[idx]*LF- EC2qqp[idx]*gamma2qqb[idx]*LQ+ 1./2.*EC2qqp[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+ 1./2.*EC2qqp[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2- EC2qqp[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+ EC2qqp[idx]*EC1qg[idx]*gamma1gq[idx]*LF- EC2qqp[idx]*EC1qg[idx]*gamma1gq[idx]*LQ+ EC2qqb[idx]*gamma2qqp[idx]*LF- EC2qqb[idx]*gamma2qqp[idx]*LQ+ 1./2.*EC2qqb[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+ 1./2.*EC2qqb[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2- EC2qqb[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+ EC2qqb[idx]*EC1qg[idx]*gamma1gq[idx]*LF- EC2qqb[idx]*EC1qg[idx]*gamma1gq[idx]*LQ ;
	//complex <double> H4st_qpqp  =+ gamma2qqp[idx]*gamma2qqbp[idx]*LF2+ gamma2qqp[idx]*gamma2qqbp[idx]*LQ2- 2.*gamma2qqp[idx]*gamma2qqbp[idx]*LQ*LF+ 1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LF3- 1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ3+ 3./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ2*LF- 3./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ*LF2+ 1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LF3- 1./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ3+ 3./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ2*LF- 3./2.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ*LF2+ 1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LF4+ 1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ4- pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ3*LF + 3./2.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2* LF2- pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF3+ EC1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LF2+ EC1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ2- 2.*EC1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ*LF+ EC1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LF2+ EC1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ2- 2.*EC1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ*LF+ EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LF3- EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ3+ 3.*EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ2*LF- 3.*EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ*LF2+ pow(EC1qg[idx],2)*pow(gamma1gq[idx],2)*LF2+ pow(EC1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2 - 2.*pow(EC1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF+ EC2qqbp[idx]*gamma2qqp[idx]*LF- EC2qqbp[idx]*gamma2qqp[idx]*LQ+ 1./2.*EC2qqbp[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+ 1./2.*EC2qqbp[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2- EC2qqbp[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+ EC2qqbp[idx]*EC1qg[idx]*gamma1gq[idx]*LF- EC2qqbp[idx]*EC1qg[idx]*gamma1gq[idx]*LQ+ EC2qqp[idx]*gamma2qqbp[idx]*LF- EC2qqp[idx]*gamma2qqbp[idx]*LQ+ 1./2.*EC2qqp[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+ 1./2.*EC2qqp[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2- EC2qqp[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+ EC2qqp[idx]*EC1qg[idx]*gamma1gq[idx]*LF- EC2qqp[idx]*EC1qg[idx]*gamma1gq[idx]*LQ ;
	//complex <double> H4st_qpqbp =+ pow(gamma2qqp[idx],2)*LF2+ pow(gamma2qqp[idx],2)*LQ2- 2.*pow(gamma2qqp[idx],2)*LQ*LF+ gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LF3- gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ3+ 3.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ2*LF- 3.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ*LF2+ 1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LF4+ 1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ4- pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ3*LF+ 3./2.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2*LF2- pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF3+ 2.*EC1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LF2+ 2.*EC1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ2 - 4.*EC1qg[idx]*gamma1gq[idx]*gamma2qqp[idx]*LQ*LF+ EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LF3- EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ3+ 3.*EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ2*LF- 3.*EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ*LF2+ pow(EC1qg[idx],2)*pow(gamma1gq[idx],2)*LF2+ pow(EC1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2- 2.*pow(EC1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF+ 2.*EC2qqp[idx]*gamma2qqp[idx]*LF- 2.*EC2qqp[idx]*gamma2qqp[idx]*LQ+ EC2qqp[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+ EC2qqp[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2- 2.*EC2qqp[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+ 2.*EC2qqp[idx]*EC1qg[idx]*gamma1gq[idx]*LF- 2.*EC2qqp[idx]*EC1qg[idx]*gamma1gq[idx]*LQ ;
	//complex <double> H4st_qbpqp =+ pow(gamma2qqbp[idx],2)*LF2+ pow(gamma2qqbp[idx],2)*LQ2- 2.*pow(gamma2qqbp[idx],2)*LQ*LF+ gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LF3- gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ3+ 3.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ2*LF- 3.*gamma1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ*LF2+ 1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LF4+ 1./4.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ4- pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ3*LF+ 3./2.*pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2*LF2- pow(gamma1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF3+ 2.*EC1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LF2+ 2.*EC1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ2 - 4.*EC1qg[idx]*gamma1gq[idx]*gamma2qqbp[idx]*LQ*LF+ EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LF3- EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ3+ 3.*EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ2*LF- 3.*EC1qg[idx]*gamma1qg[idx]*pow(gamma1gq[idx],2)*LQ*LF2+ pow(EC1qg[idx],2)*pow(gamma1gq[idx],2)*LF2+ pow(EC1qg[idx],2)*pow(gamma1gq[idx],2)*LQ2- 2.*pow(EC1qg[idx],2)*pow(gamma1gq[idx],2)*LQ*LF+ 2.*EC2qqbp[idx]*gamma2qqbp[idx]*LF- 2.*EC2qqbp[idx]*gamma2qqbp[idx]*LQ+ EC2qqbp[idx]*gamma1qg[idx]*gamma1gq[idx]*LF2+ EC2qqbp[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ2- 2.*EC2qqbp[idx]*gamma1qg[idx]*gamma1gq[idx]*LQ*LF+ 2.*EC2qqbp[idx]*EC1qg[idx]*gamma1gq[idx]*LF- 2.*EC2qqbp[idx]*EC1qg[idx]*gamma1gq[idx]*LQ ;

	complex <double> H4st_qqb   = h4st_qqb   (idx);
	complex <double> H4st_qg    = h4st_qg    (idx);
	complex <double> H4st_qq    = h4st_qq    (idx);
	complex <double> H4st_qqp   = h4st_qqp   (idx);
	complex <double> H4st_qqbp  = h4st_qqbp  (idx);
	complex <double> H4st_gg    = h4st_gg    (idx);
	complex <double> H4st_qbg   = h4st_qbg   (idx);
	complex <double> H4st_qpg   = h4st_qpg   (idx);
	complex <double> H4st_qbpg  = h4st_qbpg  (idx);
	complex <double> H4st_qbq   = h4st_qbq   (idx);
	complex <double> H4st_qbqp  = h4st_qbqp  (idx);
	complex <double> H4st_qbqbp = h4st_qbqbp (idx);
	complex <double> H4st_qpqp  = h4st_qpqp  (idx);
	complex <double> H4st_qpqbp = h4st_qpqbp (idx);
	complex <double> H4st_qbpqp = h4st_qbpqp (idx);
		
	//Mixed LR*LQF variations
	H4st_qqb  += -3*beta0*H3st_qqb*LR  -(2*beta1*LR + 3*pow(beta0,2)*LR2)*H2st_qqb - (beta2*LR + 5./2.*beta1*beta0*LR2 + pow(beta0,3)*LR3)*H1st_qqb;
	H4st_qg   += -3*beta0*H3st_qg*LR   -(2*beta1*LR + 3*pow(beta0,2)*LR2)*H2st_qg  - (beta2*LR + 5./2.*beta1*beta0*LR2 + pow(beta0,3)*LR3)*H1st_qg;
	H4st_qq   += -3*beta0*H3st_qq*LR   -(2*beta1*LR + 3*pow(beta0,2)*LR2)*H2st_qq;
	H4st_qqp  += -3*beta0*H3st_qqp*LR  -(2*beta1*LR + 3*pow(beta0,2)*LR2)*H2st_qqp;
	H4st_qqbp += -3*beta0*H3st_qqbp*LR -(2*beta1*LR + 3*pow(beta0,2)*LR2)*H2st_qqbp;
	H4st_gg   += -3*beta0*H3st_gg*LR   -(2*beta1*LR + 3*pow(beta0,2)*LR2)*H2st_gg;
	H4st_qbg  += -3*beta0*H3st_qbg*LR;
	H4st_qpg  += -3*beta0*H3st_qpg*LR;
	H4st_qbpg += -3*beta0*H3st_qbpg*LR;
	  
	qqb[idx]  += as4*H4st_qqb;
	qg[idx]   += as4*H4st_qg;
	qq[idx]   += as4*H4st_qq;
	qqp[idx]  += as4*H4st_qqp;
	qqbp[idx] += as4*H4st_qqbp;
	gg[idx]   += as4*H4st_gg;
	qbg[idx]  += as4*H4st_qbg;
	qpg[idx]  += as4*H4st_qpg;
	qbpg[idx] += as4*H4st_qbpg;
	qbq[idx]  += as4*H4st_qbq  ;
	qpqbp[idx]+= as4*H4st_qpqbp;
	qbpqp[idx]+= as4*H4st_qbpqp;
	qbqp[idx] += as4*H4st_qbqp ;
	qbqbp[idx]+= as4*H4st_qbqbp;
	qpqp[idx] += as4*H4st_qpqp ;
      }
  for (int sign = mesq::positive; sign <= mesq::negative; sign++)
    for (int i = 0; i < mellinint::mdim; i++)
      {
	int idx = pmom::index(i,sign);
	qqb[idx]  *= expc::qqb[idx];
	qg[idx]   *= expc::qqb[idx];
	qq[idx]   *= expc::qqb[idx];
	qqp[idx]  *= expc::qqb[idx];
	qqbp[idx] *= expc::qqb[idx];
	gg[idx]   *= expc::qqb[idx];
	qbg[idx]  *= expc::qqb[idx];
	qpg[idx]  *= expc::qqb[idx];
	qbpg[idx] *= expc::qqb[idx];
	qbq[idx]  *= expc::qqb[idx];
	qpqbp[idx]*= expc::qqb[idx];
	qbpqp[idx]*= expc::qqb[idx];
	qbqp[idx] *= expc::qqb[idx];
	qbqbp[idx]*= expc::qqb[idx];
	qpqp[idx] *= expc::qqb[idx];
      }      
}
